@@ -2,14 +2,11 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """Dialog classes for the Builder Code component
 """
-
-from __future__ import absolute_import, division, print_function
-from builtins import str
 import sys
 
 import keyword
@@ -17,14 +14,10 @@ import wx
 # import wx.lib.agw.aui as aui
 from collections import OrderedDict
 from psychopy.experiment.components.code import CodeComponent
+from ..validators import WarningManager
 from ...themes import ThemeMixin
 
-from psychopy.constants import PY3
-
-if PY3:
-    from importlib.util import find_spec as loader
-else:
-    from pkgutil import find_loader as loader
+from importlib.util import find_spec as loader
 hasMetapensiero = loader("metapensiero") is not None
 
 from .. import validators
@@ -37,33 +30,42 @@ class DlgCodeComponentProperties(wx.Dialog):
     _style = (wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER
               | wx.DIALOG_NO_PARENT)
 
-    def __init__(self, frame, title, params, order,
-                 helpUrl=None, suppressTitles=True, size=(800,400),
+    def __init__(self, frame, element, experiment,
+                 helpUrl=None, suppressTitles=True, size=(1000,600),
                  style=_style, editing=False, depends=[],
-                 timeout=None):
+                 timeout=None, type="Code"):
 
         # translate title
-        localizedTitle = title.replace(' Properties',
-                                       _translate(' Properties'))
-        wx.Dialog.__init__(self, None, -1, localizedTitle,
-                           size=size, style=self._style)
-        self.SetTitle(localizedTitle)  # use localized title
+        if "name" in element.params:
+            title = element.params['name'].val + _translate(' Properties')
+        elif "expName" in element.params:
+            title = element.params['expName'].val + _translate(' Properties')
+        else:
+            title = "Properties"
+        # get help url
+        if hasattr(element, 'url'):
+            helpUrl = element.url
+        else:
+            helpUrl = None
+
+        wx.Dialog.__init__(self, None, -1, title,
+                           size=size, style=style)
+        self.SetTitle(title)  # use localized title
         # self.panel = wx.Panel(self)
         self.frame = frame
         self.app = frame.app
         self.helpUrl = helpUrl
-        self.params = params  # dict
-        self.order = order
+        self.params = element.params  # dict
+        self.order = element.order
         self.title = title
         self.timeout = timeout
-        self.warningsDict = {}  # to store warnings for all fields
         self.codeBoxes = {}
         self.tabs = OrderedDict()
 
         if not editing and 'name' in self.params:
             # then we're adding a new component so ensure a valid name:
             makeValid = self.frame.exp.namespace.makeValid
-            self.params['name'].val = makeValid(params['name'].val)
+            self.params['name'].val = makeValid(self.params['name'].val)
 
         self.codeNotebook = wx.Notebook(self)
         # in AUI notebook the labels are blurry on retina mac
@@ -103,6 +105,10 @@ class DlgCodeComponentProperties(wx.Dialog):
                 self.codeTypeMenu.Bind(wx.EVT_CHOICE, self.onCodeChoice)
                 self.codeTypeName = wx.StaticText(self, wx.ID_ANY,
                                                   _translate(param.label))
+            elif paramName == 'disabled':
+                # Create bool control to disable/enable component
+                self.disableCtrl = wx.CheckBox(self, wx.ID_ANY, label=_translate('disabled'))
+                self.disableCtrl.SetValue(bool(param.val))
             else:
                 codeType = ["Py", "JS"]["JS" in paramName]  # Give CodeBox a code type
                 tabName = paramName.replace("JS ", "")
@@ -118,7 +124,7 @@ class DlgCodeComponentProperties(wx.Dialog):
                                                     pos=wx.DefaultPosition,
                                                     style=0,
                                                     prefs=self.app.prefs,
-                                                    params=params,
+                                                    params=self.params,
                                                     codeType=codeType)
                 self.codeBoxes[paramName].AddText(param.val)
                 self.codeBoxes[paramName].Bind(wx.EVT_KEY_UP, self.onKeyUp)  # For real time translation
@@ -136,6 +142,7 @@ class DlgCodeComponentProperties(wx.Dialog):
         self.okButton.SetDefault()
         self.cancelButton = wx.Button(self, wx.ID_CANCEL,
                                       _translate(" Cancel "))
+        self.warnings = WarningManager(self)  # to store warnings for all fields
         self.__do_layout()
         if openToPage is None:
             openToPage = 1
@@ -224,9 +231,9 @@ class DlgCodeComponentProperties(wx.Dialog):
         # If user doesn't have metapensiero and current choice is auto-js...
         if not hasMetapensiero and param.val.lower() == "auto->js" :
             # Throw up error dlg instructing to get metapensiero
-            msg = ("\nPy to JS auto-translation requires the metapensiero library.\n"
+            msg = _translate("\nPy to JS auto-translation requires the metapensiero library.\n"
                    "Available for Python 3.5+.\n")
-            dlg = CodeOverwriteDialog(self, -1, "Warning: requires the metapensiero library", msg)
+            dlg = CodeOverwriteDialog(self, -1, _translate("Warning: requires the metapensiero library"), msg)
             dlg.ShowModal()
             # Revert to previous choice
             self.undoCodeTypeChoice(prevCodeType)
@@ -258,9 +265,9 @@ class DlgCodeComponentProperties(wx.Dialog):
         # If code type has changed and previous code type isn't auto-js...
         if prevCodeType.lower() != 'auto->js' and self.codeChangeDetected():
             # Throw up a warning dlg to alert user of overwriting
-            msg = ("\nAuto-JS translation will overwrite your existing JavaScript code.\n"
+            msg = _translate("\nAuto-JS translation will overwrite your existing JavaScript code.\n"
                    "Press OK to continue, or Cancel.\n")
-            dlg = CodeOverwriteDialog(self, -1, "Warning: Python to JavaScript Translation", msg)
+            dlg = CodeOverwriteDialog(self, -1, _translate("Warning: Python to JavaScript Translation"), msg)
             retVal = dlg.ShowModal()
             # When window closes, if OK was not clicked revert to previous codetype
             if not retVal == wx.ID_OK:
@@ -270,7 +277,7 @@ class DlgCodeComponentProperties(wx.Dialog):
         for boxName in self.codeBoxes:
             # If it is not JS...
             if 'JS' not in boxName:
-                # Traslate to JS
+                # Translate to JS
                 self.runTranslation(boxName)
 
         if event:
@@ -406,6 +413,7 @@ class DlgCodeComponentProperties(wx.Dialog):
         nameSizer.Add(self.codeTypeName,
                       flag=wx.TOP | wx.RIGHT, border=13, proportion=0)
         nameSizer.Add(self.codeTypeMenu, 0, wx.ALIGN_CENTER_VERTICAL, 0)
+        nameSizer.Add(self.disableCtrl, flag=wx.ALL | wx.ALIGN_CENTER_VERTICAL, border=13)
 
         mainSizer = wx.BoxSizer(wx.VERTICAL)
         buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
@@ -415,18 +423,17 @@ class DlgCodeComponentProperties(wx.Dialog):
         buttonSizer.Add(self.helpButton, 0,
                         wx.ALL | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, 10)
         buttonSizer.AddStretchSpacer()
-        if sys.platform == 'darwin':
-            buttonSizer.Add(self.cancelButton, 0,
-                            wx.ALL | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL,
-                            border=3)
-            buttonSizer.Add(self.okButton, 0,
-                            wx.ALL | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL,
-                            border=3)
+        # Add Okay and Cancel buttons
+        if sys.platform == "win32":
+            btns = [self.okButton, self.cancelButton]
         else:
-            buttonSizer.Add(self.okButton, 0,
-                            wx.ALL | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, border=3)
-            buttonSizer.Add(self.cancelButton, 0,
-                            wx.ALL | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL, border=3)
+            btns = [self.cancelButton, self.okButton]
+        buttonSizer.Add(btns[0], 0,
+                        wx.ALL | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL,
+                        border=3)
+        buttonSizer.Add(btns[1], 0,
+                        wx.ALL | wx.RIGHT | wx.ALIGN_CENTER_VERTICAL,
+                        border=3)
 
         mainSizer.Add(buttonSizer, 0, wx.ALL | wx.RIGHT | wx.EXPAND, 5)
         self.SetSizer(mainSizer)
@@ -448,7 +455,7 @@ class DlgCodeComponentProperties(wx.Dialog):
             elif fieldName == 'Code Type':
                 param.val = self.codeTypeMenu.GetStringSelection()
             elif fieldName == 'disabled':
-                pass
+                param.val = self.disableCtrl.GetValue()
             else:
                 codeBox = self.codeBoxes[fieldName]
                 param.val = codeBox.GetText()
