@@ -4,11 +4,9 @@
 """Classes and functions for the coder file browser pane."""
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
-
 import wx
 from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin
 
@@ -47,7 +45,7 @@ def convertBytes(nbytes):
         return '{:.1f} B'.format(nbytes)
 
 
-class FolderItemData(object):
+class FolderItemData:
     """Class representing a folder item in the file browser."""
     __slots__ = ['name', 'abspath', 'basename']
     def __init__(self, name, abspath, basename):
@@ -56,7 +54,7 @@ class FolderItemData(object):
         self.basename = basename
 
 
-class FileItemData(object):
+class FileItemData:
     """Class representing a file item in the file browser."""
     __slots__ = ['name', 'abspath', 'basename', 'fsize', 'mod']
     def __init__(self, name, abspath, basename, fsize, mod):
@@ -104,7 +102,8 @@ class FileBrowserPanel(wx.Panel):
             ".tif": 'fileimage16.png',
             ".ppm": 'fileimage16.png',
             ".gif": 'fileimage16.png',
-            ".py": 'coderpython16.png'
+            ".py": 'coderpython16.png',
+            ".js": 'coderjs16.png'
         }
 
     def __init__(self, parent, frame):
@@ -682,4 +681,3 @@ class FileBrowserPanel(wx.Panel):
         self.currentPath = path
         self.txtAddr.SetValue(self.currentPath)
         self.updateFileBrowser()
-
