@@ -2,17 +2,12 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 # python text parser
 # this is really just for the purpose of constructing code analysis in
 # python scripts
-
-from __future__ import absolute_import, print_function
-
-from builtins import next
-from builtins import object
 import tokenize
 import re
 
@@ -21,7 +16,7 @@ import re
 # print(xx['makeRadialMatrix'].__doc__)
 
 
-class tokenBuffer(object):
+class tokenBuffer():
     # simple buffer to provide mechanism to step backwards through previous
     # tokens
 
@@ -117,7 +112,7 @@ def parsePyScript(src, indentSpaces=4):
             # slice off comment
             lineText = lineText.split('#')[0]
             lineTokens = [
-                tok.strip() for tok in re.split(' |\(|\)', lineText) if tok]
+                tok.strip() for tok in re.split(r' |\(|\)', lineText) if tok]
             defType, defName = lineTokens[:2]
             foundDefs.append((defType, defName, lineIndent, lineno))
 
