@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import requests
@@ -10,7 +10,7 @@ from psychopy import logging, prefs
 import wx
 from datetime import datetime
 
-newsURL = "http://news.psychopy.org/"
+newsURL = "https://news.psychopy.org/"
 
 CRITICAL = 40
 ANNOUNCE = 30
