@@ -2,8 +2,9 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
+import sys
 
 import wx
 import wx.html2
@@ -100,6 +101,11 @@ class PavloviaMiniBrowser(wx.Dialog):
             url = self.user.attributes['web_url']
             self.browser.LoadURL(url)
 
+    def editUserPage(self):
+        url = "https://gitlab.pavlovia.org/profile"
+        self.browser.LoadURL(url)
+        self.SetSizeWH(1240, 840)
+
     def gotoProjects(self):
         self.browser.LoadURL("https://pavlovia.org/projects.html")
 
@@ -189,8 +195,11 @@ class PavloviaCommitDialog(wx.Dialog):
                              (self.commitDescrLbl, 0, wx.ALIGN_RIGHT),
                              self.commitDescrCtrl])
         buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
-        buttonSizer.AddMany([self.btnCancel,
-                             self.btnOK])
+        if sys.platform == "win32":
+            btns = [self.btnOK, self.btnCancel]
+        else:
+            btns = [self.btnCancel, self.btnOK]
+        buttonSizer.AddMany(btns)
 
         # main sizer and layout
         mainSizer = wx.BoxSizer(wx.VERTICAL)
