@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import wx
@@ -27,7 +27,7 @@ class PavloviaMenu(wx.Menu):
 
     def __init__(self, parent):
         wx.Menu.__init__(self)
-        self.parent = parent  # type: BuilderFrame
+        self.parent = parent  # is a BuilderFrame
         PavloviaMenu.app = parent.app
         keys = self.app.keys
         # from prefs fetch info about prev usernames and projects
@@ -35,6 +35,7 @@ class PavloviaMenu(wx.Menu):
 
         # item = self.Append(wx.ID_ANY, _translate("Tell me more..."))
         # parent.Bind(wx.EVT_MENU, self.onAbout, id=item.GetId())
+        # parent.Bind(wx.EVT_MENU, self.onAbout, id=item.GetId())
 
         PavloviaMenu.knownUsers = pavlovia.knownUsers
 
@@ -46,8 +47,10 @@ class PavloviaMenu(wx.Menu):
             lastPavUser = None
         # if lastPavUser and not PavloviaMenu.currentUser:
         #     self.setUser(PavloviaMenu.appData['pavloviaUser'])
-        for name in self.knownUsers:
-            self.addToSubMenu(name, self.userMenu, self.onSetUser)
+        if self.knownUsers is not None:
+            for name in self.knownUsers:
+                self.addToSubMenu(name, self.userMenu, self.onSetUser)
+
         self.userMenu.AppendSeparator()
         self.loginBtn = self.userMenu.Append(wx.ID_ANY,
                                     _translate("Log in to Pavlovia...\t{}")
@@ -82,9 +85,9 @@ class PavloviaMenu(wx.Menu):
         self.setUser(user)
 
     def setUser(self, user=None):
-
-        if user is None and PavloviaMenu.appData['pavloviaUser']:
-            user = PavloviaMenu.appData['pavloviaUser']
+        if PavloviaMenu.appData:
+            if user is None and PavloviaMenu.appData['pavloviaUser']:
+                user = PavloviaMenu.appData['pavloviaUser']
 
         if user in [PavloviaMenu.currentUser, None]:
             return  # nothing to do here. Move along please.
