@@ -4,12 +4,9 @@
 """Basic functions, including timing, rush (imported), quit
 """
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, division, print_function
-
-from builtins import object
 import sys
 import threading
 import subprocess
@@ -25,7 +22,7 @@ from psychopy.clock import (MonotonicClock, Clock, CountdownTimer,
 # particular OS
 from psychopy.platform_specific import rush  # pylint: disable=W0611
 from psychopy import logging
-from psychopy.constants import STARTED, NOT_STARTED, FINISHED, PY3
+from psychopy.constants import STARTED, NOT_STARTED, FINISHED
 
 try:
     import pyglet
@@ -75,6 +72,12 @@ def quit():
     # pygame.quit()  # safe even if pygame was never initialised
     logging.flush()
 
+    # properly shutdown ioHub server
+    from psychopy.iohub.client import ioHubConnection
+
+    if ioHubConnection.ACTIVE_CONNECTION:
+        ioHubConnection.ACTIVE_CONNECTION.quit()
+
     for thisThread in threading.enumerate():
         if hasattr(thisThread, 'stop') and hasattr(thisThread, 'running'):
             # this is one of our event threads - kill it and wait for success
@@ -144,20 +147,15 @@ def shellCall(shellCmd, stdin='', stderr=False, env=None, encoding=None):
         else:
             cmdObjects.append(obj.decode('utf-8'))
 
-    # Since Python 3.6, we can use the `encoding` parameter.
-    if PY3:
-        if sys.version_info.minor >= 6:
-            proc = subprocess.Popen(cmdObjects, stdin=subprocess.PIPE,
-                                    stdout=subprocess.PIPE,
-                                    stderr=subprocess.PIPE,
-                                    encoding=encoding, env=env)
-        else:
-            msg = 'shellCall() requires Python 2.7, or 3.6 and newer.'
-            raise RuntimeError(msg)
-    else:
+    # the `encoding` parameter results in unicode coming back
+    if sys.version_info.minor >= 6:
         proc = subprocess.Popen(cmdObjects, stdin=subprocess.PIPE,
                                 stdout=subprocess.PIPE,
-                                stderr=subprocess.PIPE, env=env)
+                                stderr=subprocess.PIPE,
+                                encoding=encoding, env=env)
+    else:
+        msg = 'shellCall() requires Python 2.7, or 3.6 and newer.'
+        raise RuntimeError(msg)
 
     stdoutData, stderrData = proc.communicate(stdin)
 
@@ -166,3 +164,12 @@ def shellCall(shellCmd, stdin='', stderr=False, env=None, encoding=None):
         return stdoutData.strip(), stderrData.strip()
     else:
         return stdoutData.strip()
+
+
+class ComponentPlaceholder:
+    """
+    When a component is not implemented, we need an object to represent it when running, which will accept any
+    attribute given without causing an error and breaking the experiment. This object extends the base Python
+    `object` class, with no additions.
+    """
+    pass
