@@ -1,13 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, print_function
-
-# from future import standard_library
-# standard_library.install_aliases()
-from builtins import str
-from past.builtins import basestring
-from builtins import object
 import weakref
 import pickle
 import os
@@ -43,7 +36,7 @@ except ImportError:
 _experiments = weakref.WeakValueDictionary()
 
 
-class _ComparisonMixin(object):
+class _ComparisonMixin():
     def __eq__(self, other):
         # NoneType and booleans, for example, don't have a .__dict__ attribute.
         try:
@@ -295,12 +288,15 @@ class _BaseTrialHandler(_ComparisonMixin):
 
             appendFile: True or False
                 If False any existing file with this name will be
-                overwritten. If True then a new worksheet will be appended.
+                kept and a new file will be created with a slightly different
+                name. If you want to overwrite the old file, pass 'overwrite'
+                to ``fileCollisionMethod``.
+                If True then a new worksheet will be appended.
                 If a worksheet already exists with that name a number will
                 be added to make it unique.
 
             fileCollisionMethod: string
-                Collision method passed to
+                Collision method (``rename``,``overwrite``, ``fail``) passed to
                 :func:`~psychopy.tools.fileerrortools.handleFileCollision`
                 This is ignored if ``append`` is ``True``.
 
@@ -337,7 +333,9 @@ class _BaseTrialHandler(_ComparisonMixin):
             newWorkbook = False
         else:
             if not appendFile:
-                # the file exists but we're not appending, will be overwritten
+                # the file exists but we're not appending, a new file will
+                # be saved with a slightly different name, unless 
+                # fileCollisionMethod = ``overwrite``
                 fileName = handleFileCollision(fileName,
                                                fileCollisionMethod)
             wb = Workbook()  # create new workbook
@@ -508,7 +506,7 @@ class DataHandler(_ComparisonMixin, dict):
         """
         if not shape:
             shape = self.dataShape
-        if not isinstance(names, basestring):
+        if not isinstance(names, str):
             # recursively call this function until we have a string
             for thisName in names:
                 self.addDataType(thisName)
