@@ -1,22 +1,11 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
-from __future__ import absolute_import, division, print_function
-
-# from future import standard_library
-# standard_library.install_aliases()
-from builtins import zip
-from builtins import next
-from past.builtins import basestring
-from past.builtins import str
-from builtins import range
-import string
 import sys
 import os
 import pickle
 import copy
 import warnings
-import collections
 import numpy as np
 from pkg_resources import parse_version
 
@@ -29,6 +18,11 @@ from psychopy.contrib.psi import PsiObject
 from .base import _BaseTrialHandler, _ComparisonMixin
 from .utils import _getExcelCellName
 
+try:
+    from collections.abc import Iterable
+except ImportError:
+    from collections import Iterable
+
 try:
     # import openpyxl
     import openpyxl
@@ -210,7 +204,7 @@ class StairHandler(_BaseTrialHandler):
 
     def addResponse(self, result, intensity=None):
         """Add a 1 or 0 to signify a correct / detected or
-        incorrect / missed trial
+        incorrect / missed trial.
 
         This is essential to advance the staircase to a new intensity level!
 
@@ -252,7 +246,7 @@ class StairHandler(_BaseTrialHandler):
         """
         if not dataName in self.otherData:  # init the list
             if self.thisTrialN > 0:
-                # might have run trals already
+                # might have run trials already
                 self.otherData[dataName] = [None] * (self.thisTrialN - 1)
             else:
                 self.otherData[dataName] = []
@@ -266,8 +260,8 @@ class StairHandler(_BaseTrialHandler):
         """Deprecated since 1.79.00: This function name was ambiguous.
         Please use one of these instead:
 
-            .addResponse(result, intensity)
-            .addOtherData('dataName', value')
+        *   .addResponse(result, intensity)
+        *   .addOtherData('dataName', value')
 
         """
         self.addResponse(result, intensity)
@@ -466,42 +460,42 @@ class StairHandler(_BaseTrialHandler):
 
         # write the data
         reversalStr = str(self.reversalIntensities)
-        reversalStr = string.replace(reversalStr, ',', delim)
-        reversalStr = string.replace(reversalStr, '[', '')
-        reversalStr = string.replace(reversalStr, ']', '')
+        reversalStr = reversalStr.replace(',', delim)
+        reversalStr = reversalStr.replace('[', '')
+        reversalStr = reversalStr.replace(']', '')
         f.write('\nreversalIntensities=\t%s\n' % reversalStr)
 
         reversalPts = str(self.reversalPoints)
-        reversalPts = string.replace(reversalPts, ',', delim)
-        reversalPts = string.replace(reversalPts, '[', '')
-        reversalPts = string.replace(reversalPts, ']', '')
+        reversalPts = reversalPts.replace(',', delim)
+        reversalPts = reversalPts.replace('[', '')
+        reversalPts = reversalPts.replace(']', '')
         f.write('reversalIndices=\t%s\n' % reversalPts)
 
         rawIntens = str(self.intensities)
-        rawIntens = string.replace(rawIntens, ',', delim)
-        rawIntens = string.replace(rawIntens, '[', '')
-        rawIntens = string.replace(rawIntens, ']', '')
+        rawIntens = rawIntens.replace(',', delim)
+        rawIntens = rawIntens.replace('[', '')
+        rawIntens = rawIntens.replace(']', '')
         f.write('\nintensities=\t%s\n' % rawIntens)
 
         responses = str(self.data)
-        responses = string.replace(responses, ',', delim)
-        responses = string.replace(responses, '[', '')
-        responses = string.replace(responses, ']', '')
+        responses = responses.replace(',', delim)
+        responses = responses.replace('[', '')
+        responses = responses.replace(']', '')
         f.write('responses=\t%s\n' % responses)
 
         # add self.extraInfo
         if self.extraInfo is not None and not matrixOnly:
             strInfo = str(self.extraInfo)
             # dict begins and ends with {} - remove
-            # string.replace(strInfo, '{','')
-            # strInfo = string.replace(strInfo, '}','')
+            # strInfo.replace('{','')
+            # strInfo = strInfo.replace('}','')
             strInfo = strInfo[1:-1]
             # separate value from keyname
-            strInfo = string.replace(strInfo, ': ', ':\n')
+            strInfo = strInfo.replace(strInfo, ': ', ':\n')
             # separate values from each other
-            strInfo = string.replace(strInfo, ',', '\n')
-            strInfo = string.replace(strInfo, 'array([ ', '')
-            strInfo = string.replace(strInfo, '])', '')
+            strInfo = strInfo.replace(',', '\n')
+            strInfo = strInfo.replace('array([ ', '')
+            strInfo = strInfo.replace('])', '')
 
             f.write('\n%s\n' % strInfo)
 
@@ -538,7 +532,7 @@ class StairHandler(_BaseTrialHandler):
 
             fileName: string
                 the name of the file to create or append. Can include
-                relative or absolute path
+                relative or absolute path.
 
             sheetName: string
                 the name of the worksheet within the file
@@ -556,7 +550,7 @@ class StairHandler(_BaseTrialHandler):
             fileCollisionMethod: string
                 Collision method passed to
                 :func:`~psychopy.tools.fileerrortools.handleFileCollision`
-                This is ignored if ``append`` is ``True``.
+                This is ignored if ``appendFile`` is ``True``.
 
         """
 
@@ -631,7 +625,7 @@ class StairHandler(_BaseTrialHandler):
 
         # add self.extraInfo
         if self.extraInfo is not None and not matrixOnly:
-            ws.cell(column=startingCol, row=1,
+            ws.cell(column=col, row=1,
                     value='extraInfo')
             rowN = 2
             for key, val in list(self.extraInfo.items()):
@@ -650,7 +644,7 @@ class StairHandler(_BaseTrialHandler):
     def saveAsPickle(self, fileName, fileCollisionMethod='rename'):
         """Basically just saves a copy of self (with data) to a pickle file.
 
-        This can be reloaded if necess and further analyses carried out.
+        This can be reloaded if necessary and further analyses carried out.
 
         :Parameters:
 
@@ -683,7 +677,7 @@ class QuestObject_(QuestObject, _ComparisonMixin):
 
 
 class QuestHandler(StairHandler):
-    """Class that implements the Quest algorithm for quick measurement of
+    r"""Class that implements the Quest algorithm for quick measurement of
     psychophysical thresholds.
 
     Uses Andrew Straw's `QUEST <http://www.visionegg.org/Quest>`_, which is a
@@ -694,7 +688,7 @@ class QuestHandler(StairHandler):
 
     The Weibull psychometric function is given by the formula
     
-    :math:`\Psi(x) = \delta \gamma + (1 - \delta) [1 - (1 - \gamma)\, \exp(-10^{\beta (x - T + \epsilon)})]`
+    :math:`\Psi(x) = \delta \gamma + (1 - \delta) [1 - (1 - \gamma)\, \exp(-10^{\\beta (x - T + \epsilon)})]`
 
     Here, :math:`x` is an intensity or a contrast (in log10 units), and :math:`T` is estimated threshold.
     
@@ -861,6 +855,7 @@ class QuestHandler(StairHandler):
 
         self.startVal = startVal
         self.startValSd = startValSd
+        self.pThreshold = pThreshold
         self.stopInterval = stopInterval
         self._questNextIntensity = startVal
         self._range = range
@@ -965,10 +960,11 @@ class QuestHandler(StairHandler):
         if self.method == 'mean':
             self._questNextIntensity = self._quest.mean()
         elif self.method == 'mode':
-            self._questNextIntensity = self._quest.mode()
+            self._questNextIntensity = self._quest.mode()[0]
         elif self.method == 'quantile':
             self._questNextIntensity = self._quest.quantile()
-        # else: maybe raise an error
+        else:
+            raise TypeError(f"Requested method for QUEST: {self.method} is not a valid method. Please use mean, mode or quantile")
         self._nextIntensity = self._questNextIntensity
 
     def mean(self):
@@ -1273,7 +1269,7 @@ class PsiHandler(StairHandler):
     next = __next__  # allows user to call without a loop `val = trials.next()`
 
     def _checkFinished(self):
-        """checks if we are finished
+        """checks if we are finished.
         Updates attribute: `finished`
         """
         if self.nTrials is not None and len(self.intensities) >= self.nTrials:
@@ -1345,10 +1341,9 @@ class QuestPlusHandler(StairHandler):
 
         The parameter estimates can be retrieved via the `.paramEstimate`
         attribute, which returns a dictionary whose keys correspond to the
-        names of the estimated parameters
-        (i.e., `QuestPlusHandler.paramEstimate['threshold']` will provide the
-         threshold estimate). Retrieval of the marginal posterior distributions
-         works similarly: they can be accessed via the `.posterior` dictionary.
+        names of the estimated parameters (i.e., `QuestPlusHandler.paramEstimate['threshold']`
+        will provide the threshold estimate). Retrieval of the marginal posterior distributions works
+        similarly: they can be accessed via the `.posterior` dictionary.
 
         Parameters
         ----------
@@ -1425,7 +1420,7 @@ class QuestPlusHandler(StairHandler):
             select a stimulus for presentation in the next trial.
             `maxConsecutiveReps` defines how many times the exact same stimulus
             can be presented on consecutive trials.
-            For exmaple, to randomly pick a stimulus from those which will
+            For example, to randomly pick a stimulus from those which will
             produce the 4 smallest expected entropies, and to allow the same
             stimulus to be presented on two consecutive trials max, use
             `stimSelectionOptions=dict(N=4, maxConsecutiveReps=2)`.
@@ -1735,14 +1730,14 @@ class MultiStairHandler(_BaseTrialHandler):
         :class:`StairHandler` or
         :class:`QuestHandler` (e.g. the `startVal`, `minVal`, `maxVal`...)
         should be specified in the `conditions` list and may vary between
-        each staircase. In particular, the conditions /must/ include the
+        each staircase. In particular, the conditions **must** include
         a `startVal` (because this is a required argument to the above
-        handlers) a `label` to tag the staircase and a `startValSd`
+        handlers), a `label` to tag the staircase and a `startValSd`
         (only for QUEST staircases). Any parameters not specified in the
         conditions file will revert to the default for that individual
         handler.
 
-        If you need to custom the behaviour further you may want to
+        If you need to customize the behaviour further you may want to
         look at the recipe on :ref:`interleavedStairs`.
 
         :params:
@@ -1761,7 +1756,7 @@ class MultiStairHandler(_BaseTrialHandler):
                 If `sequential`, don't perform any randomization.
 
             conditions: a list of dictionaries specifying conditions
-                Can be used to control parameters for the different staicases.
+                Can be used to control parameters for the different staircases.
                 Can be imported from an Excel file using
                 `psychopy.data.importConditions`
                 MUST include keys providing, 'startVal', 'label' and
@@ -1804,7 +1799,7 @@ class MultiStairHandler(_BaseTrialHandler):
         Raises
         ------
             ValueError
-                If an unknown randomiation option was passed via the `method`
+                If an unknown randomization option was passed via the `method`
                 keyword argument.
 
         """
@@ -1836,7 +1831,7 @@ class MultiStairHandler(_BaseTrialHandler):
 
     def _checkArguments(self):
         # Did we get a `conditions` parameter, correctly formatted?
-        if not isinstance(self.conditions, collections.Iterable):
+        if not isinstance(self.conditions, Iterable):
             raise TypeError(
                 '`conditions` parameter passed to MultiStairHandler '
                 'should be a list, not a %s.' % type(self.conditions))
@@ -2010,14 +2005,14 @@ class MultiStairHandler(_BaseTrialHandler):
         the response (0 or 1) or some other data concerning the trial so
         there is now a pair of explicit methods:
 
-            addResponse(corr,intensity) #some data that alters the next
+        *   addResponse(corr,intensity) #some data that alters the next
                 trial value
-            addOtherData('RT', reactionTime) #some other data that won't
+        *   addOtherData('RT', reactionTime) #some other data that won't
                 control staircase
 
         """
         self.addResponse(result, intensity)
-        if isinstance(result, basestring):
+        if isinstance(result, str):
             raise TypeError("MultiStairHandler.addData should only receive "
                             "corr / incorr. Use .addOtherData('datName',val)")
 
@@ -2054,7 +2049,7 @@ class MultiStairHandler(_BaseTrialHandler):
         """Save a summary data file in Excel OpenXML format workbook
         (:term:`xlsx`) for processing in most spreadsheet packages.
         This format is compatible with versions of Excel (2007 or greater)
-        and and with OpenOffice (>=3.0).
+        and with OpenOffice (>=3.0).
 
         It has the advantage over the simpler text files (see
         :func:`TrialHandler.saveAsText()` )
@@ -2125,7 +2120,7 @@ class MultiStairHandler(_BaseTrialHandler):
                 `.tsv` will be added if not included.
 
             delim: a string
-                the delimitter to be used (e.g. '\t' for tab-delimitted,
+                the delimiter to be used (e.g. '\t' for tab-delimited,
                 ',' for csv files)
 
             matrixOnly: True/False
