@@ -8,11 +8,6 @@ and disable python garbage collection to see if either influences
 the precision of your frame flips.
 """
 
-from __future__ import division
-from __future__ import print_function
-
-from builtins import str
-from builtins import range
 import gc, numpy
 from psychopy import visual, logging, core, event
 
