@@ -2,36 +2,35 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
-
-from os import path
+from pathlib import Path
 from psychopy.experiment.components import BaseVisualComponent, Param, getInitVals
-from psychopy.localization import _translate
-
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'image.png')
-tooltip = _translate('Image: present images (bmp, jpg, tif...)')
+from psychopy.localization import _translate, _localized as __localized
+_localized = __localized.copy()
 
 # only use _localized values for label values, nothing functional:
-_localized = {'image': _translate('Image'),
-              'mask': _translate('Mask'),
-              'texture resolution': _translate('Texture resolution'),
-              'flipVert': _translate('Flip vertically'),
-              'flipHoriz': _translate('Flip horizontally'),
-              'interpolate': _translate('Interpolate')}
+_localized.update({'image': _translate('Image'),
+                   'mask': _translate('Mask'),
+                   'texture resolution': _translate('Texture resolution'),
+                   'flipVert': _translate('Flip vertically'),
+                   'flipHoriz': _translate('Flip horizontally'),
+                   'interpolate': _translate('Interpolate')})
 
 
 class ImageComponent(BaseVisualComponent):
     """An event class for presenting image-based stimuli"""
 
-    def __init__(self, exp, parentName, name='image', image='None', mask='None',
+    categories = ['Stimuli']
+    targets = ['PsychoPy', 'PsychoJS']
+    iconFile = Path(__file__).parent / 'image.png'
+    tooltip = _translate('Image: present images (bmp, jpg, tif...)')
+
+    def __init__(self, exp, parentName, name='image', image='', mask='',
                  interpolate='linear', units='from exp settings',
                  color='$[1,1,1]', colorSpace='rgb', pos=(0, 0),
-                 size=(0.5, 0.5), ori=0, texRes='128', flipVert=False,
+                 size=(0.5, 0.5), anchor="center", ori=0, texRes='128', flipVert=False,
                  flipHoriz=False,
                  startType='time (s)', startVal=0.0,
                  stopType='duration (s)', stopVal=1.0,
@@ -44,19 +43,17 @@ class ImageComponent(BaseVisualComponent):
             stopType=stopType, stopVal=stopVal,
             startEstim=startEstim, durationEstim=durationEstim)
         self.type = 'Image'
-        self.targets = ['PsychoPy', 'PsychoJS']
-        self.url = "http://www.psychopy.org/builder/components/image.html"
+        self.url = "https://www.psychopy.org/builder/components/image.html"
         self.exp.requirePsychopyLibs(['visual'])
         # params
-        # was set by BaseVisual but for this stim it's advanced
-        self.params['color'].categ = "Advanced"
-        self.params['colorSpace'].categ = "Advanced"
-        self.order += ['image', 'pos', 'size', 'ori', 'opacity']
+        self.order += ['image',  # Basic tab
+                       'mask', 'texture resolution',  # Texture tab
+                       ]
 
         msg = _translate(
             "The image to be displayed - a filename, including path")
         self.params['image'] = Param(
-            image, valType='str', allowedTypes=[],
+            image, valType='file', inputType="file", allowedTypes=[], categ='Basic',
             updates='constant',
             allowedUpdates=['constant', 'set every repeat', 'set every frame'],
             hint=msg,
@@ -66,43 +63,61 @@ class ImageComponent(BaseVisualComponent):
             "An image to define the alpha mask through which the image is "
             "seen - gauss, circle, None or a filename (including path)")
         self.params['mask'] = Param(
-            mask, valType='str', allowedTypes=[],
+            mask, valType='str', inputType="file", allowedTypes=[], categ='Texture',
             updates='constant',
             allowedUpdates=['constant', 'set every repeat', 'set every frame'],
             hint=msg,
-            label=_localized["mask"], categ="Advanced")
+            label=_localized["mask"])
 
         msg = _translate("Resolution of the mask if one is used.")
         self.params['texture resolution'] = Param(
-            texRes, valType='code',
+            texRes, valType='num', inputType="choice", categ='Texture',
             allowedVals=['32', '64', '128', '256', '512'],
             updates='constant', allowedUpdates=[],
             hint=msg,
-            label=_localized["texture resolution"], categ="Advanced")
+            label=_localized["texture resolution"])
 
         msg = _translate(
             "How should the image be interpolated if/when rescaled")
         self.params['interpolate'] = Param(
-            interpolate, valType='str', allowedVals=['linear', 'nearest'],
+            interpolate, valType='str', inputType="choice", allowedVals=['linear', 'nearest'], categ='Texture',
             updates='constant', allowedUpdates=[],
-            hint=msg,
-            label=_localized["interpolate"], categ="Advanced")
+            hint=msg, direct=False,
+            label=_localized["interpolate"])
 
         msg = _translate(
             "Should the image be flipped vertically (top to bottom)?")
         self.params['flipVert'] = Param(
-            flipVert, valType='bool',
+            flipVert, valType='bool', inputType="bool", categ='Layout',
             updates='constant', allowedUpdates=[],
             hint=msg,
-            label=_localized["flipVert"], categ="Advanced")
+            label=_localized["flipVert"])
 
         msg = _translate(
             "Should the image be flipped horizontally (left to right)?")
         self.params['flipHoriz'] = Param(
-            flipHoriz, valType='bool',
+            flipHoriz, valType='bool', inputType="bool", categ='Layout',
             updates='constant', allowedUpdates=[],
             hint=msg,
-            label=_localized["flipHoriz"], categ="Advanced")
+            label=_localized["flipHoriz"])
+        self.params['anchor'] = Param(
+            anchor, valType='str', inputType="choice", categ='Layout',
+            allowedVals=['center',
+                         'top-center',
+                         'bottom-center',
+                         'center-left',
+                         'center-right',
+                         'top-left',
+                         'top-right',
+                         'bottom-left',
+                         'bottom-right',
+                         ],
+            updates='constant',
+            hint=_translate("Which point on the stimulus should be anchored to its exact position?"),
+            label=_translate('Anchor'))
+
+        del self.params['fillColor']
+        del self.params['borderColor']
 
     def writeInitCode(self, buff):
         # do we need units code?
@@ -116,7 +131,7 @@ class ImageComponent(BaseVisualComponent):
         code = ("{inits[name]} = visual.ImageStim(\n"
                 "    win=win,\n"
                 "    name='{inits[name]}', {units}\n"
-                "    image={inits[image]}, mask={inits[mask]},\n"
+                "    image={inits[image]}, mask={inits[mask]}, anchor={inits[anchor]},\n"
                 "    ori={inits[ori]}, pos={inits[pos]}, size={inits[size]},\n"
                 "    color={inits[color]}, colorSpace={inits[colorSpace]}, opacity={inits[opacity]},\n"
                 "    flipHoriz={inits[flipHoriz]}, flipVert={inits[flipVert]},\n"
