@@ -2,29 +2,24 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
-from builtins import super  # provides Py3-style super() using python-future
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 
 from os import path
+from pathlib import Path
 
 from psychopy.experiment.components.keyboard import KeyboardComponent, Param, _translate
 from psychopy.experiment.utils import valid_var_re
 
 __author__ = 'Jeremy Gray'
 
-# abs path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'ioLabs.png')
-tooltip = _translate('ioLabs ButtonBox: check and record response buttons on '
-                     'ioLab Systems ButtonBox')
-
 # only use _localized values for label values, nothing functional:
-_localized = {'active': _translate('Active buttons'),
-              'lights': _translate('Lights'),
-              'lights off': _translate('Lights off')}
+_localized.update({'active': _translate('Active buttons'),
+                   'lights': _translate('Lights'),
+                   'lights off': _translate('Lights off')})
 
 
 class ioLabsButtonBoxComponent(KeyboardComponent):
@@ -39,6 +34,9 @@ class ioLabsButtonBoxComponent(KeyboardComponent):
     """
     categories = ['Responses']  # which section(s) in the components panel
     targets = ['PsychoPy']
+    iconFile = Path(__file__).parent / 'ioLabs.png'
+    tooltip = _translate('ioLabs ButtonBox: check and record response buttons on '
+                         'ioLab Systems ButtonBox')
 
     def __init__(self, exp, parentName, name='bbox',
                  active="(0,1,2,3,4,5,6,7)", store='first button',
@@ -56,14 +54,17 @@ class ioLabsButtonBoxComponent(KeyboardComponent):
             startEstim=startEstim, durationEstim=durationEstim)
 
         self.type = 'ioLabsButtonBox'
-        self.url = "http://www.psychopy.org/builder/components/ioLabs.html"
+        self.url = "https://www.psychopy.org/builder/components/ioLabs.html"
 
         self.exp.requirePsychopyLibs(['hardware'])
         del self.params['allowedKeys']
 
         # NB name and timing params always come 1st
-        self.order = ['forceEndRoutine', 'active', 'lights', 'store',
-                      'storeCorrect', 'correctAns']
+        self.order += ['forceEndRoutine',  # Basic tab
+                       'allowedKeys', 'store', 'storeCorrect', 'correctAns'  # Data tab
+                       ]
+        #self.order = ['forceEndRoutine', 'active', 'lights', 'store',
+        #              'storeCorrect', 'correctAns']
 
         msg = _translate(
             "What is the 'correct' response? NB, buttons are labelled 0 to "
@@ -82,20 +83,20 @@ class ioLabsButtonBoxComponent(KeyboardComponent):
         msg = _translate("Active buttons, such as '1,6', '(1,2,5,6)' or '0' "
                          "(without quotes)")
         self.params['active'] = Param(
-            active, valType='code', allowedTypes=[],
+            active, valType='code', inputType="single", allowedTypes=[], categ='Data',
             updates='constant',
             allowedUpdates=['constant', 'set every repeat'],
             hint=msg,
             label=_localized['active'])
 
         self.params['lights'] = Param(
-            lights, valType='bool', allowedTypes=[],
+            lights, valType='bool', inputType="bool", allowedTypes=[], categ='Hardware',
             updates='constant', allowedUpdates=[],
             hint=_translate("Turn ON the lights for the active buttons?"),
             label=_localized['lights'])
 
         self.params['lights off'] = Param(
-            lightsOff, valType='bool', allowedTypes=[],
+            lightsOff, valType='bool', inputType="bool", allowedTypes=[], categ='Hardware',
             updates='constant', allowedUpdates=[],
             hint=_translate("Turn OFF all lights at the end of each routine?"),
             label=_localized['lights off'])
@@ -250,7 +251,7 @@ class ioLabsButtonBoxComponent(KeyboardComponent):
             code += ("    # was no response the correct answer?\n"
                      "    if str(%(correctAns)s).lower() == 'none':\n"
                      "        %(name)s.corr = 1  # correct non-response\n"
-                     "    else:\n        %(name)s.corr = 0  # failed to withold a response\n")
+                     "    else:\n        %(name)s.corr = 0  # failed to withhold a response\n")
 
         buff.writeIndentedLines(code % self.params)
 
