@@ -2,33 +2,24 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-
-from __future__ import absolute_import, print_function
-
-from builtins import str
-from builtins import range
-from past.builtins import basestring
-from os import path
+from pathlib import Path
 
 from psychopy.experiment.components import BaseComponent, Param, _translate
 from psychopy.experiment import CodeGenerationException, valid_var_re
-
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'joyButtons.png')
-tooltip = _translate('JoyButtons: check and record joystick/gamepad button presses')
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 
 # only use _localized values for label values, nothing functional:
-_localized = {'allowedKeys': _translate('Allowed buttons'),
-              'store': _translate('Store'),
-              'forceEndRoutine': _translate('Force end of Routine'),
-              'storeCorrect': _translate('Store correct'),
-              'correctAns': _translate('Correct answer'),
-              'deviceNumber': _translate('Device number'),
-              'syncScreenRefresh': _translate('sync RT with screen')}
+_localized.update({'allowedKeys': _translate('Allowed buttons'),
+                   'store': _translate('Store'),
+                   'forceEndRoutine': _translate('Force end of Routine'),
+                   'storeCorrect': _translate('Store correct'),
+                   'correctAns': _translate('Correct answer'),
+                   'deviceNumber': _translate('Device number'),
+                   'syncScreenRefresh': _translate('sync RT with screen')})
 
 
 class JoyButtonsComponent(BaseComponent):
@@ -36,6 +27,8 @@ class JoyButtonsComponent(BaseComponent):
     # an attribute of the class, determines the section in components panel
     categories = ['Responses']
     targets = ['PsychoPy']
+    iconFile = Path(__file__).parent / 'joyButtons.png'
+    tooltip = _translate('JoyButtons: check and record joystick/gamepad button presses')
 
     def __init__(self, exp, parentName, name='button_resp',
                  allowedKeys="0,1,2,3,4",
@@ -53,20 +46,19 @@ class JoyButtonsComponent(BaseComponent):
             startEstim=startEstim, durationEstim=durationEstim)
 
         self.type = 'JoyButtons'
-        self.url = "http://www.psychopy.org/builder/components/joyButtons.html"
+        self.url = "https://www.psychopy.org/builder/components/joyButtons.html"
         self.exp.requirePsychopyLibs(['gui'])
 
-        # params
-
-        # NB name and timing params always come 1st
-        self.order = ['forceEndRoutine', 'allowedKeys', 'store',
-                      'storeCorrect', 'correctAns', 'deviceNumber']
+        self.order += ['forceEndRoutine',  # Basic tab
+                       'allowedKeys', 'store', 'storeCorrect', 'correctAns',  # Data tab
+                       'deviceNumber',  # Hardware tab
+                       ]
 
         msg = _translate(
             "A comma-separated list of button numbers, such as "
             "0,1,2,3,4")
         self.params['allowedKeys'] = Param(
-            allowedKeys, valType='code', allowedTypes=[],
+            allowedKeys, valType='list', inputType="single", allowedTypes=[], categ='Data',
             updates='constant',
             allowedUpdates=['constant', 'set every repeat'],
             hint=(msg),
@@ -75,16 +67,16 @@ class JoyButtonsComponent(BaseComponent):
         msg = _translate("Choose which (if any) responses to store at the "
                          "end of a trial")
         self.params['store'] = Param(
-            store, valType='str', allowedTypes=[],
+            store, valType='str', inputType="choice", allowedTypes=[], categ='Data',
             allowedVals=['last key', 'first key', 'all keys', 'nothing'],
-            updates='constant',
+            updates='constant', direct=False,
             hint=msg,
             label=_localized['store'])
 
         msg = _translate("Should a response force the end of the Routine "
                          "(e.g end the trial)?")
         self.params['forceEndRoutine'] = Param(
-            forceEndRoutine, valType='bool', allowedTypes=[],
+            forceEndRoutine, valType='bool', inputType="bool", allowedTypes=[], categ='Basic',
             updates='constant',
             hint=msg,
             label=_localized['forceEndRoutine'])
@@ -92,7 +84,7 @@ class JoyButtonsComponent(BaseComponent):
         msg = _translate("Do you want to save the response as "
                          "correct/incorrect?")
         self.params['storeCorrect'] = Param(
-            storeCorrect, valType='bool', allowedTypes=[],
+            storeCorrect, valType='bool', inputType="bool", allowedTypes=[], categ='Data',
             updates='constant',
             hint=msg,
             label=_localized['storeCorrect'])
@@ -102,7 +94,7 @@ class JoyButtonsComponent(BaseComponent):
             "correctAns column and use $correctAns to compare to the key "
             "press.")
         self.params['correctAns'] = Param(
-            correctAns, valType='str', allowedTypes=[],
+            correctAns, valType='list', inputType="single", allowedTypes=[], categ='Data',
             updates='constant',
             hint=msg,
             label=_localized['correctAns'])
@@ -111,7 +103,7 @@ class JoyButtonsComponent(BaseComponent):
             "A reaction time to a visual stimulus should be based on when "
             "the screen flipped")
         self.params['syncScreenRefresh'] = Param(
-            syncScreenRefresh, valType='bool',
+            syncScreenRefresh, valType='bool', inputType="bool", categ='Data',
             updates='constant',
             hint=msg,
             label=_localized['syncScreenRefresh'])
@@ -120,10 +112,10 @@ class JoyButtonsComponent(BaseComponent):
             "Device number, if you have multiple devices which"
             " one do you want (0, 1, 2...)")
         self.params['deviceNumber'] = Param(
-            deviceNumber, valType='code', allowedTypes=[],
+            deviceNumber, valType='int', inputType="int", allowedTypes=[], categ='Hardware',
             updates='constant', allowedUpdates=[],
             hint=msg,
-            label=_localized['deviceNumber'], categ='Advanced')
+            label=_localized['deviceNumber'])
 
     def writeStartCode(self, buff):
         code = ("from psychopy.hardware import joystick as joysticklib  "
@@ -307,16 +299,7 @@ class JoyButtonsComponent(BaseComponent):
         if allowedKeys in [None, "none", "None", "", "[]", "()"]:
             keyList=[]
         elif not allowedKeysIsVar:
-            try:
-                keyList = eval(allowedKeys)
-            except Exception:
-                raise CodeGenerationException(
-                    self.params["name"], "Allowed keys list is invalid.")
-            # this means the user typed "left","right" not ["left","right"]
-            if type(keyList) == tuple:
-                keyList = list(keyList)
-            elif isinstance(keyList, int):  # a single string/key
-                keyList = [keyList]
+            keyList = self.params['allowedKeys']
 
         code1 = ("{name}.newButtonState = {name}.device.getAllButtons()[:]\n"
                  "{name}.pressedButtons = []\n"
@@ -455,7 +438,7 @@ class JoyButtonsComponent(BaseComponent):
         if currLoop.type in ['StairHandler', 'MultiStairHandler']:
             # data belongs to a Staircase-type of object
             if self.params['storeCorrect'].val is True:
-                code = ("%s.addResponse(%s.corr)\n" %
+                code = ("%s.addResponse(%s.corr, level)\n" %
                         (currLoop.params['name'], name) +
                         "%s.addOtherData('%s.rt', %s.rt)\n"
                         % (currLoop.params['name'], name, name))
