@@ -2,39 +2,28 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
+from pathlib import Path
 
-from builtins import str
-from builtins import range
-from builtins import super  # provides Py3-style super() using python-future
-from past.builtins import basestring
-
-from os import path
-
-from psychopy.constants import PY3
 from psychopy.experiment.components import BaseComponent, Param, _translate
 from psychopy.experiment import CodeGenerationException, valid_var_re
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 from pkgutil import find_loader
 
 # Check for psychtoolbox
 havePTB = find_loader('psychtoolbox') is not None
 
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'keyboard.png')
-tooltip = _translate('Keyboard: check and record keypresses')
-
 # only use _localized values for label values, nothing functional:
-_localized = {'allowedKeys': _translate('Allowed keys'),
-              'discard previous': _translate('Discard previous'),
-              'store': _translate('Store'),
-              'forceEndRoutine': _translate('Force end of Routine'),
-              'storeCorrect': _translate('Store correct'),
-              'correctAns': _translate('Correct answer'),
-              'syncScreenRefresh': _translate('Sync timing with screen')}
+_localized.update({'allowedKeys': _translate('Allowed keys'),
+                   'discard previous': _translate('Discard previous'),
+                   'store': _translate('Store'),
+                   'forceEndRoutine': _translate('Force end of Routine'),
+                   'storeCorrect': _translate('Store correct'),
+                   'correctAns': _translate('Correct answer'),
+                   'syncScreenRefresh': _translate('Sync timing with screen')})
 
 
 class KeyboardComponent(BaseComponent):
@@ -42,6 +31,8 @@ class KeyboardComponent(BaseComponent):
     # an attribute of the class, determines the section in components panel
     categories = ['Responses']
     targets = ['PsychoPy', 'PsychoJS']
+    iconFile = Path(__file__).parent / 'keyboard.png'
+    tooltip = _translate('Keyboard: check and record keypresses')
 
     def __init__(self, exp, parentName, name='key_resp',
                  allowedKeys="'y','n','left','right','space'",
@@ -58,20 +49,22 @@ class KeyboardComponent(BaseComponent):
             startEstim=startEstim, durationEstim=durationEstim)
 
         self.type = 'Keyboard'
-        self.url = "http://www.psychopy.org/builder/components/keyboard.html"
+        self.url = "https://www.psychopy.org/builder/components/keyboard.html"
         self.exp.requirePsychopyLibs(['gui'])
 
         # params
 
         # NB name and timing params always come 1st
-        self.order = ['forceEndRoutine', 'allowedKeys', 'store',
-                      'storeCorrect', 'correctAns']
+        self.order += ['forceEndRoutine',  # Basic tab
+                       'allowedKeys', 'store', 'storeCorrect', 'correctAns'  # Data tab
+                       ]
 
         msg = _translate(
             "A comma-separated list of keys (with quotes), such as "
             "'q','right','space','left'")
         self.params['allowedKeys'] = Param(
-            allowedKeys, valType='code', allowedTypes=[],
+            allowedKeys, valType='list', inputType="single", allowedTypes=[],
+            categ='Basic',
             updates='constant',
             allowedUpdates=['constant', 'set every repeat'],
             hint=(msg),
@@ -79,10 +72,10 @@ class KeyboardComponent(BaseComponent):
 
         # hints say 'responses' not 'key presses' because the same hint is
         # also used with button boxes
-        msg = _translate("Do you want to discard all responses occuring "
+        msg = _translate("Do you want to discard all responses occurring "
                          "before the onset of this component?")
         self.params['discard previous'] = Param(
-            discardPrev, valType='bool', allowedTypes=[],
+            discardPrev, valType='bool', inputType="bool", allowedTypes=[], categ='Data',
             updates='constant',
             hint=msg,
             label=_localized['discard previous'])
@@ -90,16 +83,16 @@ class KeyboardComponent(BaseComponent):
         msg = _translate("Choose which (if any) responses to store at the "
                          "end of a trial")
         self.params['store'] = Param(
-            store, valType='str', allowedTypes=[],
+            store, valType='str', inputType="choice", allowedTypes=[], categ='Data',
             allowedVals=['last key', 'first key', 'all keys', 'nothing'],
-            updates='constant',
+            updates='constant', direct=False,
             hint=msg,
             label=_localized['store'])
 
         msg = _translate("Should a response force the end of the Routine "
                          "(e.g end the trial)?")
         self.params['forceEndRoutine'] = Param(
-            forceEndRoutine, valType='bool', allowedTypes=[],
+            forceEndRoutine, valType='bool', inputType="bool", allowedTypes=[], categ='Basic',
             updates='constant',
             hint=msg,
             label=_localized['forceEndRoutine'])
@@ -107,26 +100,35 @@ class KeyboardComponent(BaseComponent):
         msg = _translate("Do you want to save the response as "
                          "correct/incorrect?")
         self.params['storeCorrect'] = Param(
-            storeCorrect, valType='bool', allowedTypes=[],
+            storeCorrect, valType='bool', inputType="bool", allowedTypes=[], categ='Data',
             updates='constant',
             hint=msg,
             label=_localized['storeCorrect'])
 
+        self.depends += [  # allows params to turn each other off/on
+            {"dependsOn": "storeCorrect",  # must be param name
+             "condition": "== True",  # val to check for
+             "param": "correctAns",  # param property to alter
+             "true": "enable",  # what to do with param if condition is True
+             "false": "disable",  # permitted: hide, show, enable, disable
+             }
+        ]
+
         msg = _translate(
             "What is the 'correct' key? Might be helpful to add a "
             "correctAns column and use $correctAns to compare to the key "
             "press.")
         self.params['correctAns'] = Param(
-            correctAns, valType='str', allowedTypes=[],
+            correctAns, valType='str', inputType="single", allowedTypes=[], categ='Data',
             updates='constant',
-            hint=msg,
+            hint=msg, direct=False,
             label=_localized['correctAns'])
 
         msg = _translate(
             "A reaction time to a visual stimulus should be based on when "
             "the screen flipped")
         self.params['syncScreenRefresh'] = Param(
-            syncScreenRefresh, valType='bool',
+            syncScreenRefresh, valType='bool', inputType="bool", categ='Data',
             updates='constant',
             hint=msg,
             label=_localized['syncScreenRefresh'])
@@ -158,7 +160,7 @@ class KeyboardComponent(BaseComponent):
         store = self.params['store'].val
         storeCorr = self.params['storeCorrect'].val
         forceEnd = self.params['forceEndRoutine'].val
-        allowedKeys = self.params['allowedKeys'].val.strip()
+        allowedKeys = str(self.params['allowedKeys'])
         visualSync = self.params['syncScreenRefresh'].val
 
         buff.writeIndented("\n")
@@ -175,7 +177,7 @@ class KeyboardComponent(BaseComponent):
         if allowedKeysIsVar:
             # if it looks like a variable, check that the variable is suitable
             # to eval at run-time
-            stringType = '{}'.format(['basestring', 'str'][PY3])
+            stringType = 'str'
             code = ("# AllowedKeys looks like a variable named `{0}`\n"
                     "if not type({0}) in [list, tuple, np.ndarray]:\n"
                     "    if not isinstance({0}, {1}):\n"
@@ -230,17 +232,7 @@ class KeyboardComponent(BaseComponent):
         if allowedKeys in [None, "none", "None", "", "[]", "()"]:
             keyListStr = ""
         elif not allowedKeysIsVar:
-            try:
-                keyList = eval(allowedKeys)
-            except Exception:
-                raise CodeGenerationException(
-                    self.params["name"], "Allowed keys list is invalid.")
-            # this means the user typed "left","right" not ["left","right"]
-            if type(keyList) == tuple:
-                keyList = list(keyList)
-            elif isinstance(keyList, basestring):  # a single string/key
-                keyList = [keyList]
-            keyListStr = "%s" % repr(keyList)
+            keyListStr = self.params['allowedKeys']
 
         # check for keypresses
         code = ("theseKeys = {name}.getKeys(keyList={keyStr}, waitRelease=False)\n"
@@ -313,7 +305,7 @@ class KeyboardComponent(BaseComponent):
             #        "    logging.error('AllowedKeys variable `%s` is not defined.')\n"
             #        "    core.quit()\n"
             #        "if not type(%s) in [list, tuple, np.ndarray]:\n"
-            #        "    if not isinstance(%s, basestring):\n"
+            #        "    if not isinstance(%s, str):\n"
             #        "        logging.error('AllowedKeys variable `%s` is "
             #        "not string- or list-like.')\n"
             #        "        core.quit()\n" %
@@ -376,7 +368,7 @@ class KeyboardComponent(BaseComponent):
             # this means the user typed "left","right" not ["left","right"]
             if type(keyList) == tuple:
                 keyList = list(keyList)
-            elif isinstance(keyList, basestring):  # a single string/key
+            elif isinstance(keyList, str):  # a single string/key
                 keyList = [keyList]
             keyListStr = "%s" % repr(keyList)
 
@@ -463,7 +455,7 @@ class KeyboardComponent(BaseComponent):
         if currLoop.type in ['StairHandler', 'MultiStairHandler']:
             # data belongs to a Staircase-type of object
             if self.params['storeCorrect'].val is True:
-                code = ("%s.addResponse(%s.corr)\n" %
+                code = ("%s.addResponse(%s.corr, level)\n" %
                         (currLoop.params['name'], name) +
                         "%s.addOtherData('%s.rt', %s.rt)\n"
                         % (currLoop.params['name'], name, name))
@@ -496,6 +488,8 @@ class KeyboardComponent(BaseComponent):
         store = self.params['store'].val
         forceEnd = self.params['forceEndRoutine'].val
         if store == 'nothing':
+            # Still stop keyboard to prevent textbox from not working on single keypresses due to buffer
+            buff.writeIndentedLines("%(name)s.stop();\n" % self.params)
             return
         if len(self.exp.flow._loopList):
             currLoop = self.exp.flow._loopList[-1]  # last (outer-most) loop
@@ -518,24 +512,38 @@ class KeyboardComponent(BaseComponent):
 
             buff.writeIndentedLines(code % self.params)
 
-        if currLoop.type in ['StairHandler', 'MultiStairHandler']:
-            raise CodeGenerationException(
-                "StairHandlers not currently supported by PsychoJS")
+        code = (
+            "// update the trial handler\n"
+            "if (currentLoop instanceof MultiStairHandler) {\n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
+        buff.setIndentLevel(1, relative=True)
+        code = (
+                "currentLoop.addResponse(%(name)s.corr, level);\n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
+        buff.setIndentLevel(-1, relative=True)
+        code = (
+            "}\n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
+        # always add keys
+        buff.writeIndented("psychoJS.experiment.addData('%(name)s.keys', %(name)s.keys);\n" % self.params)
+
+        if self.params['storeCorrect'].val == True:
+            buff.writeIndented("psychoJS.experiment.addData('%(name)s.corr', %(name)s.corr);\n" % self.params)
+
+        # only add an RT if we had a response
+        code = ("if (typeof {name}.keys !== 'undefined') {{  // we had a response\n"
+                "    psychoJS.experiment.addData('{name}.rt', {name}.rt);\n")
+        if forceEnd:
+            code += ("    routineTimer.reset();\n"
+                     "    }}\n\n")
         else:
-            # always add keys
-            buff.writeIndented("psychoJS.experiment.addData('%(name)s.keys', %(name)s.keys);\n" % self.params)
-
-            if self.params['storeCorrect'].val == True:
-                buff.writeIndented("psychoJS.experiment.addData('%(name)s.corr', %(name)s.corr);\n" % self.params)
-
-            # only add an RT if we had a response
-            code = ("if (typeof {name}.keys !== 'undefined') {{  // we had a response\n"
-                    "    psychoJS.experiment.addData('{name}.rt', {name}.rt);\n")
-            if forceEnd:
-                code += ("    routineTimer.reset();\n"
-                         "    }}\n\n")
-            else:
-                code += "    }}\n\n"
-            buff.writeIndentedLines(code.format(loopName=currLoop.params['name'], name=name))
+            code += "    }}\n\n"
+        buff.writeIndentedLines(code.format(loopName=currLoop.params['name'], name=name))
         # Stop keyboard
         buff.writeIndentedLines("%(name)s.stop();\n" % self.params)
