@@ -2,28 +2,22 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
-from builtins import super  # provides Py3-style super() using python-future
-
-from os import path
+from pathlib import Path
 from psychopy.experiment.components import BaseComponent, Param, _translate
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 import re
 
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'mouse.png')
-tooltip = _translate('Mouse: query mouse position and buttons')
-
 # only use _localized values for label values, nothing functional:
-_localized = {'saveMouseState': _translate('Save mouse state'),
-              'forceEndRoutineOnPress': _translate('End Routine on press'),
-              'timeRelativeTo': _translate('Time relative to'),
-              'Clickable stimuli': _translate('Clickable stimuli'),
-              'Store params for clicked': _translate('Store params for clicked'),
-              'New clicks only': _translate('New clicks only')}
+_localized.update({'saveMouseState': _translate('Save mouse state'),
+                   'forceEndRoutineOnPress': _translate('End Routine on press'),
+                   'timeRelativeTo': _translate('Time relative to'),
+                   'Clickable stimuli': _translate('Clickable stimuli'),
+                   'Store params for clicked': _translate('Store params for clicked'),
+                   'New clicks only': _translate('New clicks only')})
 
 
 class MouseComponent(BaseComponent):
@@ -32,6 +26,8 @@ class MouseComponent(BaseComponent):
     """
     categories = ['Responses']
     targets = ['PsychoPy', 'PsychoJS']
+    iconFile = Path(__file__).parent / 'mouse.png'
+    tooltip = _translate('Mouse: query mouse position and buttons')
 
     def __init__(self, exp, parentName, name='mouse',
                  startType='time (s)', startVal=0.0,
@@ -46,14 +42,13 @@ class MouseComponent(BaseComponent):
             startEstim=startEstim, durationEstim=durationEstim)
 
         self.type = 'Mouse'
-        self.url = "http://www.psychopy.org/builder/components/mouse.html"
+        self.url = "https://www.psychopy.org/builder/components/mouse.html"
         self.exp.requirePsychopyLibs(['event'])
-        self.categories = ['Inputs']
 
         self.order += [
-            'forceEndRoutineOnPress',
-            'saveMouseState', 'timeRelativeTo',
-            'newClicksOnly', 'clickable', 'saveParamsClickable']
+            'forceEndRoutineOnPress',  # Basic tab
+            'saveMouseState', 'timeRelativeTo', 'newClicksOnly', 'clickable', 'saveParamsClickable',  # Data tab
+            ]
 
         # params
         msg = _translate(
@@ -61,9 +56,9 @@ class MouseComponent(BaseComponent):
             "On every video frame, every click or just at the end of the "
             "Routine?")
         self.params['saveMouseState'] = Param(
-            save, valType='str',
-            allowedVals=['final', 'on click', 'every frame', 'never'],
-            hint=msg,
+            save, valType='str', inputType="choice", categ='Data',
+            allowedVals=['final', 'on click', 'on valid click', 'every frame', 'never'],
+            hint=msg, direct=False,
             label=_localized['saveMouseState'])
 
         msg = _translate("Should a button press force the end of the routine"
@@ -73,37 +68,37 @@ class MouseComponent(BaseComponent):
         elif forceEndRoutineOnPress is False:
             forceEndRoutineOnPress = 'never'
         self.params['forceEndRoutineOnPress'] = Param(
-            forceEndRoutineOnPress, valType='str',
+            forceEndRoutineOnPress, valType='str', inputType="choice", categ='Basic',
             allowedVals=['never', 'any click', 'valid click'],
-            updates='constant',
+            updates='constant', direct=False,
             hint=msg,
             label=_localized['forceEndRoutineOnPress'])
 
         msg = _translate("What should the values of mouse.time should be "
                          "relative to?")
         self.params['timeRelativeTo'] = Param(
-            timeRelativeTo, valType='str',
+            timeRelativeTo, valType='str', inputType="choice", categ='Data',
             allowedVals=['mouse onset', 'experiment', 'routine'],
             updates='constant',
-            hint=msg,
+            hint=msg, direct=False,
             label=_localized['timeRelativeTo'])
 
-
         msg = _translate('If the mouse button is already down when we start '
                          'checking then wait for it to be released before '
                          'recording as a new click.'
                          )
         self.params['newClicksOnly'] = Param(
-            True, valType='bool',
+            True, valType='bool', inputType="bool", categ='Basic',
             updates='constant',
             hint=msg,
             label=_localized['New clicks only'])
+
         msg = _translate('A comma-separated list of your stimulus names that '
                          'can be "clicked" by the participant. '
                          'e.g. target, foil'
                          )
         self.params['clickable'] = Param(
-            '', valType='code',
+            '', valType='list', inputType="single", categ='Basic',
             updates='constant',
             hint=msg,
             label=_localized['Clickable stimuli'])
@@ -114,12 +109,11 @@ class MouseComponent(BaseComponent):
                          'clickable objects have all these params.'
                          )
         self.params['saveParamsClickable'] = Param(
-            'name,', valType='code',
-            updates='constant', allowedUpdates=[],
+            'name,', valType='list', inputType="single", categ='Data',
+            updates='constant', allowedUpdates=[], direct=False,
             hint=msg,
             label=_localized['Store params for clicked'])
 
-
     @property
     def _clickableParamsList(self):
         # convert clickableParams (str) to a list
@@ -132,7 +126,12 @@ class MouseComponent(BaseComponent):
         code = (
             "# check if the mouse was inside our 'clickable' objects\n"
             "gotValidClick = False\n"
-            "for obj in [%(clickable)s]:\n"
+            "try:\n"
+            "    iter(%(clickable)s)\n"
+            "    clickableList = %(clickable)s\n"
+            "except:\n"
+            "    clickableList = [%(clickable)s]\n"
+            "for obj in clickableList:\n"
             "    if obj.contains(%(name)s):\n"
             "        gotValidClick = True\n")
         buff.writeIndentedLines(code % self.params)
@@ -187,7 +186,7 @@ class MouseComponent(BaseComponent):
         # we need more than one
         code = ("# setup some python lists for storing info about the "
                 "%(name)s\n")
-        if self.params['saveMouseState'].val in ['every frame', 'on click']:
+        if self.params['saveMouseState'].val in ['every frame', 'on click', 'on valid click']:
             code += ("%(name)s.x = []\n"
                      "%(name)s.y = []\n"
                      "%(name)s.leftButton = []\n"
@@ -209,7 +208,7 @@ class MouseComponent(BaseComponent):
         """Write the code that will be called at the start of the routine"""
 
         code = ("// setup some python lists for storing info about the %(name)s\n")
-        if self.params['saveMouseState'].val in ['every frame', 'on click']:
+        if self.params['saveMouseState'].val in ['every frame', 'on click', 'on valid click']:
             code += ("// current position of the mouse:\n"
                      "%(name)s.x = [];\n"
                      "%(name)s.y = [];\n"
@@ -240,14 +239,6 @@ class MouseComponent(BaseComponent):
         elif timeRelative in ['routine', 'mouse onset']:
             self.clockStr = '%s.mouseClock' % self.params['name'].val
 
-        # only write code for cases where we are storing data as we go (each
-        # frame or each click)
-
-        # might not be saving clicks, but want it to force end of trial
-        if (self.params['saveMouseState'].val not in
-                ['every frame', 'on click'] and forceEnd == 'never'):
-            return
-
         buff.writeIndented("# *%s* updates\n" % self.params['name'])
 
         # writes an if statement to determine whether to draw etc
@@ -277,6 +268,14 @@ class MouseComponent(BaseComponent):
             # to get out of the if statement
             buff.setIndentLevel(-2, relative=True)
 
+        # only write code for cases where we are storing data as we go (each
+        # frame or each click)
+
+        # might not be saving clicks, but want it to force end of trial
+        if (self.params['saveMouseState'].val not in
+                ['every frame', 'on click', 'on valid click'] and forceEnd == 'never'):
+            return
+
         # if STARTED and not FINISHED!
         code = ("if %(name)s.status == STARTED:  "
                 "# only update if started and not finished!\n") % self.params
@@ -301,17 +300,17 @@ class MouseComponent(BaseComponent):
             return buff, dedent
 
         # No mouse tracking, end routine on any or valid click
-        if self.params['saveMouseState'].val == 'never' and forceEnd in ['any click', 'valid click']:
+        if self.params['saveMouseState'].val in ['never', 'final'] and forceEnd in ['any click', 'valid click']:
             buff, dedentAtEnd = _buttonPressCode(buff, dedentAtEnd)
 
             if forceEnd == 'valid click':
                     # does valid response end the trial?
-                    code = ("if gotValidClick:  # abort routine on response\n"
-                            "    continueRoutine = False\n")
+                    code = ("if gotValidClick:  \n"
+                            "    continueRoutine = False  # abort routine on response\n")
                     buff.writeIndentedLines(code)
                     buff.setIndentLevel(-dedentAtEnd, relative=True)
             else:
-                buff.writeIndented('continueRoutine = False')
+                buff.writeIndented('continueRoutine = False  # abort routine on response')
                 buff.setIndentLevel(-dedentAtEnd, relative=True)
 
         elif self.params['saveMouseState'].val != 'never':
@@ -330,9 +329,18 @@ class MouseComponent(BaseComponent):
                 buff.writeIndentedLines(mouseCode)
 
             # Continuous mouse tracking for all button press
-            if forceEnd == 'never' and self.params['saveMouseState'].val in ['on click']:
+            if forceEnd == 'never' and self.params['saveMouseState'].val in ['on click', 'on valid click']:
                 buff, dedentAtEnd = _buttonPressCode(buff, dedentAtEnd)
-                buff.writeIndentedLines(mouseCode)
+                if self.params['saveMouseState'].val in ['on click']:
+                    buff.writeIndentedLines(mouseCode)
+                elif self.params['clickable'].val and self.params['saveMouseState'].val in ['on valid click']:
+                    code = (
+                        "if gotValidClick:\n"
+                    )
+                    buff.writeIndentedLines(code)
+                    buff.setIndentLevel(+1, relative=True)
+                    buff.writeIndentedLines(mouseCode)
+                    buff.setIndentLevel(-1, relative=True)
 
             # Mouse tracking for events that end routine
             elif forceEnd in ['any click', 'valid click']:
@@ -340,17 +348,25 @@ class MouseComponent(BaseComponent):
                 # Save all mouse events on button press
                 if self.params['saveMouseState'].val in ['on click']:
                     buff.writeIndentedLines(mouseCode)
+                elif self.params['clickable'].val and self.params['saveMouseState'].val in ['on valid click']:
+                    code = (
+                        "if gotValidClick:\n"
+                    )
+                    buff.writeIndentedLines(code)
+                    buff.setIndentLevel(+1, relative=True)
+                    buff.writeIndentedLines(mouseCode)
+                    buff.setIndentLevel(-1, relative=True)
                 # also write code about clicked objects if needed.
                 if self.params['clickable'].val:
                     # does valid response end the trial?
                     if forceEnd == 'valid click':
-                        code = ("if gotValidClick:  # abort routine on response\n"
-                                "    continueRoutine = False\n")
+                        code = ("if gotValidClick:\n"
+                                "    continueRoutine = False  # abort routine on response\n")
                         buff.writeIndentedLines(code)
                 # does any response end the trial?
                 if forceEnd == 'any click':
-                    code = ("# abort routine on response\n"
-                            "continueRoutine = False\n")
+                    code = ("\n"
+                            "continueRoutine = False  # abort routine on response\n")
                     buff.writeIndentedLines(code)
                 else:
                     pass # forceEnd == 'never'
@@ -372,7 +388,7 @@ class MouseComponent(BaseComponent):
 
         # might not be saving clicks, but want it to force end of trial
         if (self.params['saveMouseState'].val not in
-                ['every frame', 'on click'] and forceEnd == 'never'):
+                ['every frame', 'on click', 'on valid click'] and forceEnd == 'never'):
             return
         buff.writeIndented("// *%s* updates\n" % self.params['name'])
 
@@ -413,8 +429,7 @@ class MouseComponent(BaseComponent):
         dedentAtEnd = 1  # keep track of how far to dedent later
 
         # write param checking code
-        if (self.params['saveMouseState'].val == 'on click'
-                or forceEnd in ['any click', 'valid click']):
+        if (self.params['saveMouseState'].val in ['on click', 'on valid click'] or forceEnd in ['any click', 'valid click']):
             code = ("_mouseButtons = %(name)s.getPressed();\n")
             buff.writeIndentedLines(code % self.params)
             # buff.setIndentLevel(1, relative=True)
@@ -433,21 +448,35 @@ class MouseComponent(BaseComponent):
             code = "_mouseButtons = %(name)s.getPressed();\n" % self.params
             buff.writeIndented(code)
 
-        # only do this if buttons were pressed
-        if self.params['saveMouseState'].val in ['on click', 'every frame']:
-            code = ("_mouseXYs = %(name)s.getPos();\n"
+        # also write code about clicked objects if needed.
+        if self.params['clickable'].val:
+            self._writeClickableObjectsCodeJS(buff)
+
+        if self.params['saveMouseState'].val in ['on click', 'on valid click', 'every frame']:
+            storeCode = (
+                    "_mouseXYs = %(name)s.getPos();\n"
                     "%(name)s.x.push(_mouseXYs[0]);\n"
                     "%(name)s.y.push(_mouseXYs[1]);\n"
                     "%(name)s.leftButton.push(_mouseButtons[0]);\n"
                     "%(name)s.midButton.push(_mouseButtons[1]);\n"
-                    "%(name)s.rightButton.push(_mouseButtons[2]);\n" %
-                    self.params)
-            code += ("%s.time.push(%s.getTime());\n" % (self.params['name'], self.clockStr))
-            buff.writeIndentedLines(code)
-
-        # also write code about clicked objects if needed.
-        if self.params['clickable'].val:
-            self._writeClickableObjectsCodeJS(buff)
+                    "%(name)s.rightButton.push(_mouseButtons[2]);\n"
+                    % self.params
+            )
+            storeCode += ("%s.time.push(%s.getTime());\n" % (self.params['name'], self.clockStr))
+            if self.params['clickable'].val and self.params['saveMouseState'].val in ['on valid click']:
+                code = (
+                    "if (gotValidClick === true) { \n"
+                )
+                buff.writeIndentedLines(code)
+                buff.setIndentLevel(+1, relative=True)
+                buff.writeIndentedLines(storeCode)
+                buff.setIndentLevel(-1, relative=True)
+                code = (
+                    "}\n"
+                )
+                buff.writeIndentedLines(code)
+            else:
+                buff.writeIndentedLines(storeCode)
 
             # does the response end the trial?
         if forceEnd == 'any click':
@@ -544,9 +573,8 @@ class MouseComponent(BaseComponent):
                     buff.writeIndented(code)
                 else:
                     # we only had one click so don't return a list
-                    code = ("if len(%s.%s): %s.addData('%s.%s', %s.%s[0])\n" %
-                            (name, property,
-                             currLoop.params['name'], name,
+                    code = ("%s.addData('%s.%s', %s.%s)\n" %
+                            (currLoop.params['name'], name,
                              property, name, property))
                     buff.writeIndented(code)
 
