@@ -2,26 +2,21 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
-
-from os import path
+from pathlib import Path
 import copy
-from psychopy import logging
-from psychopy.experiment.components import BaseVisualComponent, getInitVals, Param, _translate
 
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'movie.png')
-tooltip = _translate('Movie: play movie files')
+from psychopy.experiment.components import BaseVisualComponent, getInitVals, Param, _translate
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 
 # only use _localized values for label values, nothing functional:
-_localized = {'movie': _translate('Movie file'),
-              'forceEndRoutine': _translate('Force end of Routine'),
-              'backend': _translate('backend'),
-              'No audio': _translate('No audio')}
+_localized.update({'movie': _translate('Movie file'),
+                   'forceEndRoutine': _translate('Force end of Routine'),
+                   'backend': _translate('backend'),
+                   'No audio': _translate('No audio')})
 
 if _localized['backend'] == 'backend': # this is the only non-capitals label
     _localized['backend'] = 'Backend'
@@ -29,9 +24,14 @@ if _localized['backend'] == 'backend': # this is the only non-capitals label
 class MovieComponent(BaseVisualComponent):
     """An event class for presenting movie-based stimuli"""
 
+    categories = ['Stimuli']
+    targets = ['PsychoPy', 'PsychoJS']
+    iconFile = Path(__file__).parent / 'movie.png'
+    tooltip = _translate('Movie: play movie files')
+
     def __init__(self, exp, parentName, name='movie', movie='',
                  units='from exp settings',
-                 pos=(0, 0), size='', ori=0,
+                 pos=(0, 0), size='', anchor="center", ori=0,
                  startType='time (s)', startVal=0.0,
                  stopType='duration (s)', stopVal=1.0,
                  startEstim='', durationEstim='',
@@ -46,10 +46,11 @@ class MovieComponent(BaseVisualComponent):
             startEstim=startEstim, durationEstim=durationEstim)
 
         self.type = 'Movie'
-        self.url = "http://www.psychopy.org/builder/components/movie.html"
+        self.url = "https://www.psychopy.org/builder/components/movie.html"
         # comes immediately after name and timing params
-        self.order = ['movie', 'backend', 'No audio', 'loop', 'forceEndRoutine']
-        self.targets = ['PsychoPy', 'PsychoJS']
+        self.order += ['movie', 'forceEndRoutine', # Basic tab
+                       'loop', 'No audio', 'backend',
+                       ]
 
         # params
         self.params['stopVal'].hint = _translate(
@@ -58,30 +59,29 @@ class MovieComponent(BaseVisualComponent):
 
         msg = _translate("A filename for the movie (including path)")
         self.params['movie'] = Param(
-            movie, valType='str', allowedTypes=[],
+            movie, valType='file', inputType="file", allowedTypes=[], categ='Basic',
             updates='constant', allowedUpdates=['constant', 'set every repeat'],
             hint=msg,
             label=_localized['movie'])
 
         msg = _translate("What underlying lib to use for loading movies")
         self.params['backend'] = Param(
-            backend, valType='str',
-            allowedVals=['moviepy', 'avbin', 'opencv'],
-            hint=msg,
+            backend, valType='str', inputType="choice", categ='Playback',
+            allowedVals=['moviepy', 'avbin', 'opencv', 'vlc'],
+            hint=msg, direct=False,
             label=_localized['backend'])
 
-        # todo: msg = _translate(...)
         msg = _translate("Prevent the audio stream from being loaded/processed "
                "(moviepy and opencv only)")
         self.params["No audio"] = Param(
-            noAudio, valType='bool',
+            noAudio, valType='bool', inputType="bool", categ='Playback',
             hint=msg,
             label=_localized['No audio'])
 
         msg = _translate("Should the end of the movie cause the end of "
                          "the routine (e.g. trial)?")
         self.params['forceEndRoutine'] = Param(
-            forceEndRoutine, valType='bool', allowedTypes=[],
+            forceEndRoutine, valType='bool', inputType="bool", allowedTypes=[], categ='Basic',
             updates='constant', allowedUpdates=[],
             hint=msg,
             label=_localized['forceEndRoutine'])
@@ -89,13 +89,30 @@ class MovieComponent(BaseVisualComponent):
         msg = _translate("Whether the movie should loop back to the beginning "
                          "on completion.")
         self.params['loop'] = Param(
-            loop, valType='bool',
+            loop, valType='bool', inputType="bool", categ='Playback',
             hint=msg,
             label=_translate('Loop playback'))
+        self.params['anchor'] = Param(
+            anchor, valType='str', inputType="choice", categ='Layout',
+            allowedVals=['center',
+                         'top-center',
+                         'bottom-center',
+                         'center-left',
+                         'center-right',
+                         'top-left',
+                         'top-right',
+                         'bottom-left',
+                         'bottom-right',
+                         ],
+            updates='constant',
+            hint=_translate("Which point on the stimulus should be anchored to its exact position?"),
+            label=_translate('Anchor'))
 
         # these are normally added but we don't want them for a movie
         del self.params['color']
         del self.params['colorSpace']
+        del self.params['fillColor']
+        del self.params['borderColor']
 
     def _writeCreationCode(self, buff, useInits):
         # This will be called by either self.writeInitCode() or
@@ -106,9 +123,9 @@ class MovieComponent(BaseVisualComponent):
         #
         # leave units blank if not needed
         if self.params['units'].val == 'from exp settings':
-            unitsStr = ""
+            unitsStr = "units=''"
         else:
-            unitsStr = "units=%(units)s, " % self.params
+            unitsStr = "units=%(units)s" % self.params
 
         # If we're in writeInitCode then we need to convert params to initVals
         # because some (variable) params haven't been created yet.
@@ -119,19 +136,26 @@ class MovieComponent(BaseVisualComponent):
 
         if self.params['backend'].val == 'moviepy':
             code = ("%s = visual.MovieStim3(\n" % params['name'] +
-                    "    win=win, name='%s',%s\n" % (params['name'], unitsStr) +
+                    "    win=win, name='%s', %s,\n" % (
+                        params['name'], unitsStr) +
                     "    noAudio = %(No audio)s,\n" % params)
         elif self.params['backend'].val == 'avbin':
             code = ("%s = visual.MovieStim(\n" % params['name'] +
-                    "    win=win, name='%s',%s\n" % (params['name'], unitsStr))
+                    "    win=win, name='%s', %s,\n" % (
+                        params['name'], unitsStr))
+        elif self.params['backend'].val == 'vlc':
+            code = ("%s = visual.VlcMovieStim(\n" % params['name'] +
+                    "    win=win, name='%s', %s,\n" % (
+                        params['name'], unitsStr))
         else:
             code = ("%s = visual.MovieStim2(\n" % params['name'] +
-                    "    win=win, name='%s',%s\n" % (params['name'], unitsStr) +
-                    "    noAudio = %(No audio)s,\n" % params)
+                    "    win=win, name='%s', %s,\n" % (
+                        params['name'], unitsStr) +
+                    "    noAudio=%(No audio)s,\n" % params)
 
         code += ("    filename=%(movie)s,\n"
                  "    ori=%(ori)s, pos=%(pos)s, opacity=%(opacity)s,\n"
-                 "    loop=%(loop)s,\n"
+                 "    loop=%(loop)s, anchor=%(anchor)s,\n"
                  % params)
 
         buff.writeIndentedLines(code)
@@ -211,8 +235,9 @@ class MovieComponent(BaseVisualComponent):
         # If needed then use _writeCreationCode()
         # Movie could be created here or in writeInitCode()
         if self.params['movie'].updates != 'constant':
-            # create the code using params, not vals
-            self._writeCreationCodeJS(buff, useInits=False)
+            # create the code using params, not vals (unless set during static component)
+            useInits = 'during' in self.params['movie'].updates
+            self._writeCreationCodeJS(buff, useInits=useInits)
 
     def writeFrameCode(self, buff):
         """Write the code that will be called every frame
