@@ -2,37 +2,36 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
-from builtins import super  # provides Py3-style super() using python-future
-
-from os import path
+from pathlib import Path
 from psychopy.experiment.components import BaseComponent, Param, _translate
 from psychopy import prefs
-
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'parallelOut.png')
-tooltip = _translate('Parallel out: send signals from the parallel port')
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 
 # only use _localized values for label values, nothing functional:
-_localized = {'address': _translate('Port address'),
-              'startData': _translate("Start data"),
-              'stopData': _translate("Stop data"),
-              'syncScreen': _translate('Sync to screen')}
+_localized.update({'address': _translate('Port address'),
+                   'register': _translate('U3 Register'),
+                   'startData': _translate("Start data"),
+                   'stopData': _translate("Stop data"),
+                   'syncScreen': _translate('Sync to screen')})
 
 
 class ParallelOutComponent(BaseComponent):
     """A class for sending signals from the parallel port"""
-    categories = ['I/O']
+
+    categories = ['I/O', 'EEG']
+    targets = ['PsychoPy']
+    iconFile = Path(__file__).parent / 'parallelOut.png'
+    tooltip = _translate('Parallel out: send signals from the parallel port')
 
     def __init__(self, exp, parentName, name='p_port',
                  startType='time (s)', startVal=0.0,
                  stopType='duration (s)', stopVal=1.0,
                  startEstim='', durationEstim='',
-                 address=None, startData="1", stopData="0",
+                 address=None, register='EIO', startData="1", stopData="0",
                  syncScreen=True):
         super(ParallelOutComponent, self).__init__(
             exp, parentName, name,
@@ -41,12 +40,14 @@ class ParallelOutComponent(BaseComponent):
             startEstim=startEstim, durationEstim=durationEstim)
 
         self.type = 'ParallelOut'
-        self.url = "http://www.psychopy.org/builder/components/parallelout.html"
-        self.categories = ['I/O']
+        self.url = "https://www.psychopy.org/builder/components/parallelout.html"
         self.exp.requirePsychopyLibs(['parallel'])
 
         # params
-        self.order = ['address', 'startData', 'stopData']
+        self.order += [
+            'startData', 'stopData',  # Data tab
+            'address',  'register',   # Hardware tab
+        ]
 
         # main parameters
         addressOptions = prefs.hardware['parallelPorts'] + [u'LabJack U3'] + [u'USB2TTL8'] 
@@ -56,24 +57,37 @@ class ParallelOutComponent(BaseComponent):
         msg = _translate("Parallel port to be used (you can change these "
                          "options in preferences>general)")
         self.params['address'] = Param(
-            address, valType='str', allowedVals=addressOptions,
-            hint=msg,
-            label=_localized['address'])
+            address, valType='str', inputType="choice", allowedVals=addressOptions,
+            categ='Hardware', hint=msg, label=_localized['address'])
+
+        self.depends.append(
+            {"dependsOn": "address",  # must be param name
+             "condition": "=='LabJack U3'",  # val to check for
+             "param": "register",  # param property to alter
+             "true": "show",  # what to do with param if condition is True
+             "false": "hide",  # permitted: hide, show, enable, disable
+             }
+        )
+
+        msg = _translate("U3 Register to write byte to")
+        self.params['register'] = Param(register, valType='str',
+                                        inputType="choice", allowedVals=['EIO', 'FIO'],
+                                        categ='Hardware', hint=msg, label=_localized['register'])
 
         self.params['startData'] = Param(
-            startData, valType='code', allowedTypes=[],
+            startData, valType='code', inputType="single", allowedTypes=[], categ='Data',
             hint=_translate("Data to be sent at 'start'"),
             label=_localized['startData'])
 
         self.params['stopData'] = Param(
-            stopData, valType='code', allowedTypes=[],
+            stopData, valType='code', inputType="single", allowedTypes=[], categ='Data',
             hint=_translate("Data to be sent at 'end'"),
             label=_localized['stopData'])
 
         msg = _translate("If the parallel port data relates to visual "
                          "stimuli then sync its pulse to the screen refresh")
         self.params['syncScreen'] = Param(
-            syncScreen, valType='bool',
+            syncScreen, valType='bool', inputType="bool", categ='Data',
             allowedVals=[True, False],
             updates='constant', allowedUpdates=[],
             hint=msg,
@@ -107,11 +121,18 @@ class ParallelOutComponent(BaseComponent):
         self.writeStartTestCode(buff)
         buff.writeIndented("%(name)s.status = STARTED\n" % self.params)
 
-        if not self.params['syncScreen'].val:
-            code = "%(name)s.setData(int(%(startData)s))\n" % self.params
+        if self.params['address'].val == 'LabJack U3':
+            if not self.params['syncScreen'].val:
+                code = "%(name)s.setData(int(%(startData)s), address=%(register)s)\n" % self.params
+            else:
+                code = ("win.callOnFlip(%(name)s.setData, int(%(startData)s), address=%(register)s)\n" %
+                        self.params)
         else:
-            code = ("win.callOnFlip(%(name)s.setData, int(%(startData)s))\n" %
-                    self.params)
+            if not self.params['syncScreen'].val:
+                code = "%(name)s.setData(int(%(startData)s))\n" % self.params
+            else:
+                code = ("win.callOnFlip(%(name)s.setData, int(%(startData)s))\n" %
+                        self.params)
 
         buff.writeIndented(code)
 
@@ -123,11 +144,18 @@ class ParallelOutComponent(BaseComponent):
             self.writeStopTestCode(buff)
             buff.writeIndented("%(name)s.status = FINISHED\n" % self.params)
 
-            if not self.params['syncScreen'].val:
-                code = "%(name)s.setData(int(%(stopData)s))\n" % self.params
+            if self.params['address'].val == 'LabJack U3':
+                if not self.params['syncScreen'].val:
+                    code = "%(name)s.setData(int(%(stopData)s), address=%(register)s)\n" % self.params
+                else:
+                    code = ("win.callOnFlip(%(name)s.setData, int(%(stopData)s), address=%(register)s)\n" %
+                            self.params)
             else:
-                code = ("win.callOnFlip(%(name)s.setData, int(%(stopData)s))\n" %
-                        self.params)
+                if not self.params['syncScreen'].val:
+                    code = "%(name)s.setData(int(%(stopData)s))\n" % self.params
+                else:
+                    code = ("win.callOnFlip(%(name)s.setData, int(%(stopData)s))\n" %
+                            self.params)
 
             buff.writeIndented(code)
 
@@ -142,11 +170,17 @@ class ParallelOutComponent(BaseComponent):
         # make sure that we do switch to stopData if the routine has been
         # aborted before our 'end'
         buff.writeIndented("if %(name)s.status == STARTED:\n" % self.params)
-        if not self.params['syncScreen'].val:
-            code = "    %(name)s.setData(int(%(stopData)s))\n" % self.params
+        if self.params['address'].val == 'LabJack U3':
+            if not self.params['syncScreen'].val:
+                code = "    %(name)s.setData(int(%(stopData)s), address=%(register)s)\n" % self.params
+            else:
+                code = ("    win.callOnFlip(%(name)s.setData, int(%(stopData)s), address=%(register)s)\n" %
+                        self.params)
         else:
-            code = ("    win.callOnFlip(%(name)s.setData, int(%(stopData)s))\n" %
-                    self.params)
+            if not self.params['syncScreen'].val:
+                code = "    %(name)s.setData(int(%(stopData)s))\n" % self.params
+            else:
+                code = ("    win.callOnFlip(%(name)s.setData, int(%(stopData)s))\n" % self.params)
 
         buff.writeIndented(code)
 
