@@ -2,128 +2,124 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
+from pathlib import Path
 
-from builtins import str
-from os import path
-import copy
 from psychopy.experiment.components import BaseVisualComponent, Param, getInitVals, _translate
 from psychopy import logging
-
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'polygon.png')
-tooltip = _translate('Polygon: any regular polygon (line, triangle, square'
-                     '...circle)')
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 
 # only use _localized values for label values, nothing functional:
-_localized = {'nVertices': _translate('Num. vertices'),
-              'fillColorSpace': _translate('Fill color-space'),
-              'fillColor': _translate('Fill color'),
-              'lineColorSpace': _translate('Line color-space'),
-              'lineColor': _translate('Line color'),
-              'lineWidth': _translate('Line width'),
-              'interpolate': _translate('Interpolate'),
-              'size': _translate("Size [w,h]"),
-              'shape': _translate("Shape")}
+_localized = _localized.copy()
+_localized.update({'nVertices': _translate('Num. vertices'),
+                   'fillColor': _translate('Fill color'),
+                   'lineColor': _translate('Line color'),
+                   'lineWidth': _translate('Line width'),
+                   'interpolate': _translate('Interpolate'),
+                   'size': _translate("Size [w,h]"),
+                   'shape': _translate("Shape")})
 
 
 class PolygonComponent(BaseVisualComponent):
     """A class for presenting grating stimuli"""
 
+    categories = ['Stimuli']
+    targets = ['PsychoPy', 'PsychoJS']
+    iconFile = Path(__file__).parent / 'polygon.png'
+    tooltip = _translate('Polygon: any regular polygon (line, triangle, square'
+                         '...circle)')
+
     def __init__(self, exp, parentName, name='polygon', interpolate='linear',
-                 units='from exp settings',
-                 lineColor='$[1,1,1]', lineColorSpace='rgb', lineWidth=1,
-                 fillColor='$[1,1,1]', fillColorSpace='rgb',
-                 shape='triangle', nVertices=4,
+                 units='from exp settings', anchor='center',
+                 lineColor='white', lineColorSpace='rgb', lineWidth=1,
+                 fillColor='white', fillColorSpace='rgb',
+                 shape='triangle', nVertices=4, vertices="",
                  pos=(0, 0), size=(0.5, 0.5), ori=0,
                  startType='time (s)', startVal=0.0,
                  stopType='duration (s)', stopVal=1.0,
                  startEstim='', durationEstim=''):
         super(PolygonComponent, self).__init__(
             exp, parentName, name=name, units=units,
+            fillColor=fillColor, borderColor=lineColor,
             pos=pos, size=size, ori=ori,
             startType=startType, startVal=startVal,
             stopType=stopType, stopVal=stopVal,
             startEstim=startEstim, durationEstim=durationEstim)
 
         self.type = 'Polygon'
-        self.url = "http://www.psychopy.org/builder/components/polygon.html"
+        self.url = "https://www.psychopy.org/builder/components/polygon.html"
         self.exp.requirePsychopyLibs(['visual'])
-        self.targets = ['PsychoPy', 'PsychoJS']
-        self.order = ['shape', 'nVertices']
+        self.order += ['shape', 'nVertices',  # Basic tab
+                      ]
+        self.order.insert(self.order.index("borderColor"), "lineColor")
         self.depends = [  # allows params to turn each other off/on
             {"dependsOn": "shape",  # must be param name
              "condition": "=='regular polygon...'",  # val to check for
              "param": "nVertices",  # param property to alter
-             "true": "enable",  # what to do with param if condition is True
-             "false": "disable",  # permitted: hide, show, enable, disable
-             }
+             "true": "show",  # what to do with param if condition is True
+             "false": "hide",  # permitted: hide, show, enable, disable
+             },
+            {"dependsOn": "shape",  # must be param name
+             "condition": "=='custom polygon...'",  # val to check for
+             "param": "vertices",  # param property to alter
+             "true": "show",  # what to do with param if condition is True
+             "false": "hide",  # permitted: hide, show, enable, disable
+             },
         ]
 
         # params
         msg = _translate("How many vertices in your regular polygon?")
         self.params['nVertices'] = Param(
-            nVertices, valType='int',
+            nVertices, valType='int', inputType="single", categ='Basic',
             updates='constant',
             allowedUpdates=['constant'],
             hint=msg,
             label=_localized['nVertices'])
 
+        msg = _translate("What are the vertices of your polygon? Should be an nx2 array or a list of [x, y] lists")
+        self.params['vertices'] = Param(
+            vertices, valType='list', inputType='single', categ='Basic',
+            updates='constant',
+            allowedUpdates=['constant', 'set every repeat', 'set every frame'],
+            hint=msg,
+            label=_translate("Vertices")
+        )
+        self.params['anchor'] = Param(
+            anchor, valType='str', inputType="choice", categ='Layout',
+            allowedVals=['center',
+                         'top-center',
+                         'bottom-center',
+                         'center-left',
+                         'center-right',
+                         'top-left',
+                         'top-right',
+                         'bottom-left',
+                         'bottom-right',
+                         ],
+            updates='constant',
+            hint=_translate("Which point on the stimulus should be anchored to its exact position?"),
+            label=_translate("Anchor"))
+
         msg = _translate("What shape is this? With 'regular polygon...' you "
-                         "can set number of vertices")
+                         "can set number of vertices and with 'custom "
+                         "polygon...' you can set vertices")
         self.params['shape'] = Param(
-            shape, valType='str',
-            allowedVals=["line", "triangle", "rectangle", "cross", "star",
-                         "regular polygon..."],
-            updates='constant',
-            allowedUpdates=['constant'],
-            hint=msg,
-            label=_localized['shape'])
+            shape, valType='str', inputType="choice", categ='Basic',
+            allowedVals=["line", "triangle", "rectangle", "circle", "cross", "star",
+                         "regular polygon...", "custom polygon..."],
+            hint=msg, direct=False,
+            label=_translate("Shape"))
 
-        msg = _translate("Choice of color space for the fill color "
-                         "(rgb, dkl, lms, hsv)")
-        self.params['fillColorSpace'] = Param(
-            fillColorSpace,
-            valType='str', allowedVals=['rgb', 'dkl', 'lms', 'hsv', 'rgb255'],
-            updates='constant',
-            hint=msg,
-            label=_localized['fillColorSpace'], categ='Advanced')
-
-        msg = _translate("Fill color of this shape; Right-click to bring up a"
-                         " color-picker (rgb only)")
-        self.params['fillColor'] = Param(
-            fillColor, valType='str', allowedTypes=[],
-            updates='constant',
-            allowedUpdates=['constant', 'set every repeat', 'set every frame'],
-            hint=msg,
-            label=_localized['fillColor'], categ='Advanced')
-
-        msg = _translate("Choice of color space for the fill color "
-                         "(rgb, dkl, lms, hsv)")
-        self.params['lineColorSpace'] = Param(
-            lineColorSpace, valType='str',
-            allowedVals=['rgb', 'dkl', 'lms', 'hsv'],
-            updates='constant',
-            hint=msg,
-            label=_localized['lineColorSpace'], categ='Advanced')
-
-        msg = _translate("Line color of this shape; Right-click to bring"
-                         " up a color-picker (rgb only)")
-        self.params['lineColor'] = Param(
-            lineColor, valType='str', allowedTypes=[],
-            updates='constant',
-            allowedUpdates=['constant', 'set every repeat', 'set every frame'],
-            hint=msg,
-            label=_localized['lineColor'], categ='Advanced')
+        self.params['lineColor'] = self.params['borderColor']
+        del self.params['borderColor']
 
         msg = _translate("Width of the shape's line (always in pixels - this"
                          " does NOT use 'units')")
         self.params['lineWidth'] = Param(
-            lineWidth, valType='code', allowedTypes=[],
+            lineWidth, valType='num', inputType="single", allowedTypes=[], categ='Appearance',
             updates='constant',
             allowedUpdates=['constant', 'set every repeat', 'set every frame'],
             hint=msg,
@@ -132,25 +128,19 @@ class PolygonComponent(BaseVisualComponent):
         msg = _translate(
             "How should the image be interpolated if/when rescaled")
         self.params['interpolate'] = Param(
-            interpolate, valType='str', allowedVals=['linear', 'nearest'],
-            updates='constant', allowedUpdates=[],
+            interpolate, valType='str', inputType="choice", allowedVals=['linear', 'nearest'], categ='Texture',
+            updates='constant', allowedUpdates=[], direct=False,
             hint=msg,
-            label=_localized['interpolate'], categ='Advanced')
+            label=_localized['interpolate'])
 
-        msg = _translate(
+
+        self.params['size'].hint = _translate(
             "Size of this stimulus [w,h]. Note that for a line only the "
             "first value is used, for triangle and rect the [w,h] is as "
             "expected,\n but for higher-order polygons it represents the "
             "[w,h] of the ellipse that the polygon sits on!! ")
-        self.params['size'] = Param(
-            size, valType='code', allowedTypes=[],
-            updates='constant',
-            allowedUpdates=['constant', 'set every repeat', 'set every frame'],
-            hint=msg,
-            label=_localized['size'])
 
         del self.params['color']
-        del self.params['colorSpace']
 
     def writeInitCode(self, buff):
         # do we need units code?
@@ -167,6 +157,8 @@ class PolygonComponent(BaseVisualComponent):
 
         if self.params['shape'] == 'regular polygon...':
             vertices = self.params['nVertices']
+        elif self.params['shape'] == 'custom polygon...':
+            vertices = self.params['vertices']
         else:
             vertices = self.params['shape']
         if vertices in ['line', '2']:
@@ -176,11 +168,15 @@ class PolygonComponent(BaseVisualComponent):
         elif vertices in ['triangle', '3']:
             code = ("%s = visual.ShapeStim(\n" % inits['name'] +
                     "    win=win, name='%s',%s\n" % (inits['name'], unitsStr) +
-                    "    vertices=[[-%(size)s[0]/2.0,-%(size)s[1]/2.0], [+%(size)s[0]/2.0,-%(size)s[1]/2.0], [0,%(size)s[1]/2.0]],\n" % inits)
+                    "    size=%(size)s, vertices='triangle',\n" % inits)
         elif vertices in ['rectangle', '4']:
             code = ("%s = visual.Rect(\n" % inits['name'] +
                     "    win=win, name='%s',%s\n" % (inits['name'], unitsStr) +
                     "    width=%(size)s[0], height=%(size)s[1],\n" % inits)
+        elif vertices in ['circle', '100']:
+            code = ("%s = visual.ShapeStim(\n" % inits['name'] +
+                    "    win=win, name='%s',%s\n" % (inits['name'], unitsStr) +
+                    "    size=%(size)s, vertices='circle',\n" % inits)
         elif vertices in ['star']:
             code = ("%s = visual.ShapeStim(\n" % inits['name'] +
                     "    win=win, name='%s', vertices='star7',%s\n" % (inits['name'], unitsStr) +
@@ -189,15 +185,19 @@ class PolygonComponent(BaseVisualComponent):
             code = ("%s = visual.ShapeStim(\n" % inits['name'] +
                     "    win=win, name='%s', vertices='cross',%s\n" % (inits['name'], unitsStr) +
                     "    size=%(size)s,\n" % inits)
-        else:
+        elif isinstance(vertices, (int, float, str)):
             code = ("%s = visual.Polygon(\n" % inits['name'] +
                     "    win=win, name='%s',%s\n" % (inits['name'], unitsStr) +
                     "    edges=%s," % str(inits['nVertices'].val) +
                     " size=%(size)s,\n" % inits)
+        else:
+            code = ("%s = visual.ShapeStim(\n" % inits['name'] +
+                    "    win=win, name='%s', vertices=%s,%s\n" % (inits['name'], vertices, unitsStr) +
+                    "    size=%(size)s,\n" % inits)
 
-        code += ("    ori=%(ori)s, pos=%(pos)s,\n"
-                 "    lineWidth=%(lineWidth)s, lineColor=%(lineColor)s, lineColorSpace=%(lineColorSpace)s,\n"
-                 "    fillColor=%(fillColor)s, fillColorSpace=%(fillColorSpace)s,\n"
+        code += ("    ori=%(ori)s, pos=%(pos)s, anchor=%(anchor)s,\n"
+                 "    lineWidth=%(lineWidth)s, "
+                 "    colorSpace=%(colorSpace)s,  lineColor=%(lineColor)s, fillColor=%(fillColor)s,\n"
                  "    opacity=%(opacity)s, " % inits)
 
         depth = -self.getPosInRoutine()
@@ -254,6 +254,10 @@ class PolygonComponent(BaseVisualComponent):
             code = ("{name} = new visual.Rect ({{\n"
                     "  win: psychoJS.window, name: '{name}', {unitsStr}\n"
                     "  width: {size}[0], height: {size}[1],\n")
+        elif vertices in ['circle', '100']:
+            code = ("{name} = new visual.Polygon({{\n"
+                    "  win: psychoJS.window, name: '{name}', {unitsStr}\n"
+                    "  edges: 100, size:{size},\n")
         elif vertices in ['star']:
             code = ("{name} = new visual.ShapeStim ({{\n"
                     "  win: psychoJS.window, name: '{name}', {unitsStr}\n"
