@@ -2,52 +2,47 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
-from builtins import super  # provides Py3-style super() using python-future
-
-from builtins import str
-from builtins import map
-from os import path
-import re
+from pathlib import Path
+from psychopy.tools.stringtools import getArgs
 from psychopy.experiment.components import BaseComponent, Param, _translate
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 
 __author__ = 'Jeremy Gray'
 
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'ratingscale.png')
-tooltip = _translate('Rating scale: obtain numerical or categorical '
-                     'responses')
-
 # only use _localized values for label values, nothing functional:
-_localized = {'visualAnalogScale': _translate('Visual analog scale'),
-              'categoryChoices': _translate('Category choices'),
-              'scaleDescription': _translate('Scale description'),
-              'low': _translate('Lowest value'),
-              'high': _translate('Highest value'),
-              'labels': _translate('Labels'),
-              'marker': _translate('Marker type'),
-              'markerStart': _translate('Marker start'),
-              'size': _translate('Size'),
-              'pos': _translate('Position [x,y]'),
-              'tickHeight': _translate('Tick height'),
-              'disappear': _translate('Disappear'),
-              'forceEndRoutine': _translate('Force end of Routine'),
-              'showAccept': _translate('Show accept'),
-              'singleClick': _translate('Single click'),
-              'storeHistory': _translate('Store history'),
-              'storeRating': _translate('Store rating'),
-              'storeRatingTime': _translate('Store rating time'),
-              'customize_everything': _translate('Customize everything :')}
+_localized.update({'visualAnalogScale': _translate('Visual analog scale'),
+                   'categoryChoices': _translate('Category choices'),
+                   'scaleDescription': _translate('Scale description'),
+                   'low': _translate('Lowest value'),
+                   'high': _translate('Highest value'),
+                   'labels': _translate('Labels'),
+                   'marker': _translate('Marker type'),
+                   'markerStart': _translate('Marker start'),
+                   'size': _translate('Size'),
+                   'pos': _translate('Position [x,y]'),
+                   'tickHeight': _translate('Tick height'),
+                   'disappear': _translate('Disappear'),
+                   'forceEndRoutine': _translate('Force end of Routine'),
+                   'showAccept': _translate('Show accept'),
+                   'singleClick': _translate('Single click'),
+                   'storeHistory': _translate('Store history'),
+                   'storeRating': _translate('Store rating'),
+                   'storeRatingTime': _translate('Store rating time'),
+                   'customize_everything': _translate('Customize everything :')})
 
 
 class RatingScaleComponent(BaseComponent):
     """A class for presenting a rating scale as a builder component
     """
     categories = ['Responses']
+    targets = ['PsychoPy']
+    iconFile = Path(__file__).parent / 'ratingscale.png'
+    tooltip = _translate('Rating scale: obtain numerical or categorical '
+                         'responses')
 
     def __init__(self, exp, parentName,
                  name='rating',
@@ -76,7 +71,7 @@ class RatingScaleComponent(BaseComponent):
             stopType=stopType, stopVal=stopVal,
             startEstim=startEstim, durationEstim=durationEstim)
         self.type = 'RatingScale'
-        self.url = "http://www.psychopy.org/builder/components/ratingscale.html"
+        self.url = "https://www.psychopy.org/builder/components/ratingscale.html"
         self.exp.requirePsychopyLibs(['visual', 'event'])
 
         # params
@@ -87,116 +82,106 @@ class RatingScaleComponent(BaseComponent):
         # normal params:
         # = the usual as inherited from BaseVisual plus:
         self.params['visualAnalogScale'] = Param(
-            visualAnalogScale, valType='bool', allowedTypes=[],
+            visualAnalogScale, valType='bool',  inputType="bool", allowedTypes=[], categ='Data',
             updates='constant', allowedUpdates=[],
             hint=_translate("Show a continuous visual analog scale; returns"
                             " 0.00 to 1.00; takes precedence over numeric "
                             "scale or categorical choices"),
             label=_localized['visualAnalogScale'])
         self.params['categoryChoices'] = Param(
-            categoryChoices, valType='str', allowedTypes=[],
+            categoryChoices, valType='list',  inputType="single", allowedTypes=[], categ='Basic',
             updates='constant', allowedUpdates=[],
             hint=_translate("A list of categories (non-numeric alternatives)"
                             " to present, space or comma-separated; these "
                             "take precedence over a numeric scale"),
             label=_localized['categoryChoices'])
         self.params['scaleDescription'] = Param(
-            scaleDescription, valType='str', allowedTypes=[],
+            scaleDescription, valType='str',  inputType="single", allowedTypes=[], categ='Basic',
             updates='constant', allowedUpdates=[],
             hint=_translate("Brief instructions, such as a description of "
                             "the scale numbers as seen by the subject."),
             label=_localized['scaleDescription'])
         self.params['low'] = Param(
-            low, valType='code', allowedTypes=[],
+            low, valType='num',  inputType="single", allowedTypes=[], categ='Data',
             updates='constant', allowedUpdates=[],
             hint=_translate("Lowest rating (low end of the scale); not"
                             " used for categories."),
             label=_localized['low'])
         self.params['high'] = Param(
-            high, valType='code', allowedTypes=[],
+            high, valType='num',  inputType="single", allowedTypes=[], categ='Data',
             updates='constant', allowedUpdates=[],
             hint=_translate("Highest rating (top end of the scale); "
                             "not used for categories."),
             label=_localized['high'])
         self.params['labels'] = Param(
-            labels, valType='str', allowedTypes=[],
+            labels, valType='list',  inputType="single", allowedTypes=[], categ='Data',
             updates='constant', allowedUpdates=[],  # categ="Advanced",
             hint=_translate("Labels for the ends of the scale, "
                             "separated by commas"),
             label=_localized['labels'])
         self.params['marker'] = Param(
-            marker, valType='str', allowedTypes=[],
+            marker, valType='str', inputType="choice", allowedTypes=[], categ='Interface',
+            allowedVals=['triangle', 'circle', 'glow', 'slider', 'hover'],
             updates='constant', allowedUpdates=[],  # categ="Advanced",
             hint=_translate("Style for the marker: triangle, circle, glow, "
                             "slider, hover"),
             label=_localized['marker'])
         self.params['markerStart'] = Param(
-            markerStart, valType='str', allowedTypes=[],
+            markerStart, valType='num',  inputType="single", allowedTypes=[], categ='Data',
             updates='constant', allowedUpdates=[],  # categ="Advanced",
             hint=_translate("initial position for the marker"),
             label=_localized['markerStart'])
 
         # advanced params:
         self.params['singleClick'] = Param(
-            singleClick, valType='bool', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
+            singleClick, valType='bool',  inputType="bool", allowedTypes=[],
+            updates='constant', allowedUpdates=[], categ="Interface",
             hint=_translate("Should clicking the line accept that rating "
                             "(without needing to confirm via 'accept')?"),
             label=_localized['singleClick'])
         self.params['disappear'] = Param(
-            disappear, valType='bool', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
+            disappear, valType='bool',  inputType="bool", allowedTypes=[],
+            updates='constant', allowedUpdates=[], categ="Interface",
             hint=_translate("Hide the scale when a rating has been accepted;"
                             " False to remain on-screen"),
             label=_localized['disappear'])
         self.params['showAccept'] = Param(
-            showAccept, valType='bool', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
+            showAccept, valType='bool',  inputType="bool", allowedTypes=[],
+            updates='constant', allowedUpdates=[], categ="Interface",
             hint=_translate("Should the accept button by visible?"),
             label=_localized['showAccept'])
         self.params['storeRating'] = Param(
-            storeRating, valType='bool', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
+            storeRating, valType='bool', inputType="bool", allowedTypes=[],
+            updates='constant', allowedUpdates=[], categ="Data",
             hint=_translate("store the rating"),
             label=_localized['storeRating'])
         self.params['storeRatingTime'] = Param(
-            storeRatingTime, valType='bool', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
+            storeRatingTime, valType='bool',  inputType="bool", allowedTypes=[],
+            updates='constant', allowedUpdates=[], categ="Data",
             hint=_translate("store the time taken to make the choice (in "
                             "seconds)"),
             label=_localized['storeRatingTime'])
         self.params['storeHistory'] = Param(
-            storeHistory, valType='bool', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
+            storeHistory, valType='bool', inputType="bool", allowedTypes=[],
+            updates='constant', allowedUpdates=[], categ="Data",
             hint=_translate("store the history of (selection, time)"),
             label=_localized['storeHistory'])
         self.params['forceEndRoutine'] = Param(
-            forceEndRoutine, valType='bool', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
+            forceEndRoutine, valType='bool',  inputType="bool", allowedTypes=[],
+            updates='constant', allowedUpdates=[], categ="Basic",
             hint=_translate("Should accepting a rating cause the end of the "
                             "routine (e.g. trial)?"),
             label=_localized['forceEndRoutine'])
-        self.params['size'] = Param(
-            size, valType='code', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
-            hint=_translate("Relative size on the screen; size > 1 is larger"
-                            " than default; size < 1 is smaller"),
-            label=_localized['size'])
         self.params['tickHeight'] = Param(
-            tickHeight, valType='str', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
+            tickHeight, valType='num', inputType="single", allowedTypes=[],
+            updates='constant', allowedUpdates=[], categ="Interface",
             hint=_translate("height of tick marks (1 is upward, 0 is hidden,"
                             " -1 is downward)"),
             label=_localized['tickHeight'])
-        self.params['pos'] = Param(
-            pos, valType='str', allowedTypes=[],
-            updates='constant', allowedUpdates=[], categ="Advanced",
-            hint=_translate("x,y position on the screen"),
-            label=_localized['pos'])
 
         # customization:
         self.params['customize_everything'] = Param(
-            customize_everything, valType='extendedStr', allowedTypes=[],
+            customize_everything, valType='code', inputType="multi", allowedTypes=[],
             updates='constant', allowedUpdates=[], categ="Custom",
             hint=_translate("Use this text to create the rating scale as you"
                             " would in a code component; overrides all"
@@ -204,21 +189,53 @@ class RatingScaleComponent(BaseComponent):
                             "forceEndRoutine, storeRatingTime, storeRating"),
             label=_localized['customize_everything'])
 
+        self.params['size'] = Param(size,
+            valType='list', inputType="single", categ='Layout',
+            updates='constant', allowedTypes=[],
+            allowedUpdates=['constant', 'set every repeat', 'set every frame'],
+            hint=_translate("Size of this stimulus (either a single value or "
+                         "x,y pair, e.g. 2.5, [1,2] "),
+            label=_localized['size'])
+
+        self.params['pos'] = Param(pos,
+            valType='list', inputType="single", categ='Layout',
+            updates='constant', allowedTypes=[],
+            allowedUpdates=['constant', 'set every repeat', 'set every frame'],
+            hint=_translate("Position of this stimulus (e.g. [1,2] )"),
+            label=_localized['pos'])
+
     def writeInitCode(self, buff):
         # build up an initialization string for RatingScale():
         _in = "%(name)s = visual.RatingScale(win=win, name='%(name)s'"
         init_str = _in % self.params
         if self.params['customize_everything'].val.strip() != '':
-            # clean it up a little, remove win=*, leading / trailing typos
-            orig = self.params['customize_everything'].val
-            custom = re.sub(r"[\\s,]*win=[^,]*,", '', orig)
-            init_str += ', ' + custom.lstrip('(, ').strip('), ')
+            # Add preamble
+            init_str += ", "
+            # Get arguments
+            args = getArgs(self.params['customize_everything'].val)
+            # Remove anything which would override win or name
+            if 'win' in args:
+                del args['win']
+            if 'name' in args:
+                del args['name']
+            # Convert each argument to a string
+            argList = [f"{key}={val}" for key, val in args.items()]
+            # Recombine these arguments into a (now santized) string
+            init_str += ", ".join(argList)
         else:
             if self.params['marker'].val:
                 init_str += ', marker=%s' % repr(self.params['marker'].val)
                 if self.params['marker'].val == 'glow':
                     init_str += ', markerExpansion=5'
-            init_str += ", size=%s" % self.params['size']
+
+            s = str(self.params['size'].val)
+            s = s.lstrip('([ ').strip(')] ')
+            try:
+                size = list(map(float, s.split(','))) * 2
+                init_str += ", size=%s" % size[0]
+            except Exception:
+                pass  # size = None
+
             s = str(self.params['pos'].val)
             s = s.lstrip('([ ').strip(')] ')
             try:
