@@ -3,31 +3,28 @@
 
 """
 Part of the PsychoPy library
-Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 Distributed under the terms of the GNU General Public License (GPL).
 """
 
-from __future__ import absolute_import, print_function
-from builtins import super  # provides Py3-style super() using python-future
-
-from os import path
+from pathlib import Path
 from psychopy.experiment.components import BaseComponent, Param, getInitVals, _translate
 from psychopy.sound._base import knownNoteNames
-
-# the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'sound.png')
-tooltip = _translate('Sound: play recorded files or generated sounds',)
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 
 # only use _localized values for label values, nothing functional:
-_localized = {'sound': _translate('Sound'),
-              'volume': _translate('Volume'),
-              'syncScreenRefresh': _translate('sync RT with screen')}
+_localized.update({'sound': _translate('Sound'),
+                   'volume': _translate('Volume'),
+                   'syncScreenRefresh': _translate('Sync Start With Screen')})
 
 
 class SoundComponent(BaseComponent):
     """An event class for presenting sound stimuli"""
     categories = ['Stimuli']
+    targets = ['PsychoPy', 'PsychoJS']
+    iconFile = Path(__file__).parent / 'sound.png'
+    tooltip = _translate('Sound: play recorded files or generated sounds', )
 
     def __init__(self, exp, parentName, name='sound_1', sound='A', volume=1,
                  startType='time (s)', startVal='0.0',
@@ -40,10 +37,12 @@ class SoundComponent(BaseComponent):
             stopType=stopType, stopVal=stopVal,
             startEstim=startEstim, durationEstim=durationEstim)
         self.type = 'Sound'
-        self.url = "http://www.psychopy.org/builder/components/sound.html"
+        self.url = "https://www.psychopy.org/builder/components/sound.html"
         self.exp.requirePsychopyLibs(['sound'])
-        self.targets = ['PsychoPy', 'PsychoJS']
-        self.order = ["sound", "volume"]
+        self.order += [
+            "sound",  # Basic tab
+            "volume", "hammingWindow",  # Playback tab
+        ]
         # params
         self.params['stopType'].allowedVals = ['duration (s)']
         self.params['stopType'].hint = _translate('The maximum duration of a'
@@ -55,13 +54,13 @@ class SoundComponent(BaseComponent):
         hnt = _translate("A sound can be a note name (e.g. A or Bf), a number"
                          " to specify Hz (e.g. 440) or a filename")
         self.params['sound'] = Param(
-            sound, valType='str', allowedTypes=[], updates='constant',
+            sound, valType='str', inputType="file", allowedTypes=[], updates='constant', categ='Basic',
             allowedUpdates=['constant', 'set every repeat'],
             hint=hnt,
             label=_localized['sound'])
         _allowed = ['constant', 'set every repeat', 'set every frame']
         self.params['volume'] = Param(
-            volume, valType='code', allowedTypes=[], updates='constant',
+            volume, valType='num', inputType="single", allowedTypes=[], updates='constant', categ='Playback',
             allowedUpdates=_allowed[:],  # use a copy
             hint=_translate("The volume (in range 0 to 1)"),
             label=_localized["volume"])
@@ -69,12 +68,12 @@ class SoundComponent(BaseComponent):
             "A reaction time to a sound stimulus should be based on when "
             "the screen flipped")
         self.params['syncScreenRefresh'] = Param(
-            syncScreenRefresh, valType='bool',
+            syncScreenRefresh, valType='bool', inputType="bool", categ='Basic',
             updates='constant',
             hint=msg,
             label=_localized['syncScreenRefresh'])
         self.params['hamming'] = Param(
-            True, valType='bool', updates='constant',
+            True, valType='bool', inputType="bool", updates='constant', categ='Playback',
             hint=_translate(
                   "For tones we can apply a Hamming window to prevent 'clicks' that "
                   "are caused by a sudden onset. This delays onset by roughly 1ms."),
@@ -194,10 +193,10 @@ class SoundComponent(BaseComponent):
         else:
             # sounds with stop values
             self.writeStopTestCodeJS(buff)
-            code = ("if (%(stopVal)s > 0.5) {"
+            code = ("if (%(stopVal)s > 0.5) {\n"
                     "  %(name)s.stop();  // stop the sound (if longer than duration)\n"
-                    "  %(name)s.status = PsychoJS.Status.FINISHED;\n"
-                    "}\n")
+                    "}\n"
+                    "%(name)s.status = PsychoJS.Status.FINISHED;\n")
             buff.writeIndentedLines(code % self.params)
             # because of the 'if' statement of the time test
             buff.setIndentLevel(-1, relative=True)
