@@ -3,24 +3,20 @@
 
 """
 Part of the PsychoPy library
-Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 Distributed under the terms of the GNU General Public License (GPL).
 """
 
-from __future__ import absolute_import, print_function
-
-from builtins import str
 from os import path
+from pathlib import Path
 from psychopy.experiment.components import BaseComponent, Param, _translate
+from psychopy.localization import _localized as __localized
+_localized = __localized.copy()
 
 __author__ = 'Jon Peirce'
 
 # the absolute path to the folder containing this path
-thisFolder = path.abspath(path.dirname(__file__))
-iconFile = path.join(thisFolder, 'static.png')
-tooltip = _translate('Static: Static screen period (e.g. an ISI). '
-                     'Useful for pre-loading stimuli.')
-_localized = {'Custom code': _translate('Custom code')}
+_localized.update({'Custom code': _translate('Custom code')})
 
 
 class StaticComponent(BaseComponent):
@@ -31,6 +27,10 @@ class StaticComponent(BaseComponent):
     # override the categories property below
     # an attribute of the class, determines the section in the components panel
     categories = ['Custom']
+    targets = ['PsychoPy', 'PsychoJS']
+    iconFile = Path(__file__).parent / 'static.png'
+    tooltip = _translate('Static: Static screen period (e.g. an ISI). '
+                         'Useful for pre-loading stimuli.')
 
     def __init__(self, exp, parentName, name='ISI',
                  startType='time (s)', startVal=0.0,
@@ -39,42 +39,12 @@ class StaticComponent(BaseComponent):
         BaseComponent.__init__(self, exp, parentName, name=name)
         self.updatesList = []  # a list of dicts {compParams, fieldName}
         self.type = 'Static'
-        self.url = "http://www.psychopy.org/builder/components/static.html"
+        self.url = "https://www.psychopy.org/builder/components/static.html"
         hnt = _translate(
             "Custom code to be run during the static period (after updates)")
-        self.params['code'] = Param("", valType='code',
+        self.params['code'] = Param("", valType='code', inputType="multi", categ='Custom',
                                     hint=hnt,
                                     label=_localized['Custom code'])
-        self.order = ['name']  # make name come first (others don't matter)
-
-        hnt = _translate("How do you want to define your start point?")
-        self.params['startType'] = Param(startType, valType='str',
-                                         allowedVals=['time (s)', 'frame N'],
-                                         hint=hnt)
-        hnt = _translate("How do you want to define your end point?")
-        _allow = ['duration (s)', 'duration (frames)', 'time (s)', 'frame N']
-        self.params['stopType'] = Param(stopType, valType='str',
-                                        allowedVals=_allow,  # copy not needed
-                                        hint=hnt)
-        hnt = _translate("When does the component start?")
-        self.params['startVal'] = Param(startVal, valType='code',
-                                        allowedTypes=[],
-                                        hint=hnt)
-        hnt = _translate("When does the component end? (blank is endless)")
-        self.params['stopVal'] = Param(stopVal, valType='code',
-                                       allowedTypes=[],
-                                       updates='constant', allowedUpdates=[],
-                                       hint=hnt)
-        hnt = _translate("(Optional) expected start (s), purely for "
-                         "representing in the timeline")
-        self.params['startEstim'] = Param(startEstim, valType='code',
-                                          allowedTypes=[],
-                                          hint=hnt)
-        hnt = _translate("(Optional) expected duration (s), purely for "
-                         "representing in the timeline")
-        self.params['durationEstim'] = Param(durationEstim, valType='code',
-                                             allowedTypes=[],
-                                             hint=hnt)
 
     def addComponentUpdate(self, routine, compName, fieldName):
         self.updatesList.append({'compName': compName,
@@ -85,21 +55,99 @@ class StaticComponent(BaseComponent):
         # have to do this in a loop rather than a simple remove
         target = {'compName': compName, 'fieldName': fieldName,
                   'routine': routine}
+
         for item in self.updatesList:
-            if item == target:
+            # check if dict has the same fields
+            for key in ('compName', 'fieldName', 'routine'):
+                if item[key] != target[key]:
+                    break
+            else:
                 self.updatesList.remove(item)
 
+            # NB - should we break out of it here if an item is found?
+
     def writeInitCode(self, buff):
         code = ("%(name)s = clock.StaticPeriod(win=win, "
                 "screenHz=expInfo['frameRate'], name='%(name)s')\n")
         buff.writeIndented(code % self.params)
 
+    def writeInitCodeJS(self, buff):
+        code = (
+            "%(name)s = new core.MinimalStim({\n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
+        buff.setIndentLevel(+1, relative=True)
+        code = (
+                "name: \"%(name)s\", \n"
+                "win: psychoJS.window,\n"
+                "autoDraw: false, \n"
+                "autoLog: true, \n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
+        buff.setIndentLevel(-1, relative=True)
+        code = (
+            "});\n"
+        )
+        buff.writeIndented(code % self.params)
+
     def writeFrameCode(self, buff):
         self.writeStartTestCode(buff)
         # to get out of the if statement
         buff.setIndentLevel(-1, relative=True)
         self.writeStopTestCode(buff)
 
+    def writeFrameCodeJS(self, buff):
+        # Start test
+        self.writeStartTestCodeJS(buff)
+        self.writeParamUpdates(buff, target="PsychoJS")
+        buff.setIndentLevel(-1, relative=True)
+        buff.writeIndentedLines("}\n")
+
+        # Stop test, with stop actions
+        self.writeStopTestCodeJS(buff)
+        for update in self.updatesList:
+
+            # Get params for update
+            compName = update['compName']
+            fieldName = update['fieldName']
+            # routine = self.exp.routines[update['routine']]
+            if hasattr(compName, 'params'):
+                prms = compName.params  # it's already a compon so get params
+            else:
+                # it's a name so get compon and then get params
+                prms = self.exp.getComponentFromName(str(compName)).params
+            if prms[fieldName].valType == "file":
+                # Check resource manager status
+                code = (
+                    f"if (psychoJS.serverManager.getResourceStatus(%({fieldName})s) === core.ServerManager.ResourceStatus.DOWNLOADED) {{\n"
+                )
+                buff.writeIndentedLines(code % prms)
+                # Print confirmation
+                buff.setIndentLevel(+1, relative=True)
+                code = (
+                    "console.log('finished downloading resources specified by component %(name)s');\n"
+                )
+                buff.writeIndentedLines(code % self.params)
+                # else...
+                buff.setIndentLevel(-1, relative=True)
+                code = (
+                    "} else {\n"
+                )
+                buff.writeIndentedLines(code % self.params)
+                # Print warning if not downloaded
+                buff.setIndentLevel(+1, relative=True)
+                code = (
+                    "console.log('resource specified in %(name)s took longer than expected to download');\n"
+                )
+                buff.writeIndentedLines(code % self.params)
+                buff.setIndentLevel(-1, relative=True)
+                buff.writeIndentedLines("}\n")
+        # Escape stop code indent
+        buff.setIndentLevel(-1, relative=True)
+        buff.writeIndentedLines("}\n")
+
     def writeStartTestCode(self, buff):
         """This will be executed as the final component in the routine
         """
@@ -151,7 +199,7 @@ class StaticComponent(BaseComponent):
 
         # pass  # the clock.StaticPeriod class handles its own stopping
 
-    def writeParamUpdates(self, buff, updateType=None, paramNames=None):
+    def writeParamUpdates(self, buff, updateType=None, paramNames=None, target="PsychoPy"):
         """Write updates. Unlike most components, which us this method
         to update themselves, the Static Component uses this to update
         *other* components
@@ -159,30 +207,53 @@ class StaticComponent(BaseComponent):
         if updateType == 'set every repeat':
             return  # the static component doesn't need to change itself
         if len(self.updatesList):
-            code = "# updating other components during *%s*\n"
+            # Comment to mark start of updates
+            if target == "PsychoJS":
+                code = "// Updating other components during *%s*\n"
+            else:
+                code = "# Updating other components during *%s*\n"
             buff.writeIndented(code % self.params['name'])
+            # Do updates
             for update in self.updatesList:
                 # update = {'compName':compName,'fieldName':fieldName,
                 #    'routine':routine}
                 compName = update['compName']
                 fieldName = update['fieldName']
-                routine = self.exp.routines[update['routine']]
+                # routine = self.exp.routines[update['routine']]
                 if hasattr(compName, 'params'):
                     prms = compName.params  # it's already a compon so get params
                 else:
                     # it's a name so get compon and then get params
                     prms = self.exp.getComponentFromName(str(compName)).params
+                # If in JS, prepare resources
+                if target == "PsychoJS" and prms[fieldName].valType == "file":
+                    # Do resource manager stuff
+                    code = (
+                        f"console.log('register and start downloading resources specified by component %(name)s');\n"
+                        f"await psychoJS.serverManager.prepareResources(%({fieldName})s);\n"
+                    )
+                    buff.writeIndentedLines(code % prms)
+                # Set values
                 self.writeParamUpdate(buff, compName=compName,
                                       paramName=fieldName,
                                       val=prms[fieldName],
                                       updateType=prms[fieldName].updates,
                                       params=prms)
-            code = "# component updates done\n"
+            # Comment to mark end of updates
+            if target == "PsychoJS":
+                code = "// Component updates done\n"
+            else:
+                code = "# Component updates done\n"
+            buff.writeIndentedLines(code)
 
             # Write custom code
             if self.params['code']:
-                code += ("# Adding custom code for {name}\n"
-                         "{code}\n".format(name=self.params['name'],
-                                           code=self.params['code']))
-
-            buff.writeIndentedLines(code)
+                # Comment to mark start of custom code
+                if target == "PsychoJS":
+                    code = "// Adding custom code for %(name)s\n"
+                else:
+                    code = "# Adding custom code for %(name)s\n"
+                buff.writeIndentedLines(code % self.params)
+                # Write custom code
+                code = "%(code)s\n"
+                buff.writeIndentedLines(code % self.params)
