@@ -2,16 +2,16 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """Describes the Flow of an experiment
 """
 
-from __future__ import absolute_import, print_function
-from past.builtins import basestring
+from xml.etree.ElementTree import Element
 
-from psychopy.experiment.routine import Routine
+from psychopy.experiment import getAllStandaloneRoutines
+from psychopy.experiment.routines._base import Routine, BaseStandaloneRoutine
 from psychopy.experiment.loops import LoopTerminator, LoopInitiator
 
 
@@ -54,6 +54,22 @@ class Flow(list):
     def __repr__(self):
         return "psychopy.experiment.Flow(%s)" % (str(list(self)))
 
+    @property
+    def _xml(self):
+        # Make root element
+        element = Element("Flow")
+        # Add an element for every Routine, Loop Initiator, Loop Terminator
+        for item in self:
+            sub = item._xml
+            if isinstance(item, Routine) or isinstance(item, BaseStandaloneRoutine):
+                # Remove all sub elements (we only need its name)
+                comps = [comp for comp in sub]
+                for comp in comps:
+                    sub.remove(comp)
+            element.append(sub)
+
+        return element
+
     def addLoop(self, loop, startPos, endPos):
         """Adds initiator and terminator objects for the loop
         into the Flow list"""
@@ -89,10 +105,10 @@ class Flow(list):
                         toBeRemoved.append(comp)
             for comp in toBeRemoved:
                 self.remove(comp)
-        elif component.getType() == 'Routine':
+        elif component.getType() in ['Routine'] + list(getAllStandaloneRoutines()):
             if id is None:
                 # a Routine may come up multiple times - remove them all
-                # self.remove(component)  # cant do this - two empty routines
+                # self.remove(component)  # can't do this - two empty routines
                 # (with diff names) look the same to list comparison
                 toBeRemoved = []
                 for id, compInFlow in enumerate(self):
@@ -138,7 +154,7 @@ class Flow(list):
                     if (field.label.lower() in ['text', 'customize'] or
                             not field.valType in ('str', 'code')):
                         continue
-                    if (isinstance(field.val, basestring) and
+                    if (isinstance(field.val, str) and
                             field.val != field.val.strip()):
                         trailingWhitespace.append(
                             (field.val, key, component, entry))
@@ -215,6 +231,8 @@ class Flow(list):
         for entry in self:
             self._currentRoutine = entry
             entry.writeMainCode(script)
+            if hasattr(entry, "writeRoutineEndCode"):
+                entry.writeRoutineEndCode(script)
         # tear-down code (very few components need this)
         for entry in self:
             self._currentRoutine = entry
@@ -261,7 +279,7 @@ class Flow(list):
             if not loopStack:  # if not currently in a loop
                 if thisEntry.getType() == 'LoopInitiator':
                     code = ("const {name}LoopScheduler = new Scheduler(psychoJS);\n"
-                            "flowScheduler.add({name}LoopBegin, {name}LoopScheduler);\n"
+                            "flowScheduler.add({name}LoopBegin({name}LoopScheduler));\n"
                             "flowScheduler.add({name}LoopScheduler);\n"
                             "flowScheduler.add({name}LoopEnd);\n"
                             .format(name=thisEntry.loop.params['name'].val))
