@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """Experiment classes:
@@ -16,19 +16,15 @@ The code that writes out a *_lastrun.py experiment file is (in order):
     settings.SettingsComponent.writeEndCode()
 """
 
-from __future__ import absolute_import, print_function
-from builtins import object
-# from future import standard_library
+from copy import deepcopy
+from xml.etree.ElementTree import Element
 
 from psychopy.experiment import getInitVals
 from psychopy.localization import _localized, _translate
 from psychopy.experiment.params import Param
 from .components import getInitVals, getAllComponents
 
-# standard_library.install_aliases()
-
-
-class TrialHandler(object):
+class TrialHandler():
     """A looping experimental control object
             (e.g. generating a psychopy TrialHandler or StairHandler).
             """
@@ -55,33 +51,33 @@ class TrialHandler(object):
         self.order = ['name']  # make name come first (others don't matter)
         self.params = {}
         self.params['name'] = Param(
-            name, valType='code', updates=None, allowedUpdates=None,
+            name, valType='code', inputType="single", updates=None, allowedUpdates=None,
             label=_localized['Name'],
             hint=_translate("Name of this loop"))
         self.params['nReps'] = Param(
-            nReps, valType='code', updates=None, allowedUpdates=None,
+            nReps, valType='num', inputType="spin", updates=None, allowedUpdates=None,
             label=_localized['nReps'],
             hint=_translate("Number of repeats (for each condition)"))
         self.params['conditions'] = Param(
-            list(conditions), valType='str',
+            list(conditions), valType='str', inputType="single",
             updates=None, allowedUpdates=None,
             label=_localized['conditions'],
             hint=_translate("A list of dictionaries describing the "
                             "parameters in each condition"))
         self.params['conditionsFile'] = Param(
-            conditionsFile, valType='str', updates=None, allowedUpdates=None,
+            conditionsFile, valType='file', inputType="table", updates=None, allowedUpdates=None,
             label=_localized['conditions'],
             hint=_translate("Name of a file specifying the parameters for "
                             "each condition (.csv, .xlsx, or .pkl). Browse "
                             "to select a file. Right-click to preview file "
                             "contents, or create a new file."))
         self.params['endPoints'] = Param(
-            list(endPoints), valType='num', updates=None, allowedUpdates=None,
+            list(endPoints), valType='num', inputType="single", updates=None, allowedUpdates=None,
             label=_localized['endPoints'],
             hint=_translate("The start and end of the loop (see flow "
                             "timeline)"))
         self.params['Selected rows'] = Param(
-            selectedRows, valType='str',
+            selectedRows, valType='str', inputType="single",
             updates=None, allowedUpdates=None,
             label=_localized['Selected rows'],
             hint=_translate("Select just a subset of rows from your condition"
@@ -89,21 +85,21 @@ class TrialHandler(object):
                             "0:5, 5:-1"))
         # NB staircase is added for the sake of the loop properties dialog:
         self.params['loopType'] = Param(
-            loopType, valType='str',
+            loopType, valType='str', inputType="choice",
             allowedVals=['random', 'sequential', 'fullRandom',
                          'staircase', 'interleaved staircases'],
             label=_localized['loopType'],
             hint=_translate("How should the next condition value(s) be "
                             "chosen?"))
         self.params['random seed'] = Param(
-            randomSeed, valType='code', updates=None, allowedUpdates=None,
+            randomSeed, valType='code', inputType="single", updates=None, allowedUpdates=None,
             label=_localized['random seed'],
             hint=_translate("To have a fixed random sequence provide an "
                             "integer of your choosing here. Leave blank to "
                             "have a new random sequence on each run of the "
                             "experiment."))
         self.params['isTrials'] = Param(
-            isTrials, valType='bool', updates=None, allowedUpdates=None,
+            isTrials, valType='bool', inputType="bool", updates=None, allowedUpdates=None,
             label=_localized["Is trials"],
             hint=_translate("Indicates that this loop generates TRIALS, "
                             "rather than BLOCKS of trials or stimuli within "
@@ -202,17 +198,26 @@ class TrialHandler(object):
         elif isinstance(nReps, str):
             nReps = nReps.strip("$")
 
-        code = ("\nfunction {loopName}LoopBegin({loopName}LoopScheduler) {{\n"
-                "  // set up handler to look after randomisation of conditions etc\n"
-                "  {loopName} = new TrialHandler({{\n"
-                "    psychoJS: psychoJS,\n"
-                "    nReps: {nReps}, method: TrialHandler.Method.{loopType},\n"
-                "    extraInfo: expInfo, originPath: undefined,\n"
-                "    trialList: {trialList},\n"
-                "    seed: {seed}, name: '{loopName}'\n"
-                "  }});\n"
-                "  psychoJS.experiment.addLoop({loopName}); // add the loop to the experiment\n"
-                "  currentLoop = {loopName};  // we're now the current loop\n"
+        code = ("\nfunction {loopName}LoopBegin({loopName}LoopScheduler, snapshot) {{\n"
+                "  return async function() {{\n"
+                .format(loopName=self.params['name'],
+                        loopType=(self.params['loopType'].val).upper(),
+                        nReps=nReps,
+                        trialList=trialList,
+                        seed=seed))
+        buff.writeIndentedLines(code)
+        buff.setIndentLevel(2, relative=True)
+        code = ("TrialHandler.fromSnapshot(snapshot); // update internal variables (.thisN etc) of the loop\n\n"
+                "// set up handler to look after randomisation of conditions etc\n"
+                "{loopName} = new TrialHandler({{\n"
+                "  psychoJS: psychoJS,\n"
+                "  nReps: {nReps}, method: TrialHandler.Method.{loopType},\n"
+                "  extraInfo: expInfo, originPath: undefined,\n"
+                "  trialList: {trialList},\n"
+                "  seed: {seed}, name: '{loopName}'\n"
+                "}});\n"
+                "psychoJS.experiment.addLoop({loopName}); // add the loop to the experiment\n"
+                "currentLoop = {loopName};  // we're now the current loop\n"
                 .format(loopName=self.params['name'],
                         loopType=(self.params['loopType'].val).upper(),
                         nReps=nReps,
@@ -222,15 +227,15 @@ class TrialHandler(object):
         
         # for the scheduler
         if modular:
-            code = ("\n  // Schedule all the trials in the trialList:\n"
-                    "  for (const {thisName} of {loopName}) {{\n"
-                    "    const snapshot = {loopName}.getSnapshot();\n"
-                    "    {loopName}LoopScheduler.add(importConditions(snapshot));\n")
+            code = ("\n// Schedule all the trials in the trialList:\n"
+                    "for (const {thisName} of {loopName}) {{\n"
+                    "  const snapshot = {loopName}.getSnapshot();\n"
+                    "  {loopName}LoopScheduler.add(importConditions(snapshot));\n")
         else:
-            code = ("\n  // Schedule all the trials in the trialList:\n"
-                    "  {loopName}.forEach(function() {{\n"
-                    "    const snapshot = {loopName}.getSnapshot();\n\n"
-                    "    {loopName}LoopScheduler.add(importConditions(snapshot));\n")
+            code = ("\n// Schedule all the trials in the trialList:\n"
+                    "{loopName}.forEach(function() {{\n"
+                    "  const snapshot = {loopName}.getSnapshot();\n\n"
+                    "  {loopName}LoopScheduler.add(importConditions(snapshot));\n")
         buff.writeIndentedLines(code.format(loopName=self.params['name'],
                                             thisName=self.thisName))
         # then we need to include begin, eachFrame and end code for each entry within that loop
@@ -240,28 +245,32 @@ class TrialHandler(object):
         for thisChild in thisLoop:
             if thisChild.getType() == 'Routine':
                 code += (
-                    "    {loopName}LoopScheduler.add({childName}RoutineBegin(snapshot));\n"
-                    "    {loopName}LoopScheduler.add({childName}RoutineEachFrame(snapshot));\n"
-                    "    {loopName}LoopScheduler.add({childName}RoutineEnd(snapshot));\n"
+                    "  {loopName}LoopScheduler.add({childName}RoutineBegin(snapshot));\n"
+                    "  {loopName}LoopScheduler.add({childName}RoutineEachFrame());\n"
+                    "  {loopName}LoopScheduler.add({childName}RoutineEnd());\n"
                     .format(childName=thisChild.params['name'],
                             loopName=self.params['name'])
                     )
             else:  # for a LoopInitiator
                 code += (
-                    "    const {childName}LoopScheduler = new Scheduler(psychoJS);\n"
-                    "    {loopName}LoopScheduler.add({childName}LoopBegin, {childName}LoopScheduler);\n"
-                    "    {loopName}LoopScheduler.add({childName}LoopScheduler);\n"
-                    "    {loopName}LoopScheduler.add({childName}LoopEnd);\n"
+                    "  const {childName}LoopScheduler = new Scheduler(psychoJS);\n"
+                    "  {loopName}LoopScheduler.add({childName}LoopBegin({childName}LoopScheduler, snapshot));\n"
+                    "  {loopName}LoopScheduler.add({childName}LoopScheduler);\n"
+                    "  {loopName}LoopScheduler.add({childName}LoopEnd);\n"
                     .format(childName=thisChild.params['name'],
                             loopName=self.params['name'])
                     )
 
-        code += "    {loopName}LoopScheduler.add(endLoopIteration({loopName}LoopScheduler, snapshot));\n"
-        code += "  }}%s\n" % ([');', ''][modular])
+        code += "  {loopName}LoopScheduler.add(endLoopIteration({loopName}LoopScheduler, snapshot));\n"
+        code += "}}%s\n" % ([');', ''][modular])
         code += ("\n"
-                 "  return Scheduler.Event.NEXT;\n"
-                 "}}\n")
+                 "return Scheduler.Event.NEXT;\n")
         buff.writeIndentedLines(code.format(loopName=self.params['name']))
+        buff.setIndentLevel(-2, relative=True)
+        buff.writeIndentedLines(
+                 "  }\n"
+                 "}\n"
+        )
 
     def writeLoopEndCode(self, buff):
         # Just within the loop advance data line if loop is whole trials
@@ -303,7 +312,7 @@ class TrialHandler(object):
 
     def writeLoopEndCodeJS(self, buff):
         # Just within the loop advance data line if loop is whole trials
-        code = ("\nfunction {funName}LoopEnd() {{\n"
+        code = ("\nasync function {funName}LoopEnd() {{\n"
                 "  psychoJS.experiment.removeLoop({name});\n\n".format(funName=self.params['name'].val,
                                                                        name=self.params['name']))
         code += ("  return Scheduler.Event.NEXT;\n"
@@ -313,8 +322,12 @@ class TrialHandler(object):
     def getType(self):
         return 'TrialHandler'
 
+    @property
+    def name(self):
+        return self.params['name'].val
 
-class StairHandler(object):
+
+class StairHandler:
     """A staircase experimental control object.
     """
 
@@ -339,68 +352,72 @@ class StairHandler(object):
             hint=_translate("Name of this loop"),
             label=_localized['Name'])
         self.params['nReps'] = Param(
-            nReps, valType='code',
+            nReps, valType='num', inputType='spin',
             label=_localized['nReps'],
             hint=_translate("(Minimum) number of trials in the staircase"))
         self.params['start value'] = Param(
-            startVal, valType='code',
+            startVal, valType='num', inputType='single',
             label=_localized['start value'],
             hint=_translate("The initial value of the parameter"))
         self.params['max value'] = Param(
-            maxVal, valType='code',
+            maxVal, valType='num', inputType='single',
             label=_localized['max value'],
             hint=_translate("The maximum value the parameter can take"))
         self.params['min value'] = Param(
-            minVal, valType='code',
+            minVal, valType='num', inputType='single',
             label=_localized['min value'],
             hint=_translate("The minimum value the parameter can take"))
         self.params['step sizes'] = Param(
-            stepSizes, valType='code',
+            stepSizes, valType='list', inputType='single',
             label=_localized['step sizes'],
             hint=_translate("The size of the jump at each step (can change"
                             " on each 'reversal')"))
         self.params['step type'] = Param(
-            stepType, valType='str', allowedVals=['lin', 'log', 'db'],
+            stepType, valType='str', inputType='choice', allowedVals=['lin', 'log', 'db'],
             label=_localized['step type'],
             hint=_translate("The units of the step size (e.g. 'linear' will"
                             " add/subtract that value each step, whereas "
                             "'log' will ad that many log units)"))
         self.params['N up'] = Param(
-            nUp, valType='code',
+            nUp, valType='num', inputType='spin',
             label=_localized['N up'],
             hint=_translate("The number of 'incorrect' answers before the "
                             "value goes up"))
         self.params['N down'] = Param(
-            nDown, valType='code',
+            nDown, valType='num', inputType='spin',
             label=_localized['N down'],
             hint=_translate("The number of 'correct' answers before the "
                             "value goes down"))
         self.params['N reversals'] = Param(
-            nReversals, valType='code',
+            nReversals, valType='num', inputType='spin',
             label=_localized['N reversals'],
             hint=_translate("Minimum number of times the staircase must "
                             "change direction before ending"))
         # these two are really just for making the dialog easier (they won't
         # be used to generate code)
         self.params['loopType'] = Param(
-            'staircase', valType='str',
+            'staircase', valType='str', inputType='choice',
             allowedVals=['random', 'sequential', 'fullRandom', 'staircase',
                          'interleaved staircases'],
             label=_localized['loopType'],
             hint=_translate("How should the next trial value(s) be chosen?"))
         # NB this is added for the sake of the loop properties dialog
         self.params['endPoints'] = Param(
-            list(endPoints), valType='num',
+            list(endPoints), valType='num', inputType='spin',
             label=_localized['endPoints'],
             hint=_translate('Where to loop from and to (see values currently'
                             ' shown in the flow view)'))
         self.params['isTrials'] = Param(
-            isTrials, valType='bool', updates=None, allowedUpdates=None,
+            isTrials, valType='bool', inputType='bool', updates=None, allowedUpdates=None,
             label=_localized["Is trials"],
             hint=_translate("Indicates that this loop generates TRIALS, "
                             "rather than BLOCKS of trials or stimuli within"
                             " a trial. It alters how data files are output"))
 
+    @property
+    def name(self):
+        return self.params['name'].val
+
     def writeInitCode(self, buff):
         # not needed - initialise the staircase only when needed
         pass
@@ -456,7 +473,7 @@ class StairHandler(object):
         return 'StairHandler'
 
 
-class MultiStairHandler(object):
+class MultiStairHandler:
     """To handle multiple interleaved staircases
     """
 
@@ -476,55 +493,59 @@ class MultiStairHandler(object):
         self.order = ['name']  # make name come first
         self.params = {}
         self.params['name'] = Param(
-            name, valType='code',
+            name, valType='code', inputType='single',
             label=_localized['Name'],
             hint=_translate("Name of this loop"))
         self.params['nReps'] = Param(
-            nReps, valType='code',
+            nReps, valType='num', inputType='spin',
             label=_localized['nReps'],
             hint=_translate("(Minimum) number of trials in *each* staircase"))
         self.params['stairType'] = Param(
-            stairType, valType='str',
-            allowedVals=['simple', 'QUEST', 'quest', 'questplus'],
+            stairType, valType='str', inputType='choice',
+            allowedVals=['simple', 'QUEST'],
             label=_localized['stairType'],
             hint=_translate("How to select the next staircase to run"))
         self.params['switchMethod'] = Param(
-            switchStairs, valType='str',
+            switchStairs, valType='str', inputType='choice',
             allowedVals=['random', 'sequential', 'fullRandom'],
             label=_localized['switchMethod'],
             hint=_translate("How to select the next staircase to run"))
         # these two are really just for making the dialog easier (they won't
         # be used to generate code)
         self.params['loopType'] = Param(
-            'staircase', valType='str',
+            'staircase', valType='str', inputType='choice',
             allowedVals=['random', 'sequential', 'fullRandom', 'staircase',
                          'interleaved staircases'],
             label=_localized['loopType'],
             hint=_translate("How should the next trial value(s) be chosen?"))
         self.params['endPoints'] = Param(
-            list(endPoints), valType='num',
+            list(endPoints), valType='num', inputType='spin',
             label=_localized['endPoints'],
             hint=_translate('Where to loop from and to (see values currently'
                             ' shown in the flow view)'))
         self.params['conditions'] = Param(
-            list(conditions), valType='str', updates=None,
-            allowedUpdates=None,
+            list(conditions), valType='list', inputType='single',
+            updates=None, allowedUpdates=None,
             label=_localized['conditions'],
             hint=_translate("A list of dictionaries describing the "
                             "differences between each staircase"))
         self.params['conditionsFile'] = Param(
-            conditionsFile, valType='str', updates=None, allowedUpdates=None,
+            conditionsFile, valType='file', inputType='table', updates=None, allowedUpdates=None,
             label=_localized['conditions'],
             hint=_translate("An xlsx or csv file specifying the parameters "
                             "for each condition"))
         self.params['isTrials'] = Param(
-            isTrials, valType='bool', updates=None, allowedUpdates=None,
+            isTrials, valType='bool', inputType='bool', updates=None, allowedUpdates=None,
             label=_localized["Is trials"],
             hint=_translate("Indicates that this loop generates TRIALS, "
                             "rather than BLOCKS of trials or stimuli within "
                             "a trial. It alters how data files are output"))
         pass  # don't initialise at start of exp, create when needed
 
+    @property
+    def name(self):
+        return self.params['name'].val
+
     def writeLoopStartCode(self, buff):
         # create a 'thisName' for use in "for thisTrial in trials:"
         makeLoopIndex = self.exp.namespace.makeLoopIndex
@@ -556,6 +577,102 @@ class MultiStairHandler(object):
                     "    exec(paramName + '= condition[paramName]')\n")
             buff.writeIndentedLines(code)
 
+    def writeLoopStartCodeJS(self, buff, modular):
+        inits = deepcopy(self.params)
+        # For JS, stairType needs to be code
+        inits['stairType'].valType = "code"
+        # Method needs to be code and upper
+        inits['switchMethod'].valType = "code"
+        inits['switchMethod'].val = inits['switchMethod'].val.upper()
+
+        code = (
+            "\nfunction %(name)sLoopBegin(%(name)sLoopScheduler, snapshot) {\n"
+        )
+        buff.writeIndentedLines(code % inits)
+
+        buff.setIndentLevel(1, relative=True)
+        code = (
+                "return async function() {\n"
+        )
+        buff.writeIndentedLines(code % inits)
+
+        buff.setIndentLevel(1, relative=True)
+        code = (
+                    "// setup a MultiStairTrialHandler\n"
+                    "%(name)sConditions = TrialHandler.importConditions(psychoJS.serverManager, %(conditionsFile)s);\n"
+                    "%(name)s = new data.MultiStairHandler({stairType:MultiStairHandler.StaircaseType.%(stairType)s, \n"
+        )
+        buff.writeIndentedLines(code % inits)
+
+        buff.setIndentLevel(1, relative=True)
+        code = (
+                        "psychoJS: psychoJS,\n"
+                        "name: '%(name)s',\n"
+                        "varName: '%(name)sVal',\n"
+                        "nTrials: %(nReps)s,\n"
+                        "conditions: %(name)sConditions,\n"
+                        "method: TrialHandler.Method.%(switchMethod)s\n"
+        )
+        buff.writeIndentedLines(code % inits)
+
+        buff.setIndentLevel(-1, relative=True)
+        code = (
+                    "});\n"
+                    "psychoJS.experiment.addLoop(%(name)s); // add the loop to the experiment\n"
+                    "currentLoop = %(name)s;  // we're now the current loop\n"
+                    "// Schedule all the trials in the trialList:\n"
+                    "for (const thisQuestLoop of %(name)s) {\n"
+        )
+        buff.writeIndentedLines(code % inits)
+
+        buff.setIndentLevel(1, relative=True)
+        thisLoop = self.exp.flow.loopDict[self]
+        for thisChild in thisLoop:
+            if thisChild.getType() == 'Routine':
+                code = (
+                        "const snapshot = %(name)s.getSnapshot();\n"
+                        "{loopName}LoopScheduler.add(importConditions(snapshot));\n"
+                        "{loopName}LoopScheduler.add({childName}RoutineBegin(snapshot));\n"
+                        "{loopName}LoopScheduler.add({childName}RoutineEachFrame());\n"
+                        "{loopName}LoopScheduler.add({childName}RoutineEnd());\n"
+                        "{loopName}LoopScheduler.add(endLoopIteration({loopName}LoopScheduler, snapshot));\n"
+                        .format(childName=thisChild.params['name'],
+                                loopName=self.params['name'])
+                    )
+            else:  # for a LoopInitiator
+                code = (
+                        "const snapshot = %(name)s.getSnapshot();\n"
+                        "const {childName}LoopScheduler = new Scheduler(psychoJS);\n"
+                        "{loopName}LoopScheduler.add(importConditions(snapshot));\n"
+                        "{loopName}LoopScheduler.add({childName}LoopBegin({childName}LoopScheduler, snapshot));\n"
+                        "{loopName}LoopScheduler.add({childName}LoopScheduler);\n"
+                        "{loopName}LoopScheduler.add({childName}LoopEnd);\n"
+                        "{loopName}LoopScheduler.add(endLoopIteration({loopName}LoopScheduler, snapshot));\n"
+                        .format(childName=thisChild.params['name'],
+                                loopName=self.params['name'])
+                        )
+            buff.writeIndentedLines(code % inits)
+
+        buff.setIndentLevel(-1, relative=True)
+        code = (
+                "}"
+                "\n\n"
+                "return Scheduler.Event.NEXT;\n"                
+        )
+        buff.writeIndentedLines(code % inits)
+
+        buff.setIndentLevel(-1, relative=True)
+        code = (
+                "}"
+        )
+        buff.writeIndentedLines(code % inits)
+
+        buff.setIndentLevel(-1, relative=True)
+        code = (
+            "}"
+        )
+        buff.writeIndentedLines(code % inits)
+
     def writeLoopEndCode(self, buff):
         # Just within the loop advance data line if loop is whole trials
         if self.params['isTrials'].val:
@@ -574,6 +691,27 @@ class MultiStairHandler(object):
                         "delim=',')\n")
                 buff.writeIndented(code % self.params)
 
+    def writeLoopEndCodeJS(self, buff):
+        code = (
+            "\n"
+            "async function %(name)sLoopEnd() {\n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
+        buff.setIndentLevel(1, relative=True)
+        code = (
+                "// terminate loop\n"
+                "psychoJS.experiment.removeLoop(%(name)s);\n"
+                "return Scheduler.Event.NEXT;\n"
+        )
+        buff.writeIndentedLines(code % self.params)
+
+        buff.setIndentLevel(-1, relative=True)
+        code = (
+            "}"
+        )
+        buff.writeIndentedLines(code % self.params)
+
     def getType(self):
         return 'MultiStairHandler'
 
@@ -582,7 +720,7 @@ class MultiStairHandler(object):
         pass
 
 
-class LoopInitiator(object):
+class LoopInitiator:
     """A simple class for inserting into the flow.
     This is created automatically when the loop is created"""
 
@@ -592,6 +730,31 @@ class LoopInitiator(object):
         self.exp = loop.exp
         loop.initiator = self
 
+    @property
+    def _xml(self):
+        # Make root element
+        element = Element("LoopInitiator")
+        element.set("loopType", self.loop.__class__.__name__)
+        element.set("name", self.loop.params['name'].val)
+        # Add an element for each parameter
+        for key, param in sorted(self.loop.params.items()):
+            # Create node
+            paramNode = Element("Param")
+            paramNode.set("name", key)
+            # Assign values
+            if hasattr(param, 'updates'):
+                paramNode.set('updates', "{}".format(param.updates))
+            if hasattr(param, 'val'):
+                paramNode.set('val', u"{}".format(param.val).replace("\n", "&#10;"))
+            if hasattr(param, 'valType'):
+                paramNode.set('valType', param.valType)
+            element.append(paramNode)
+        return element
+
+    @property
+    def name(self):
+        return self.loop.name
+
     def getType(self):
         return 'LoopInitiator'
 
@@ -615,7 +778,7 @@ class LoopInitiator(object):
         pass
 
 
-class LoopTerminator(object):
+class LoopTerminator:
     """A simple class for inserting into the flow.
     This is created automatically when the loop is created"""
 
@@ -625,6 +788,18 @@ class LoopTerminator(object):
         self.exp = loop.exp
         loop.terminator = self
 
+    @property
+    def _xml(self):
+        # Make root element
+        element = Element("LoopTerminator")
+        element.set("name", self.loop.params['name'].val)
+
+        return element
+
+    @property
+    def name(self):
+        return self.loop.name
+
     def getType(self):
         return 'LoopTerminator'
 
