@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 #  Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """
@@ -12,7 +12,6 @@ This lib will attempt to use PyQt (4 or 5) if possible and will revert to
 using wxPython if PyQt is not found.
 """
 
-from __future__ import absolute_import, print_function
 import sys
 from .. import constants
 
@@ -27,10 +26,7 @@ else:
 haveQt = False  # until we confirm otherwise
 if wxApp is None:  # i.e. don't try this if wx is already running
     # set order for attempts on PyQt4/PyQt5
-    if constants.PY3:  # much more like to have PyQt5 on Python3
-        importOrder = ['PyQt5', 'PyQt4']
-    else:  # more likely the other way on Py27
-        importOrder = ['PyQt4', 'PyQt5']
+    importOrder = ['PyQt5', 'PyQt4']
     # then check each in turn
     for libname in importOrder:
         try:
