@@ -4,22 +4,12 @@
 # To build simple dialogues etc. (requires pyqt4)
 #
 #  Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, print_function
-from .. import constants
-
-from builtins import str
-from past.builtins import unicode
-
 haveQt = False  # until we confirm otherwise
-if constants.PY3:  # much more like to have PyQt5 on Python3
-    importOrder = ['PyQt5', 'PyQt4']
-else:  # more likely the other way on Py27
-    importOrder = ['PyQt4', 'PyQt5']
+importOrder = ['PyQt5', 'PyQt4']
 
-haveQt = False
 for libname in importOrder:
     try:
         exec("import {}".format(libname))
@@ -205,7 +195,7 @@ class Dlg(QtWidgets.QDialog):
                 thisType = self.inputFieldTypes[name]
 
                 try:
-                    if thisType in (str, unicode, bytes):
+                    if thisType in (str, bytes):
                         self.data[ix] = str(new_text)
                     elif thisType == tuple:
                         jtext = "[" + str(new_text) + "]"
@@ -368,16 +358,16 @@ class DlgFromDict(Dlg):
 
     labels : dict
         A dictionary defining labels (values) to be displayed instead of
-        key strings (keys) defined in ``dictionary´´. Not all keys in
-        ``dictionary´´ need to be contained in labels.
+        key strings (keys) defined in `dictionary`. Not all keys in
+        `dictionary` need to be contained in labels.
 
     fixed : list
         A list of keys for which the values shall be displayed in non-editable
         fields
     
     order : list
-        A list of keys defining the display order of keys in ``dictionary´´.
-        If not all keys in ``dictionary´´ are contained in ``order´´, those
+        A list of keys defining the display order of keys in `dictionary`.
+        If not all keys in `dictionary`` are contained in `order`, those
         will appear in random order after all ordered keys.
 
     tip : list
@@ -391,20 +381,20 @@ class DlgFromDict(Dlg):
         A boolean flag indicating that keys are to be sorted alphabetically.
 
     copyDict : bool
-        If False, modify ``dictionary`` in-place. If True, a copy of
+        If False, modify `dictionary` in-place. If True, a copy of
         the dictionary is created, and the altered version (after
         user interaction) can be retrieved from
         :attr:~`psychopy.gui.DlgFromDict.dictionary`.
         
     labels : dict
         A dictionary defining labels (dict values) to be displayed instead of
-        key strings (dict keys) defined in ``dictionary´´. Not all keys in
-        ``dictionary´´ need to be contained in labels.
+        key strings (dict keys) defined in `dictionary`. Not all keys in
+        `dictionary´ need to be contained in labels.
 
     show : bool
         Whether to immediately display the dialog upon instantiation.
-         If False, it can be displayed at a later time by calling
-         its `show()` method.
+        If False, it can be displayed at a later time by calling
+        its `show()` method.
 
     e.g.:
 
@@ -466,6 +456,7 @@ class DlgFromDict(Dlg):
             self.dictionary = dictionary
 
         self._keys = list(self.dictionary.keys())
+        self._labels = labels
 
         if order:
             self._keys = list(order) + list(set(self._keys).difference(set(order)))
@@ -494,8 +485,12 @@ class DlgFromDict(Dlg):
         ok_data = self.exec_()
         if ok_data:
             for n, thisKey in enumerate(self._keys):
+                if thisKey in self._labels:
+                    labelKey = self._labels[thisKey]
+                else:
+                    labelKey = thisKey
                 try:
-                    self.dictionary[thisKey] = self.inputFieldTypes[thisKey](self.data[n])
+                    self.dictionary[thisKey] = self.inputFieldTypes[labelKey](self.data[n])
                 except ValueError:
                     self.dictionary[thisKey] = self.data[n]
 
@@ -516,8 +511,8 @@ def fileSaveDlg(initFilePath="", initFileName="",
             can be set to custom prompts
         allowed: string
             a string to specify file filters.
-            e.g. "Text files (\*.txt) ;; Image files (\*.bmp \*.gif)"
-            See http://pyqt.sourceforge.net/Docs/PyQt4/qfiledialog.html
+            e.g. "Text files (\\*.txt) ;; Image files (\\*.bmp \\*.gif)"
+            See https://www.riverbankcomputing.com/static/Docs/PyQt4/qfiledialog.html
             #getSaveFileName
             for further details
 
@@ -563,8 +558,8 @@ def fileOpenDlg(tryFilePath="",
 
         allowed: string (available since v1.62.01)
             a string to specify file filters.
-            e.g. "Text files (\*.txt) ;; Image files (\*.bmp \*.gif)"
-            See http://pyqt.sourceforge.net/Docs/PyQt4/qfiledialog.html
+            e.g. "Text files (\\*.txt) ;; Image files (\\*.bmp \\*.gif)"
+            See https://www.riverbankcomputing.com/static/Docs/PyQt4/qfiledialog.html
             #getOpenFileNames
             for further details
 
@@ -694,6 +689,7 @@ if __name__ == '__main__':
     info = {'Observer': 'jwp', 'GratingOri': 45,
             'ExpVersion': 1.1, 'Group': ['Test', 'Control']}
     dictDlg = DlgFromDict(dictionary=info, title='TestExperiment',
+                          labels={'Group': 'Participant Group'},
                           fixed=['ExpVersion'])
     if dictDlg.OK:
         print(info)
