@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 # Acknowledgements:
@@ -12,11 +12,6 @@
 # Acknowledgements
 #    This code was written by Jon Peirce
 
-from __future__ import absolute_import, division, print_function
-
-from builtins import bytes
-from builtins import range
-from builtins import object
 import sys
 try:
     import serial
@@ -34,7 +29,7 @@ except ImportError:
 eol = "\n\r"  # unusual for a serial port?!
 
 
-class ColorCAL(object):
+class ColorCAL:
     """A class to handle the CRS Ltd ColorCAL device
     """
 
@@ -116,7 +111,7 @@ class ColorCAL(object):
         self.calibMatrix = self.getCalibMatrix()
 
     def sendMessage(self, message, timeout=0.1):
-        """Send a command to the photometer and wait an alloted
+        """Send a command to the photometer and wait an allotted
         timeout for a response.
         """
         # flush the read buffer first
