@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """This provides a basic ButtonBox class, and imports the `ioLab python library
@@ -12,10 +12,6 @@
 # And iolabs.py (lowercase) did not solve it either, something is case
 # insensitive somewhere
 
-
-from __future__ import absolute_import, division, print_function
-
-from builtins import range
 from numpy import ubyte
 from psychopy import core, event, logging
 
