@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """PhotoResearch spectrophotometers
@@ -10,17 +10,11 @@ See http://www.photoresearch.com/
 
 --------
 """
-from __future__ import absolute_import, print_function
 
-from builtins import str
-from builtins import range
-from builtins import object
 from psychopy import logging
-import struct
 import sys
 import time
 import numpy
-from psychopy.constants import PY3
 
 try:
     import serial
@@ -28,7 +22,7 @@ except ImportError:
     serial = False
 
 
-class PR650(object):
+class PR650:
     """An interface to the PR650 via the serial port.
 
     (Added in version 1.63.02)
@@ -159,12 +153,9 @@ class PR650(object):
         if message in ('d5\n', 'D5\n'):
             # we need a spectrum which will have multiple lines
             reply = self.com.readlines()
-            if PY3:
-                reply = [thisLine.decode('utf-8') for thisLine in reply]
+            reply = [thisLine.decode('utf-8') for thisLine in reply]
         else:
-            reply = self.com.readline()
-            if PY3:
-                reply = reply.decode('utf-8')
+            reply = self.com.readline().decode('utf-8')
         return reply
 
     def measure(self, timeOut=30.0):
@@ -273,8 +264,7 @@ class PR650(object):
                 thisNm, thisPower = point.split(',')
                 nm.append(thisNm)
                 power.append(thisPower.replace('\r\n', ''))
-            if progDlg:
-                progDlg.Update(n)
+
         return numpy.asarray(nm), numpy.asarray(power)
 
 
