@@ -1,11 +1,9 @@
 #!/usr/bin/env python
-#  -*- coding: utf-8 -*-
-
-# Part of the psychopy.iohub library.
-# Copyright (C) 2012-2016 iSolver Software Solutions
+# -*- coding: utf-8 -*-
+# Part of the PsychoPy library
+# Copyright (C) 2012-2020 iSolver Software Solutions (C) 2021 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
-from __future__ import division, absolute_import, print_function
-from past.builtins import unicode
+
 import os
 import sys
 import time
@@ -20,9 +18,8 @@ try:
     import psychopy.logging as psycho_logging
 except ImportError:
     psycho_logging = None
-from past.builtins import basestring
 from ..lazy_import import lazy_import
-from .. import _pkgroot, IOHUB_DIRECTORY
+from .. import IOHUB_DIRECTORY
 from ..util import yload, yLoader
 from ..errors import print2err, ioHubError, printExceptionDetailsToStdErr
 from ..util import isIterable, updateDict, win32MessagePump
@@ -36,7 +33,22 @@ getTime = Computer.getTime
 
 _currentSessionInfo = None
 
-class DeviceRPC(object):
+def windowInfoDict(win):
+    windict = dict(handle=win._hw_handle, pos=win.pos, size=win.size,
+                   units=win.units, useRetina=win.useRetina, monitor=None)
+    if win.monitor:
+        windict['monitor'] = dict(resolution=win.monitor.getSizePix(),
+                                  width=win.monitor.getWidth(),
+                                  distance=win.monitor.getDistance())
+    return windict
+
+def getFullClassName(klass):
+    module = klass.__module__
+    if module == 'builtins':
+        return klass.__qualname__  # avoid outputs like 'builtins.str'
+    return module + '.' + klass.__qualname__
+
+class DeviceRPC():
     '''
     ioHubDeviceView creates an RPC interface with the iohub server. Each
     iohub device method exposed by an ioHubDeviceView is represented
@@ -108,7 +120,7 @@ class DeviceRPC(object):
 
 # pylint: disable=protected-access
 
-class ioHubDeviceView(object):
+class ioHubDeviceView():
     """
     ioHubDeviceView is used as a client / PsychoPy process side representation
     of an ioHub device that is actually running on the separate iohub process.
@@ -124,12 +136,11 @@ class ioHubDeviceView(object):
     it connects to the ioHub Process.
     """
 
-    def __init__(self, hubClient, device_class_name, device_config):
+    def __init__(self, hubClient, device_class_path, device_class_name, device_config):
         self.hubClient = hubClient
         self.name = device_config.get('name', device_class_name.lower())
         self.device_class = device_class_name
-        #self._preRemoteMethodCallFunctions = dict()
-        #self._postRemoteMethodCallFunctions = dict()
+        self.device_class_path=device_class_path
 
         rpc_request = ('EXP_DEVICE', 'GET_DEV_INTERFACE', device_class_name)
         r = self.hubClient._sendToHubServer(rpc_request)
@@ -137,23 +148,10 @@ class ioHubDeviceView(object):
 
     def __getattr__(self, name):
         if name in self._methods:
-            #if name in self._preRemoteMethodCallFunctions:
-            #    f, ka = self._preRemoteMethodCallFunctions[name]
-            #    f(ka)
-            r = DeviceRPC(self.hubClient._sendToHubServer, self.device_class,
-                          name)
-            #if name in self._postRemoteMethodCallFunctions:
-            #    f, ka = self._postRemoteMethodCallFunctions[name]
-            #    f(ka)
+            r = DeviceRPC(self.hubClient._sendToHubServer, self.device_class, name)
             return r
         raise AttributeError(self, name)
 
-#    def setPreRemoteMethodCallFunction(self, methodName, funcCall, **kwargs):
-#        self._preRemoteMethodCallFunctions[methodName] = (funcCall, kwargs)
-
-#    def setPostRemoteMethodCallFunction(self, methodName, func_call, **kwargs):
-#        self._postRemoteMethodCallFunctions[methodName] = (func_call, kwargs)
-
     def getName(self):
         """
         Gets the name given to the device in the ioHub configuration file.
@@ -167,18 +165,20 @@ class ioHubDeviceView(object):
         """
         return self.name
 
-    def getIOHubDeviceClass(self):
+    def getIOHubDeviceClass(self, full=False):
         """
         Gets the ioHub Device class associated with the oHubDeviceView.
         This is specified for a device in the ioHub configuration file.
         ( the device: device_class: property )
 
-        Args:
-            None
+        :param full:
 
         Returns:
             (class): ioHub Device class associated with this ioHubDeviceView
+
         """
+        if full:
+            return self.device_class_path
         return self.device_class
 
     def getDeviceInterface(self):
@@ -198,7 +198,7 @@ class ioHubDeviceView(object):
 
 # pylint: enable=protected-access
 
-class ioHubDevices(object):
+class ioHubDevices():
     """
     Provides .name access to the the ioHub device's created when the ioHub
     Server is started. Each iohub device is accessible via a dynamically
@@ -227,7 +227,7 @@ class ioHubDevices(object):
     def getNames(self):
         return self._devicesByName.keys()
 
-class ioHubConnection(object):
+class ioHubConnection():
     """ioHubConnection is responsible for creating, sending requests to, and
     reading replies from the ioHub Process. This class is also used to
     shut down and disconnect the ioHub Server process.
@@ -290,7 +290,7 @@ class ioHubConnection(object):
         self._iohub_server_config = None
         self._shutdown_attempted = False
         self._cv_order = None
-
+        self._message_cache = []
         self.iohub_status = self._startServer(ioHubConfig, ioHubConfigAbsPath)
         if self.iohub_status != 'OK':
             raise RuntimeError('Error starting ioHub server: {}'.format(self.iohub_status))
@@ -441,9 +441,28 @@ class ioHubConnection(object):
         Create and send an Experiment MessageEvent to the ioHub Server
         for storage in the ioDataStore hdf5 file.
 
-        .. note::
-            MessageEvents can be thought of as DeviceEvents from the
-            virtual PsychoPy Process "Device".
+        Args:
+            text (str): The text message for the message event. 128 char max.
+
+            category (str): A str grouping code for the message. Optional.
+                            32 char max.
+
+            offset (float): Optional sec.msec offset applied to the
+                            message event time stamp. Default 0.
+
+            sec_time (float): Absolute sec.msec time stamp for the message in.
+                              If not provided, or None, then the MessageEvent
+                              is time stamped when this method is called
+                              using the global timer (core.getTime()).
+        """
+        self.cacheMessageEvent(text, category, offset, sec_time)
+        self._sendToHubServer(('EXP_DEVICE', 'EVENT_TX', self._message_cache))
+        self._message_cache = []
+
+    def cacheMessageEvent(self, text, category='', offset=0.0, sec_time=None):
+        """
+        Create an Experiment MessageEvent and store in local cache.
+        Message must be sent before it is saved to hdf5 file.
 
         Args:
             text (str): The text message for the message event. 128 char max.
@@ -458,17 +477,22 @@ class ioHubConnection(object):
                               If not provided, or None, then the MessageEvent
                               is time stamped when this method is called
                               using the global timer (core.getTime()).
-
-        Returns:
-            bool: True
-
         """
-        msg_evt = MessageEvent._createAsList(text, # pylint: disable=protected-access
+        self._message_cache.append(MessageEvent._createAsList(text, # pylint: disable=protected-access
                                              category=category,
                                              msg_offset=offset,
-                                             sec_time=sec_time)
-        self._sendToHubServer(('EXP_DEVICE', 'EVENT_TX', [msg_evt, ]))
-        return True
+                                             sec_time=sec_time))
+
+    def sendMessageEvents(self, messageList=[]):
+        if messageList:
+            self.cacheMessageEvents(messageList)
+        if self._message_cache:
+            self._sendToHubServer(('EXP_DEVICE', 'EVENT_TX', self._message_cache))
+            self._message_cache = []
+
+    def cacheMessageEvents(self, messageList):
+        for m in messageList:
+            self._message_cache.append(MessageEvent._createAsList(**m))
 
     def getHubServerConfig(self):
         """Returns a dict containing the current ioHub Server configuration.
@@ -618,7 +642,7 @@ class ioHubConnection(object):
 
         for cond_name in self._cv_order:
             cond_val = trial[cond_name]
-            if isinstance(cond_val, basestring):
+            if isinstance(cond_val, str):
                 numpy_dtype = (cond_name, 'S', 256)
             elif isinstance(cond_val, int):
                 numpy_dtype = (cond_name, 'i8')
@@ -653,7 +677,7 @@ class ioHubConnection(object):
             data = list(cv_row.values())
 
         for i, d in enumerate(data):
-            if isinstance(d, unicode):
+            if isinstance(d, str):
                 data[i] = d.encode('utf-8')
 
         cvt_rpc = ('RPC', 'extendConditionVariableTable',
@@ -678,6 +702,10 @@ class ioHubConnection(object):
             ('RPC', 'unregisterWindowHandles', winHandles))
         return r[2]
 
+    def updateWindowPos(self, win, x, y):
+        r = self._sendToHubServer(('RPC', 'updateWindowPos', (win._hw_handle, (x, y))))
+        return r[2]
+
     def getTime(self):
         """
         **Deprecated Method:** Use Computer.getTime instead. Remains here for
@@ -784,16 +812,17 @@ class ioHubConnection(object):
         return self._addDeviceView(dev_name, device_class_name)
 
     def flushDataStoreFile(self):
-        """Manually tell the ioDataStore to flush any events it has buffered in
-        memory to disk.".
+        """Manually tell the iohub datastore to flush any events it has buffered in
+        memory to disk. Any cached message events are sent to the iohub server
+        before flushing the iohub datastore.
 
         Args:
             None
 
         Returns:
             None
-
         """
+        self.sendMessageEvents()
         r = self._sendToHubServer(('RPC', 'flushIODataStoreFile'))
         return r
 
@@ -902,32 +931,11 @@ class ioHubConnection(object):
 
         self._iohub_server_config = ioHubConfig
 
-        # >>>> Check for orphaned ioHub Process and kill if found...
-        iopFileName = os.path.join(rootScriptPath, '.iohpid')
-        if os.path.exists(iopFileName):
-            try:
-                iopFile = open(iopFileName, 'r')
-                line = iopFile.readline()
-                iopFile.close()
-                os.remove(iopFileName)
-                _, iohub_pid = line.split(':')
-                iohub_pid = int(iohub_pid.strip())
-                try:
-                    old_iohub_process = psutil.Process(iohub_pid)
-                    if old_iohub_process.name == 'python.exe':
-                        old_iohub_process.kill()
-                except psutil.NoSuchProcess:
-                    pass
-            except Exception: # pylint: disable=broad-except
-                import traceback
-                traceback.print_exc()
-
         if sys.platform == 'darwin':
             self._osxKillAndFreePort()
-        # <<<< Done handling orphaned iohub process fail safe.
 
         # >>>> Start iohub subprocess
-        run_script = os.path.join(IOHUB_DIRECTORY, 'launchHubProcess.py')
+        run_script = os.path.join(IOHUB_DIRECTORY, 'start_iohub_process.py')
         subprocessArgList = [sys.executable,
                              run_script,
                              '%.6f' % Computer.global_clock.getLastResetTime(),
@@ -958,7 +966,11 @@ class ioHubConnection(object):
 
         self._server_process = subprocess.Popen(subprocessArgList,
                                                 env=envars,
-                                                cwd=IOHUB_DIRECTORY)
+                                                cwd=IOHUB_DIRECTORY,
+                                                # set sub process stderr to be stdout so PsychoPy Runner
+                                                # shows errors from iohub
+                                                stderr=subprocess.STDOUT,
+                                                )
 
         # Get iohub server pid and psutil process object
         # for affinity and process priority setting.
@@ -993,18 +1005,13 @@ class ioHubConnection(object):
                 whs = []
                 # pylint: disable=protected-access
                 for w in window.openWindows:
-                    whs.append(w()._hw_handle)
+                    winfo = windowInfoDict(w())
+                    whs.append(winfo)
+                    w().backend.onMoveCallback = self.updateWindowPos
                 self.registerWindowHandles(*whs)
         except ImportError:
             pass
 
-        # Save ioHub ProcessID to file so next time it is started,
-        # it can be checked and killed if necessary
-        iopFile = open(iopFileName, 'w')
-        iopFile.write("ioHub PID: {}".format(Computer.iohub_process_id))
-        iopFile.flush()
-        iopFile.close()
-
         # Sending experiment_info if available.....
         if experiment_info:
             self._sendExperimentInfo(experiment_info)
@@ -1054,7 +1061,7 @@ class ioHubConnection(object):
             dev_cls_name = "{}".format(dev_cls_name)
             dev_name = dev_cls_name.lower()
             cls_name_start = dev_name.rfind('.')
-            dev_mod_pth = '%s.devices.' % _pkgroot
+            dev_mod_pth = 'psychopy.iohub.devices.'
             if cls_name_start > 0:
                 dev_mod_pth2 = dev_name[:cls_name_start]
                 dev_mod_pth = '{0}{1}'.format(dev_mod_pth, dev_mod_pth2)
@@ -1084,14 +1091,15 @@ class ioHubConnection(object):
                 # need to touch local_module since it was lazy loaded
 
                 # pylint: disable=exec-used
-                exec('import {}.client.{}'.format(_pkgroot,
-                                                  dev_cls_name.lower()))
+                exec('import psychopy.iohub.client.{}'.format(dev_cls_name.lower()))
                 local_class = getattr(local_module, dev_cls_name, False)
 
             if local_class:
                 d = local_class(self, dev_cls_name, dev_config)
             else:
-                d = ioHubDeviceView(self, dev_cls_name, dev_config)
+                full_device_class_name = getFullClassName(dev_cls)[len('psychopy.iohub.devices.'):]
+                full_device_class_name = full_device_class_name.replace('eyetracker.EyeTracker', 'EyeTracker')
+                d = ioHubDeviceView(self, full_device_class_name, dev_cls_name, dev_config)
 
             self.devices.addDevice(name, d)
             return d
@@ -1171,7 +1179,7 @@ class ioHubConnection(object):
             raise ioHubError(result)
         # Otherwise return the result
         
-        if constants.PY3 and result is not None:
+        if result is not None:
             # Use recursive conversion funcs                     
             if isinstance(result, list) or  isinstance(result, tuple):
                 result = self._convertList(result)
@@ -1186,8 +1194,7 @@ class ioHubConnection(object):
         and returns a new or existing experiment ID based on that criteria.
         """
         fieldOrder = (('experiment_id', 0), ('code', ''), ('title', ''),
-                      ('description', ''), ('version', ''),
-                      ('total_sessions_to_run', 0))
+                      ('description', ''), ('version', ''))
         values = []
         for key, defaultValue in fieldOrder:
             if key in experimentInfoDict:
@@ -1271,6 +1278,9 @@ class ioHubConnection(object):
 
     def _shutDownServer(self):
         if self._shutdown_attempted is False:
+            # send any cached experiment messages
+            self.sendMessageEvents()
+
             try:
                 from psychopy.visual import window
                 window.IOHUB_ACTIVE = False
@@ -1313,7 +1323,7 @@ class ioHubConnection(object):
             if isIterable(d0):
                 return False
             else:
-                if isinstance(d0, basestring) and d0.find('ERROR') >= 0:
+                if isinstance(d0, str) and d0.find('ERROR') >= 0:
                     return data
                 return False
         else:
@@ -1341,7 +1351,7 @@ class ioHubConnection(object):
 
 ##############################################################################
 
-class ioEvent(object):
+class ioEvent():
     """
     Parent class for all events generated by a psychopy.iohub.client
     Device wrapper.
@@ -1410,14 +1420,13 @@ class ioEvent(object):
                                                  self.id)
 
 _lazyImports = """
-from {pkgroot}.client.connect import launchHubServer
-from {pkgroot}.client import keyboard
-#from {pkgroot}.client import wintabtablet
-""".format(pkgroot=_pkgroot)
+from psychopy.iohub.client.connect import launchHubServer
+from psychopy.iohub.client import keyboard
+from psychopy.iohub.client import wintab
+"""
 
 try:
     lazy_import(globals(), _lazyImports)
 except Exception as e: #pylint: disable=broad-except
     print2err('lazy_import Exception:', e)
     exec(_lazyImports) #pylint: disable=exec-used
-
