@@ -1,11 +1,10 @@
-﻿# -*- coding: utf-8 -*-
-# Part of the psychopy.iohub library.
-# Copyright (C) 2012-2016 iSolver Software Solutions
+# -*- coding: utf-8 -*-
+# Part of the PsychoPy library
+# Copyright (C) 2012-2020 iSolver Software Solutions (C) 2021 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 import sys
 from .. import Device, Computer
-from ... import _ispkg
 from ...constants import DeviceConstants
 from ...errors import print2err, printExceptionDetailsToStdErr
 import pyglet
@@ -22,12 +21,11 @@ class Display(Device):
 
     """
     _coord_type_mappings = dict(pix='pix', pixel='pix', pixels='pix',
-                                deg='deg', degree='deg', degrees='deg',
-                                cm='cm',  # mm='mm', inch='inch', inches='inch',
+                                deg='deg', degree='deg', degrees='deg', cm='cm',
                                 norm='norm', normalize='norm', normalized='norm',
-                                # perc='perc',percent='perc', percentage='perc'
+                                height='height'
                                 )
-    _supported_origin_types = ['center', ]  # ,'top_left','bottom_left']
+    _supported_origin_types = ['center', ]
 
     _enabled_display_instances = []
     _computer_display_runtime_info_list = None
@@ -52,9 +50,7 @@ class Display(Device):
             self._xwindow = None
 
         if Display._computer_display_runtime_info_list is None:
-            Display._computer_display_runtime_info_list =\
-                Display._createAllRuntimeInfoDicts()
-
+            Display._computer_display_runtime_info_list = Display._createAllRuntimeInfoDicts()
         self._addRuntimeInfoToDisplayConfig()
 
     def getDeviceNumber(self):
@@ -98,6 +94,15 @@ class Display(Device):
         """
         return len(cls._computer_display_runtime_info_list)
 
+    @classmethod
+    def getAllDisplayBounds(cls):
+        """
+        Returns pixel display bounds (l,t r,b) for each detected display.
+        
+        :return: list of (l,t r,b) tuples
+        """
+        return [d.get('bounds') for d in cls._computer_display_runtime_info_list]
+
     def getRuntimeInfo(self, display_index = None):
         """
         Returns a dictionary containing run-time determined settings for the
@@ -149,6 +154,21 @@ class Display(Device):
         """
         return self.getConfiguration()['reporting_unit_type']
 
+    def getColorSpace(self):
+        """
+        Returns the color space to use for PsychoPy Windows.
+
+        Please refer to the psychoPy documentation for a detailed description of
+        supported color spaces.
+
+        Args:
+            None
+
+        Returns:
+            str: Display color space
+        """
+        return self.getConfiguration()['color_space']
+
     def getPixelsPerDegree(self):
         """Returns the Display's horizontal and vertical pixels per degree This
         is currently calculated using the PsychoPy built in function. Therefore
@@ -157,7 +177,7 @@ class Display(Device):
 
         The physical characteristics of the Display and the Participants viewing distance
         will either be based on the ioHub settings specified, or based on the information
-        saved in the PsychoPy Monitor Configuartion file that can be optionally
+        saved in the PsychoPy Monitor Configuration file that can be optionally
         given to the Display Device before it is instantiated.
 
         Args:
@@ -217,8 +237,7 @@ class Display(Device):
 
     def getCoordBounds(self):
         """Get the Display's left, top, right, and bottom border bounds,
-        specified in the coordinate space returned by
-        Display.getCoordinateType()
+        specified in the coordinate space returned by Display.getCoordinateType()
 
         Args:
             None
@@ -251,7 +270,7 @@ class Display(Device):
     def getPhysicalDimensions(self):
         """Returns the Display's physical screen area ( width,  height ) as
         specified in the ioHub Display devices configuration settings or by a
-        PsychoPy Monitor Configuartion file.
+        PsychoPy Monitor Configuration file.
 
         Args:
             None
@@ -473,7 +492,10 @@ class Display(Device):
                 runtime_info['bounds'] = (x, y, x + w, y + h)
                 runtime_info['primary'] = runtime_info['bounds'] == dbounds
                 if mode:
-                    runtime_info['retrace_rate'] = mode.rate
+                    rate = mode.rate
+                    if rate == 0.0:
+                        rate = 60
+                    runtime_info['retrace_rate'] = rate
                     runtime_info['bits_per_pixel'] = mode.depth
                     if mode and mode.width > 0 and mode.height > 0:
                         runtime_info['pixel_resolution'] = mode.width, mode.height
@@ -498,8 +520,7 @@ class Display(Device):
             runtime_info = self._getRuntimeInfoByIndex(self.device_number)
             display_config['runtime_info'] = runtime_info
 
-            if _ispkg is False:
-                self._createPsychopyCalibrationFile()
+            self._createPsychopyCalibrationFile()
 
             pixel_width = runtime_info['pixel_width']
             pixel_height = runtime_info['pixel_height']
@@ -548,11 +569,13 @@ class Display(Device):
         coord_type = self.getCoordinateType()
         if coord_type in Display._coord_type_mappings:
             coord_type = Display._coord_type_mappings[coord_type]
-        else:
-            print2err(
-                ' *** Display device error: Unknown coordinate type: {0}'.format(coord_type))
+        elif coord_type is None:
+            print2err(' *** iohub warning: Display / Monitor unit type has not been set.')
             return
-
+        else:
+            print2err(' *** iohub error: Unknown Display / Monitor coordinate type: {0}'.format(coord_type))
+            return
+        
         self._pix2coord = None
 
         # For now, use psychopy unit conversions so that drawing positions match
@@ -567,7 +590,7 @@ class Display(Device):
             return (x - w / 2), -y + h / 2
 
         def psychopy2displayPix(cx, cy):
-            return l + (cx + w / 2), t + (cy + h / 2)
+            return l + (cx + w / 2), b - (cy + h / 2)
 
         if coord_type == 'pix':
             def pix2coord(self, x, y, display_index=None):
@@ -621,6 +644,27 @@ class Display(Device):
                         return cx, cy
                     self._coord2pix = cmcoord2pix
 
+                elif coord_type == 'height':
+                    def pix2heightcoord(self, x, y, display_index=None):
+                        if display_index == self.getIndex():
+                            ppx, ppy = display2psychopyPix(x, y)
+                            return ppx / h, ppy / h
+                        return x, y
+                    self._pix2coord = pix2heightcoord
+
+                    def height2pix(self, x, y, display_index=None):
+                        if display_index == self.getIndex():
+                            if False: #TODO: Deal with win.useRetina when we don't have a win.
+                                x = x * h / 2.0
+                                y = y * h / 2.0
+                            else:
+                                x = x * h
+                                y = y * h
+                            return psychopy2displayPix(x, y)
+
+                        return x, y
+                    self._coord2pix = height2pix
+
                 elif coord_type == 'deg':
                     def pix2degcoord(self, x, y, display_index=None):
                         if display_index == self.getIndex():
@@ -639,9 +683,10 @@ class Display(Device):
                                     degy, self._psychopy_monitor))
                         return degx, degy
                     self._coord2pix = degcoord2pix
-            except ImportError:
-                print2err(
-                    'WARNING: iohub.devices.Display: cm and deg coord types only supported with psychopy.')
+            except:
+                print2err('Error during _calculateCoordMappingFunctions')
+                printExceptionDetailsToStdErr()
+
 
     def _createPsychopyCalibrationFile(self):
         display_config = self.getConfiguration()
