@@ -1,9 +1,6 @@
-"""
-ioHub Common Eye Tracker Interface for EyeLink(C) Systems.
-EyeLink(C) calibration graphics implemented using PsychoPy.
-"""
-# Part of the PsychoPy.iohub library
-# Copyright (C) 2012-2016 iSolver Software Solutions
+# -*- coding: utf-8 -*-
+# Part of the PsychoPy library
+# Copyright (C) 2012-2020 iSolver Software Solutions (C) 2021 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 import numpy as np
 from PIL import Image, ImageOps
@@ -12,62 +9,64 @@ import sys
 import tempfile
 import os
 from ..... import DeviceEvent, Computer
-from ......constants import EventConstants, KeyboardConstants
+from ......constants import EventConstants
 from ......errors import print2err, printExceptionDetailsToStdErr
-from ......util import convertCamelToSnake, win32MessagePump
+from ......util import convertCamelToSnake, win32MessagePump, updateSettings
 import pylink
 
 
-class FixationTarget(object):
-
+class FixationTarget():
     def __init__(self, psychopy_eyelink_graphics):
-        self.calibrationPointOuter = visual.Circle(
-            psychopy_eyelink_graphics.window,
+        win = psychopy_eyelink_graphics.window
+        color_type = psychopy_eyelink_graphics.getCalibSetting(['color_type'])
+        unit_type = psychopy_eyelink_graphics.getCalibSetting(['unit_type'])
+
+        outer_fill_color = outer_line_color = psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'outer_color'])
+        inner_fill_color = inner_line_color = psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'inner_color'])
+
+        if outer_fill_color is None:
+            outer_fill_color = psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'outer_fill_color'])
+            outer_line_color = psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'outer_line_color'])
+        if inner_fill_color is None:
+            inner_fill_color = psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'inner_fill_color'])
+            inner_line_color = psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'inner_line_color'])
+
+        self.calibrationPoint = visual.TargetStim(
+            win, name="CP", style="circles",
+            radius=psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'outer_diameter']) / 2.0,
+            fillColor=outer_fill_color,
+            borderColor=outer_line_color,
+            lineWidth=psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'outer_stroke_width']),
+            innerRadius=psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'inner_diameter']) / 2.0,
+            innerFillColor=inner_fill_color,
+            innerBorderColor=inner_line_color,
+            innerLineWidth=psychopy_eyelink_graphics.getCalibSetting(['target_attributes', 'inner_stroke_width']),
             pos=(0, 0),
-            lineWidth=1.0,
-            lineColor=psychopy_eyelink_graphics.CALIBRATION_POINT_OUTER_COLOR,
-            lineColorSpace='rgb255',
-            fillColor=psychopy_eyelink_graphics.CALIBRATION_POINT_OUTER_COLOR,
-            fillColorSpace='rgb255',
-            radius=psychopy_eyelink_graphics.CALIBRATION_POINT_OUTER_RADIUS,
-            name='CP_OUTER',
-            units='pix',
-            opacity=1.0,
-            interpolate=False)
-        self.calibrationPointInner = visual.Circle(
-            psychopy_eyelink_graphics.window,
-            pos=(0, 0), lineWidth=1.0,
-            lineColor=psychopy_eyelink_graphics.CALIBRATION_POINT_INNER_COLOR,
-            lineColorSpace='rgb255',
-            fillColor=psychopy_eyelink_graphics.CALIBRATION_POINT_INNER_COLOR,
-            fillColorSpace='rgb255',
-            radius=psychopy_eyelink_graphics.CALIBRATION_POINT_INNER_RADIUS,
-            name='CP_INNER',
-            units='pix',
-            opacity=1.0,
-            interpolate=False)
+            units=unit_type,
+            colorSpace=color_type,
+            autoLog=False
+        )
+        self.calibrationPointOuter = self.calibrationPoint.outer
+        self.calibrationPointInner = self.calibrationPoint.inner
 
     def draw(self, pos=None):
         if pos:
             self.calibrationPointOuter.pos = pos
             self.calibrationPointInner.pos = pos
-        self.calibrationPointOuter.draw()
-        self.calibrationPointInner.draw()
-
+        self.calibrationPoint.draw()
 
 # Intro Screen
-class BlankScreen(object):
+class BlankScreen():
 
     def __init__(self, psychopy_win, color):
         self.display_size = psychopy_win.size
         w, h = self.display_size
-        self.win = psychopy_win
+        win = psychopy_win
         self.color = color
-        self.background = visual.Rect(self.win, w, h,
+        self.background = visual.Rect(win, w, h,
                                       lineColor=self.color,
-                                      lineColorSpace='rgb255',
+                                      colorSpace=win.colorSpace,
                                       fillColor=self.color,
-                                      fillColorSpace='rgb255',
                                       units='pix',
                                       name='BACKGROUND',
                                       opacity=1.0,
@@ -78,24 +77,18 @@ class BlankScreen(object):
 
 
 # Intro Screen
-class TextLine(object):
-
-    def __init__(self, psychopy_win):
-        self.display_size = psychopy_win.size
-        self.win = psychopy_win
-
+class TextLine():
+    def __init__(self, parent):
+        self.display_size = parent.window.size
+        win = parent.window
+        tcolor, tctype = parent.getTextColorAndType()
         self.textLine = visual.TextStim(
-            self.win,
+            win,
             text='***********************',
-            pos=(
-                0,
-                0),
+            pos=(0, 0),
             height=30,
-            color=(
-                0,
-                0,
-                0),
-            colorSpace='rgb255',
+            color=tcolor,
+            colorSpace=tctype,
             opacity=1.0,
             contrast=1.0,
             units='pix',
@@ -112,54 +105,49 @@ class TextLine(object):
 
 
 # Intro Screen
-class IntroScreen(object):
+class IntroScreen():
+    def __init__(self, parent):
+        window = parent.window
+        self.display_size = window.size
+        font_height = 24
+        space_per_lines = font_height * 2.5
+        if window.useRetina:
+            topline_y = window.size[1] / 4 - font_height * 2
+        else:
+            topline_y = window.size[1] / 2 - font_height * 2
+        wrap_width = window.size[1] * .8
 
-    def __init__(self, psychopy_win):
-        self.display_size = psychopy_win.size
-        self.window = psychopy_win
-        line_count = 13
-        font_height = 30
-        space_per_lines = int(font_height * 2.5)
-        total_line_height = space_per_lines * line_count
-        topline_y = int(min(total_line_height / 1.5,
-                            self.display_size[1] / 2 - 20))
+        tcolor, tctype = parent.getTextColorAndType()
 
-        left_margin = -self.display_size[0] / 6
         self.introlines = []
 
-        self.introlines.append(
-            visual.TextStim(
-                self.window,
-                text='>>>> Eyelink System Setup:  Keyboard Actions <<<<',
-                pos=(
-                    0,
-                    topline_y),
-                height=int(
-                    font_height * 1.66),
-                color=(
-                    0,
-                    0,
-                    0),
-                colorSpace='rgb255',
-                opacity=1.0,
-                contrast=1.0,
-                units='pix',
-                ori=0.0,
-                antialias=True,
-                bold=True,
-                italic=False,
-                anchorVert='top',
-                wrapWidth=self.display_size[0] * .8))
+        self.introlines.append(visual.TextStim(window,
+                                               text='>>>> Eyelink System Setup:  Keyboard Actions <<<<',
+                                               pos=(0, topline_y),
+                                               height=font_height * 1.2,
+                                               color=tcolor,
+                                               colorSpace=tctype,
+                                               opacity=1.0,
+                                               contrast=1.0,
+                                               units='pix',
+                                               ori=0.0,
+                                               antialias=True,
+                                               bold=True,
+                                               italic=False,
+                                               wrapWidth=wrap_width))
 
-        self.introlines.append(visual.TextStim(self.window,
+        if window.useRetina:
+            left_margin = -window.size[0] / 4
+        else:
+            left_margin = -window.size[0] / 2
+        left_margin = left_margin * .4
+        topline_y = topline_y - space_per_lines / 3
+        self.introlines.append(visual.TextStim(window,
                                                text='* ENTER: Begin Camera Setup Mode',
-                                               pos=(left_margin,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
+                                               pos=(left_margin, topline_y - space_per_lines * (len(self.introlines))),
                                                height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
+                                               color=tcolor,
+                                               colorSpace=tctype,
                                                opacity=1.0,
                                                contrast=1.0,
                                                units='pix',
@@ -167,37 +155,34 @@ class IntroScreen(object):
                                                antialias=True,
                                                bold=False,
                                                italic=False,
+                                               alignText='left',
                                                anchorHoriz='left',
-                                               wrapWidth=self.display_size[0] * .8))
+                                               wrapWidth=wrap_width))
 
-        self.introlines.append(visual.TextStim(self.window,
+        self.introlines.append(visual.TextStim(window,
                                                text='* C: Start Calibration Procedure',
                                                pos=(left_margin,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
+                                                    topline_y - space_per_lines * (len(self.introlines))),
                                                height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
+                                               color=tcolor,
+                                               colorSpace=tctype,
                                                opacity=1.0,
                                                contrast=1.0,
                                                units='pix',
                                                ori=0.0,
                                                antialias=True,
-                                               bold=False,
+                                               bold=True,
                                                italic=False,
+                                               alignText='left',
                                                anchorHoriz='left',
-                                               wrapWidth=self.display_size[0] * .8))
+                                               wrapWidth=wrap_width))
 
-        self.introlines.append(visual.TextStim(self.window,
+        self.introlines.append(visual.TextStim(window,
                                                text='* V: Start Validation Procedure',
-                                               pos=(left_margin,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
+                                               pos=(left_margin, topline_y - space_per_lines * (len(self.introlines))),
                                                height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
+                                               color=tcolor,
+                                               colorSpace=tctype,
                                                opacity=1.0,
                                                contrast=1.0,
                                                units='pix',
@@ -205,55 +190,49 @@ class IntroScreen(object):
                                                antialias=True,
                                                bold=False,
                                                italic=False,
+                                               alignText='left',
                                                anchorHoriz='left',
-                                               wrapWidth=self.display_size[0] * .8))
+                                               wrapWidth=wrap_width))
 
-        self.introlines.append(visual.TextStim(self.window,
-                                               text='* ESCAPE or Q: Exit EyeLink System Setup',
-                                               pos=(left_margin,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
+        self.introlines.append(visual.TextStim(window,
+                                               text='* ESCAPE: Exit EyeLink System Setup',
+                                               pos=(left_margin, topline_y - space_per_lines * (len(self.introlines))),
                                                height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
+                                               color=tcolor,
+                                               colorSpace=tctype,
                                                opacity=1.0,
                                                contrast=1.0,
                                                units='pix',
                                                ori=0.0,
                                                antialias=True,
-                                               bold=False,
+                                               bold=True,
                                                italic=False,
+                                               alignText='left',
                                                anchorHoriz='left',
-                                               wrapWidth=self.display_size[0] * .8))
+                                               wrapWidth=wrap_width))
 
-        self.introlines.append(visual.TextStim(self.window,
-                                               text='-- Camera Setup Mode Specific Actions --',
-                                               pos=(0,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
+        topline_y = topline_y - space_per_lines / 3
+        self.introlines.append(visual.TextStim(window,
+                                               text='------ Camera Setup Mode Specific Actions ------',
+                                               pos=(0, topline_y - space_per_lines * (len(self.introlines))),
+                                               height=font_height * 1.2,
+                                               color=tcolor,
+                                               colorSpace=tctype,
+                                               opacity=1.0,
+                                               contrast=1.0,
+                                               units='pix',
+                                               ori=0.0,
+                                               antialias=True,
+                                               bold=True,
+                                               italic=False,
+                                               wrapWidth=wrap_width))
+
+        topline_y = topline_y - space_per_lines / 3
+        self.introlines.append(visual.TextStim(window, text='* Left / Right Arrow: Switch Between Camera Views',
+                                               pos=(left_margin, topline_y - space_per_lines * (len(self.introlines))),
                                                height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
-                                               opacity=1.0,
-                                               contrast=1.0,
-                                               units='pix',
-                                               ori=0.0,
-                                               antialias=True,
-                                               bold=False,
-                                               italic=False,
-                                               wrapWidth=self.display_size[0] * .8))
-
-        self.introlines.append(visual.TextStim(self.window,
-                                               text='* Left / Right Arrow: Switch Between Camera Views',
-                                               pos=(left_margin,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
-                                               height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
+                                               color=tcolor,
+                                               colorSpace=tctype,
                                                opacity=1.0,
                                                contrast=1.0,
                                                units='pix',
@@ -261,18 +240,16 @@ class IntroScreen(object):
                                                antialias=True,
                                                bold=False,
                                                italic=False,
+                                               alignText='left',
                                                anchorHoriz='left',
-                                               wrapWidth=self.display_size[0] * .8))
+                                               wrapWidth=wrap_width))
 
-        self.introlines.append(visual.TextStim(self.window,
+        self.introlines.append(visual.TextStim(window,
                                                text='* A: Auto-Threshold Image',
-                                               pos=(left_margin,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
+                                               pos=(left_margin, topline_y - space_per_lines * (len(self.introlines))),
                                                height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
+                                               color=tcolor,
+                                               colorSpace=tctype,
                                                opacity=1.0,
                                                contrast=1.0,
                                                units='pix',
@@ -280,18 +257,16 @@ class IntroScreen(object):
                                                antialias=True,
                                                bold=False,
                                                italic=False,
+                                               alignText='left',
                                                anchorHoriz='left',
-                                               wrapWidth=self.display_size[0] * .8))
+                                               wrapWidth=wrap_width))
 
-        self.introlines.append(visual.TextStim(self.window,
+        self.introlines.append(visual.TextStim(window,
                                                text='* Up / Down Arrow: Manually Adjust Pupil Threshold',
-                                               pos=(left_margin,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
+                                               pos=(left_margin, topline_y - space_per_lines * (len(self.introlines))),
                                                height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
+                                               color=tcolor,
+                                               colorSpace=tctype,
                                                opacity=1.0,
                                                contrast=1.0,
                                                units='pix',
@@ -299,18 +274,16 @@ class IntroScreen(object):
                                                antialias=True,
                                                bold=False,
                                                italic=False,
+                                               alignText='left',
                                                anchorHoriz='left',
-                                               wrapWidth=self.display_size[0] * .8))
+                                               wrapWidth=wrap_width))
 
-        self.introlines.append(visual.TextStim(self.window,
+        self.introlines.append(visual.TextStim(window,
                                                text='* + or -: Manually Adjust CR Threshold.',
-                                               pos=(left_margin,
-                                                    topline_y - space_per_lines * (len(self.introlines) + 2)),
+                                               pos=(left_margin, topline_y - space_per_lines * (len(self.introlines))),
                                                height=font_height,
-                                               color=(0,
-                                                      0,
-                                                      0),
-                                               colorSpace='rgb255',
+                                               color=tcolor,
+                                               colorSpace=tctype,
                                                opacity=1.0,
                                                contrast=1.0,
                                                units='pix',
@@ -318,8 +291,9 @@ class IntroScreen(object):
                                                antialias=True,
                                                bold=False,
                                                italic=False,
+                                               alignText='left',
                                                anchorHoriz='left',
-                                               wrapWidth=self.display_size[0] * .8))
+                                               wrapWidth=wrap_width))
 
     def draw(self):
         for s in self.introlines:
@@ -327,24 +301,32 @@ class IntroScreen(object):
 
 
 class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
-    IOHUB_HEARTBEAT_INTERVAL = 0.050   # seconds between forced run through of
-    # micro threads, since one is blocking
+    # seconds between forced run through of micro threads, since one is blocking
     # on camera setup.
+    IOHUB_HEARTBEAT_INTERVAL = 0.050
 
-    WINDOW_BACKGROUND_COLOR = (128, 128, 128)
-    CALIBRATION_POINT_OUTER_RADIUS = 15.0, 15.0
-    CALIBRATION_POINT_OUTER_EDGE_COUNT = 64
-    CALIBRATION_POINT_OUTER_COLOR = (255, 255, 255)
-    CALIBRATION_POINT_INNER_RADIUS = 3.0, 3.0
-    CALIBRATION_POINT_INNER_EDGE_COUNT = 32
-    CALIBRATION_POINT_INNER_COLOR = (25, 25, 25)
-
-    def __init__(self, eyetrackerInterface, targetForegroundColor=None,
-                 targetBackgroundColor=None, screenColor=None,
-                 targetOuterDiameter=None, targetInnerDiameter=None):
+    def __init__(self, eyetrackerInterface, calibration_args):
         pylink.EyeLinkCustomDisplay.__init__(self)
-
         self._eyetrackerinterface = eyetrackerInterface
+        display = eyetrackerInterface._display_device
+        self._device_config = self._eyetrackerinterface.getConfiguration()
+        updateSettings(self._device_config.get('calibration'), calibration_args)
+        self._calibration_args = self._device_config.get('calibration')
+        #print2err("self._calibration_args:", self._calibration_args)
+        unit_type = self.getCalibSetting('unit_type')
+        if unit_type is None:
+            unit_type = display.getCoordinateType()
+            self._calibration_args['unit_type'] = unit_type
+        color_type = self.getCalibSetting('color_type')
+        if color_type is None:
+            color_type = display.getColorSpace()
+            self._calibration_args['color_type'] = color_type
+
+        if display.getCoordinateType() != unit_type:
+            raise RuntimeWarning("EyeLink Calibration requires same unit type"
+                                 " as window {} vs {}.".format(display.getCoordinateType(),
+                                                               unit_type))
+
         self.tracker = eyetrackerInterface._eyelink
         self._ioKeyboard = None
         self._ioMouse = None
@@ -352,7 +334,7 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
         self.imgstim_size = None
         self.rgb_index_array = None
 
-        self.screenSize = self._eyetrackerinterface._display_device.getPixelResolution()
+        self.screenSize = display.getPixelResolution()
         self.width = self.screenSize[0]
         self.height = self.screenSize[1]
 
@@ -365,14 +347,6 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
         else:
             self.byteorder = 0
 
-        EyeLinkCoreGraphicsIOHubPsychopy.CALIBRATION_POINT_OUTER_COLOR = targetForegroundColor
-        EyeLinkCoreGraphicsIOHubPsychopy.CALIBRATION_POINT_INNER_COLOR = targetBackgroundColor
-        EyeLinkCoreGraphicsIOHubPsychopy.WINDOW_BACKGROUND_COLOR = screenColor
-        EyeLinkCoreGraphicsIOHubPsychopy.CALIBRATION_POINT_OUTER_RADIUS = targetOuterDiameter / \
-            2.0, targetOuterDiameter / 2.0
-        EyeLinkCoreGraphicsIOHubPsychopy.CALIBRATION_POINT_INNER_RADIUS = targetInnerDiameter / \
-            2.0, targetInnerDiameter / 2.0
-
         self.tmp_file = os.path.join(tempfile.gettempdir(), '_eleye.png')
 
         self.tracker.setOfflineMode()
@@ -384,21 +358,19 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
             self.tracker.sendCommand('autothreshold_repeat=YES')
             self.tracker.sendCommand('enable_camera_position_detect=YES')
 
-        display = self._eyetrackerinterface._display_device
         self.window = visual.Window(display.getPixelResolution(),
                                     monitor=display.getPsychopyMonitorName(),
-                                    units=display.getCoordinateType(),
-                                    color=self.WINDOW_BACKGROUND_COLOR,
-                                    colorSpace='rgb255',
+                                    units=unit_type,
+                                    color=self.getCalibSetting(['screen_background_color']),
+                                    colorSpace=color_type,
                                     fullscr=True,
                                     allowGUI=False,
                                     screen=display.getIndex()
                                     )
 
-        self.blankdisplay = BlankScreen(
-            self.window, self.WINDOW_BACKGROUND_COLOR)
-        self.textmsg = TextLine(self.window)
-        self.introscreen = IntroScreen(self.window)
+        self.blankdisplay = BlankScreen(self.window, self.getCalibSetting(['screen_background_color']))
+        self.textmsg = TextLine(self)
+        self.introscreen = IntroScreen(self)
         self.fixationpoint = FixationTarget(self)
         self.imagetitlestim = None
         self.eye_image = None
@@ -406,10 +378,35 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
         self.eye_frame_size = (0, 0)
 
         self._registerEventMonitors()
-        #self._ioMouse.setSystemCursorVisibility(False)
         self._lastMsgPumpTime = Computer.getTime()
         self.clearAllEventBuffers()
 
+    def getCalibSetting(self, setting):
+        if isinstance(setting, str):
+            setting = [setting, ]
+        calibration_args = self._calibration_args
+        if setting:
+            for s in setting[:-1]:
+                calibration_args = calibration_args.get(s)
+            return calibration_args.get(setting[-1])
+
+    def getTextColorAndType(self):
+        color_type = self.getCalibSetting('color_type')
+        if color_type is None:
+            color_type = self.window.colorSpace
+        tcolor = self.getCalibSetting(['text_color'])
+        if tcolor is None:
+            # If no calibration text color provided, base it on the window background color
+            from psychopy.iohub.util import complement
+            sbcolor = self.getCalibSetting(['screen_background_color'])
+            if sbcolor is None:
+                sbcolor = self.window.color
+            from psychopy.colors import Color
+            tcolor_obj = Color(sbcolor, color_type)
+            tcolor = complement(*tcolor_obj.rgb255)
+            color_type = 'rgb255'
+        return tcolor, color_type
+
     def clearAllEventBuffers(self):
         pylink.flushGetkeyQueue()
         self.tracker.resetData()
@@ -419,7 +416,8 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
 
     def _registerEventMonitors(self):
         self._iohub_server = self._eyetrackerinterface._iohub_server
-
+        kbDevice = None
+        mouseDevice = None
         if self._iohub_server:
             for dev in self._iohub_server.devices:
                 if dev.__class__.__name__ == 'Keyboard':
@@ -430,13 +428,7 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
         if kbDevice:
             eventIDs = []
             for event_class_name in kbDevice.__class__.EVENT_CLASS_NAMES:
-                eventIDs.append(
-                    getattr(
-                        EventConstants,
-                        convertCamelToSnake(
-                            event_class_name[
-                                :-5],
-                            False)))
+                eventIDs.append(getattr(EventConstants, convertCamelToSnake(event_class_name[:-5], False)))
 
             self._ioKeyboard = kbDevice
             self._ioKeyboard._addEventListener(self, eventIDs)
@@ -447,13 +439,7 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
         if mouseDevice:
             eventIDs = []
             for event_class_name in mouseDevice.__class__.EVENT_CLASS_NAMES:
-                eventIDs.append(
-                    getattr(
-                        EventConstants,
-                        convertCamelToSnake(
-                            event_class_name[
-                                :-5],
-                            False)))
+                eventIDs.append(getattr(EventConstants, convertCamelToSnake(event_class_name[:-5], False)))
 
             self._ioMouse = mouseDevice
             self._ioMouse._addEventListener(self, eventIDs)
@@ -474,7 +460,7 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
             key_index = KeyboardInputEvent.CLASS_ATTRIBUTE_NAMES.index('key')
             char = event[key_index]
             if isinstance(char, bytes):
-                char = str(event[key_index],'utf-8')
+                char = str(event[key_index], 'utf-8')
 
             if char:
                 char = char.lower()
@@ -488,8 +474,8 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
             elif char == 'return':
                 pylink_key = pylink.ENTER_KEY
                 self.state = None
-            elif char == ' ':
-                pylink_key = ord(char)
+            elif char == ' ' or char == 'space':
+                pylink_key = ord(' ')
             elif char == 'c':
                 pylink_key = ord(char)
                 self.state = 'calibration'
@@ -498,14 +484,16 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
                 self.state = 'validation'
             elif char == 'a':
                 pylink_key = ord(char)
+            elif char == 'o':
+                pylink_key = ord(char)
             elif char == 'pageup':
                 pylink_key = pylink.PAGE_UP
             elif char == 'pagedown':
                 pylink_key = pylink.PAGE_DOWN
-            elif char == '-':
-                pylink_key = ord(char)
-            elif char == '=':
-                pylink_key = ord(char)
+            elif char == '-' or char == 'minus':
+                pylink_key = ord('-')
+            elif char == '=' or char == 'equal':
+                pylink_key = ord('=')
             elif char == 'up':
                 pylink_key = pylink.CURS_UP
             elif char == 'down':
@@ -565,8 +553,8 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
     def draw_cal_target(self, x, y):
         """Draws calibration target."""
         # convert to psychopy pix coords
-        x = x - self.window.size[0] / 2
-        y = -(y - self.window.size[1] / 2)
+        x, y = self._eyetrackerinterface._eyeTrackerToDisplayCoords((x, y))
+
         self.blankdisplay.draw()
         self.fixationpoint.draw((x, y))
         self.window.flip()
@@ -579,7 +567,7 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
             self.clear_cal_display()
             self.last_mouse_state = -1
             if self.rgb_index_array is None:
-                self.rgb_index_array = np.zeros((int(height/2), int(width/2)), dtype=np.uint8)
+                self.rgb_index_array = np.zeros((int(height / 2), int(width / 2)), dtype=np.uint8)
 
     def exit_image_display(self):
         """Exits the image display."""
@@ -589,18 +577,14 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
         """Display the current camera, Pupil, and CR thresholds above the
         camera image when in Camera Setup Mode."""
         if self.imagetitlestim is None:
+            tcolor, tctype = self.getTextColorAndType()
             self.imagetitlestim = visual.TextStim(
                 self.window,
                 text=text,
-                pos=(
-                    0,
-                    self.window.size[1] / 2 - 15),
+                pos=(0, self.window.size[1] / 2 - 15),
                 height=28,
-                color=(
-                    0,
-                    0,
-                    0),
-                colorSpace='rgb255',
+                color=tcolor,
+                colorSpace=tctype,
                 opacity=1.0,
                 contrast=1.0,
                 units='pix',
@@ -632,7 +616,7 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
         if line == totlines:
             try:
                 image = Image.fromarray(self.rgb_index_array,
-                                           mode='P')
+                                        mode='P')
                 image.putpalette(self.rgb_pallete)
                 image = ImageOps.fit(image, [640, 480])
                 if self.eye_image is None:
@@ -653,7 +637,7 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
                 printExceptionDetailsToStdErr()
 
     def set_image_palette(self, r, g, b):
-        """Set color palette ued by host pc when sending images.
+        """Set color palette used by host pc when sending images.
 
         Saves the different r,g,b values provided by the eyelink host
         palette. When building up each eye image frame, eyelink sends
@@ -673,7 +657,7 @@ class EyeLinkCoreGraphicsIOHubPsychopy(pylink.EyeLinkCustomDisplay):
         """Prints alert message to psychopy stderr."""
         print2err('eyelink_graphics.alert_printf(): %s' % msg)
 
-###
+    ###
 
     def play_beep(self, pylink_sound_index):
         """
