@@ -1,10 +1,7 @@
 # -*- coding: utf-8 -*-
-# Part of the psychopy.iohub library.
-# Copyright (C) 2012-2016 iSolver Software Solutions
+# Part of the PsychoPy library
+# Copyright (C) 2012-2020 iSolver Software Solutions (C) 2021 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
-from __future__ import division, absolute_import
-
-from past.builtins import xrange
 
 import os
 import sys
@@ -16,15 +13,14 @@ import gevent
 from gevent.server import DatagramServer
 from gevent import Greenlet
 
+import numpy
+
 try:
     import msgpack_numpy
     msgpack_numpy.patch()
 except ImportError:
     pass
 
-from past.builtins import basestring, unicode
-from psychopy.constants import PY3
-from . import _pkgroot
 from . import IOHUB_DIRECTORY, EXP_SCRIPT_DIRECTORY, _DATA_STORE_AVAILABLE
 from .errors import print2err, printExceptionDetailsToStdErr, ioHubError
 from .net import MAX_PACKET_SIZE
@@ -36,20 +32,19 @@ from .devices import Computer
 from .devices.deviceConfigValidation import validateDeviceConfiguration
 getTime = Computer.getTime
 
-MAX_PACKET_SIZE = 64 * 1024
-
 # pylint: disable=protected-access
 # pylint: disable=broad-except
 
-def convertByteKeysToStr(rdict):
+def convertByteStrings(rdict):
     if rdict is None or len(rdict)==0:
         return rdict
     result = dict()
-    for k, i in rdict.items():
+    for k, i in rdict.items(): 
         if isinstance(k, bytes):
-            result[k.decode('utf-8')] = i
-        else:
-            result[k] = i
+            k = k.decode('utf-8')
+        if isinstance(i, bytes):
+            i = i.decode('utf-8')
+        result[k] = i
     return result
 
 class udpServer(DatagramServer):
@@ -65,17 +60,34 @@ class udpServer(DatagramServer):
         self.unpacker = msgpack.Unpacker(use_list=True)
         self.unpack = self.unpacker.unpack
         self.feed = self.unpacker.feed
+        self.multipacket_reads = 0
         DatagramServer.__init__(self, address)
 
     def handle(self, request, replyTo):
         if self._running is False:
             return False
+
         self.feed(request)
+
+        if self.multipacket_reads > 0:
+            # Multi packet request handling...
+            self.multipacket_reads -= 1
+            if self.multipacket_reads > 0:
+                # If reading part of multi packet request, just return and wait for next part of request
+                return False
+
         request = self.unpack()
-        # print2err(">> Rx Packet: {}, {}".format(request, replyTo))
+
+        if request[0] == 'IOHUB_MULTIPACKET_REQUEST':
+            # setup multi packet request read
+            self.multipacket_reads = request[1]
+            return False
+        else:
+            self.multipacket_reads = 0
+
         request_type = request.pop(0)
-        if not isinstance(request_type, unicode):
-            request_type = unicode(request_type, 'utf-8') # convert bytes to string for compatibility
+        if not isinstance(request_type, str):
+            request_type = str(request_type, 'utf-8') # convert bytes to string for compatibility
 
         if request_type == 'SYNC_REQ':
             self.sendResponse(['SYNC_REPLY', getTime()], replyTo)
@@ -105,10 +117,9 @@ class udpServer(DatagramServer):
 
             result = None
             try:
-                result = getattr(self, unicode(callable_name, 'utf-8'))
-            except TypeError as e:
-                if "decoding str is not supported" in e:
-                    pass
+                if isinstance(callable_name, bytes):
+                    callable_name = callable_name.decode('utf-8')
+                result = getattr(self, callable_name)
             except Exception:
                 print2err('RPC_ATTRIBUTE_ERROR')
                 printExceptionDetailsToStdErr()
@@ -117,15 +128,24 @@ class udpServer(DatagramServer):
 
             if result and callable(result):
                 funcPtr = result
+                nargs = []
+                if args:
+                    for a in args:
+                        if isinstance(a, bytes):
+                            nargs.append(a.decode('utf-8'))
+                        else:
+                            nargs.append(a)
+                    args = nargs
+                    
                 try:
                     if args is None and kwargs is None:
                         result = funcPtr()
                     elif args and kwargs:
-                        result = funcPtr(*args, **convertByteKeysToStr(kwargs))
+                        result = funcPtr(*args, **convertByteStrings(kwargs))
                     elif args and not kwargs:
                         result = funcPtr(*args)
                     elif not args and kwargs:
-                        result = funcPtr(**convertByteKeysToStr(kwargs))
+                        result = funcPtr(**convertByteStrings(kwargs))
                     edata = ('RPC_RESULT', callable_name, result)
                     self.sendResponse(edata, replyTo)
                     return True
@@ -172,7 +192,7 @@ class udpServer(DatagramServer):
                 class_kwargs = {}
                 if len(request):
                     class_kwargs = request.pop(0)
-                custom_tasks[tasklet_label] = task_cls(**convertByteKeysToStr(class_kwargs))
+                custom_tasks[tasklet_label] = task_cls(**convertByteStrings(class_kwargs))
                 custom_tasks[tasklet_label].start()
             except Exception:
                 print2err(
@@ -216,8 +236,8 @@ class udpServer(DatagramServer):
 
     def handleExperimentDeviceRequest(self, request, replyTo):
         request_type = request.pop(0)
-        if not isinstance(request_type, unicode):
-            request_type = unicode(request_type, 'utf-8') # convert bytes to string for compatibility
+        if not isinstance(request_type, str):
+            request_type = str(request_type, 'utf-8') # convert bytes to string for compatibility
         io_dev_dict = ioServer.deviceDict
         if request_type == 'EVENT_TX':
             exp_events = request.pop(0)
@@ -228,11 +248,11 @@ class udpServer(DatagramServer):
             return True
         elif request_type == 'DEV_RPC':
             dclass = request.pop(0)
-            if not isinstance(dclass, unicode):
-                dclass = unicode(dclass, 'utf-8')
+            if not isinstance(dclass, str):
+                dclass = str(dclass, 'utf-8')
             dmethod = request.pop(0)
-            if not isinstance(dmethod, unicode):
-                dmethod = unicode(dmethod, 'utf-8')
+            if not isinstance(dmethod, str):
+                dmethod = str(dmethod, 'utf-8')
             args = None
             kwargs = None
             if len(request) == 1:
@@ -269,13 +289,14 @@ class udpServer(DatagramServer):
             result = []
             try:
                 if args and kwargs:
-                    result = method(*args, **convertByteKeysToStr(kwargs))
+                    result = method(*args, **convertByteStrings(kwargs))
                 elif args:
                     result = method(*args)
                 elif kwargs:
-                    result = method(**convertByteKeysToStr(kwargs))
+                    result = method(**convertByteStrings(kwargs))
                 else:
                     result = method()
+                #print2err("DEV_RPC_RESULT: ", result)
                 self.sendResponse(('DEV_RPC_RESULT', result), replyTo)
                 return True
             except Exception:
@@ -300,8 +321,8 @@ class udpServer(DatagramServer):
 
         elif request_type == 'GET_DEV_INTERFACE':
             dclass = request.pop(0)
-            if not isinstance(dclass, unicode):
-                dclass = unicode(dclass, 'utf-8')
+            if not isinstance(dclass, str):
+                dclass = str(dclass, 'utf-8')
             data = None
             if dclass in ['EyeTracker', 'DAQ']:
                 for dname, hdevice in ioServer.deviceDict.items():
@@ -353,9 +374,12 @@ class udpServer(DatagramServer):
                 pkt_cnt = int(reply_data_sz // max_pkt_sz) + 1
                 mpr_payload = ('IOHUB_MULTIPACKET_RESPONSE', pkt_cnt)
                 self.sendResponse(mpr_payload, address)
-                for p in xrange(pkt_cnt - 1):
+                gevent.sleep(0.0001)
+                for p in range(pkt_cnt - 1):
                     si = p*max_pkt_sz
                     self.socket.sendto(reply_data[si:si+max_pkt_sz], address)
+                    # macOS hangs if we do not sleep gevent between each msg packet
+                    gevent.sleep(0.0001)
                 si = (p+1)*max_pkt_sz
                 self.socket.sendto(reply_data[si:reply_data_sz], address)
             else:
@@ -394,19 +418,43 @@ class udpServer(DatagramServer):
     def registerWindowHandles(self, *win_hwhds):
         if self.iohub:
             for wh in win_hwhds:
-                if wh not in self.iohub._pyglet_window_hnds:
-                    self.iohub._pyglet_window_hnds.append(wh)
+                if wh['handle'] not in self.iohub._psychopy_windows.keys():
+                    self.iohub._psychopy_windows[wh['handle']] = wh
+                    wh['size'] = numpy.asarray(wh['size'])
+                    wh['pos'] = numpy.asarray(wh['pos'])
+                    if wh['monitor']:
+                        from psychopy import monitors
+                        monitor = wh['monitor']
+                        monitor['monitor'] = monitors.Monitor('{}'.format(wh['handle']))
+                        monitor['monitor'].setDistance(monitor['distance'])
+                        monitor['monitor'].setWidth(monitor['width'])
+                        monitor['monitor'].setSizePix(monitor['resolution'])
+                    self.iohub.log('Registered Win: {}'.format(wh))
 
     def unregisterWindowHandles(self, *win_hwhds):
         if self.iohub:
             for wh in win_hwhds:
-                if wh in self.iohub._pyglet_window_hnds:
-                    self.iohub._pyglet_window_hnds.remove(wh)
+                if wh in self.iohub._psychopy_windows.keys():
+                    del self.iohub._psychopy_windows[wh]
+                    self.iohub.log('Removed Win: {}'.format(wh))
+
+    def updateWindowPos(self, win_hwhd, pos):
+        """
+        Update stored psychopy window position.
+        :param win_hwhd:
+        :param pos:
+        :return:
+        """
+        winfo = self.iohub._psychopy_windows.get(win_hwhd)
+        if winfo:
+            winfo['pos'] = pos
+            self.iohub.log('Update Win: {}'.format(winfo))
+        else:
+            print2err('warning: win_hwhd {} not registered with iohub server.'.format(win_hwhd))
+            self.iohub.log('updateWindowPos warning: win_hwhd {} not registered with iohub server.'.format(win_hwhd))
 
     def createExperimentSessionEntry(self, sessionInfoDict):
-        if PY3:
-            sessionInfoDict = {str(k, 'utf-8'): str(v, 'utf-8')
-                               for k, v in sessionInfoDict.items()}
+        sessionInfoDict = convertByteStrings(sessionInfoDict)
         self.iohub.sessionInfoDict = sessionInfoDict
         dsfile = self.iohub.dsfile
         if dsfile:
@@ -421,7 +469,7 @@ class udpServer(DatagramServer):
         if dsfile:
             output = []
             for a in numpy_dtype:
-                if isinstance(a[1], basestring):
+                if isinstance(a[1], str):
                     output.append(tuple(a))
                 else:
                     temp = [a[0], []]
@@ -511,20 +559,17 @@ class DeviceMonitor(Greenlet):
             stime = ctime()
             self.device._poll()
             i = self.sleep_interval - (ctime() - stime)
-            if i > 0.001:
-                gevent.sleep(i)
-            else:
-                gevent.sleep(0.001)
+            gevent.sleep(max(0,i))
 
     def __del__(self):
         self.device = None
 
 
-class ioServer(object):
+class ioServer():
     eventBuffer = None
     deviceDict = {}
     _logMessageBuffer = deque(maxlen=128)
-    _pyglet_window_hnds = []
+    _psychopy_windows = {}
     status = 'OFFLINE'
     def __init__(self, rootScriptPathDir, config=None):
         self._session_id = None
@@ -616,14 +661,15 @@ class ioServer(object):
             print2err('Error PubSub Device listener association ....')
             printExceptionDetailsToStdErr()
 
-    def processDeviceConfigDictionary(self, dev_mod_path, dev_cls_name,
-                                      dev_conf, def_dev_conf):
+    def processDeviceConfigDictionary(self, dev_mod_path, dev_cls_name, dev_conf, def_dev_conf):
         for dparam, dvalue in def_dev_conf.items():
-            if dparam not in dev_conf:
+            if dparam in dev_conf:
+                if isinstance(dvalue, (dict, OrderedDict)):
+                    self.processDeviceConfigDictionary(None, None, dev_conf.get(dparam), dvalue)
+            elif dparam not in dev_conf:
                 if isinstance(dvalue, (dict, OrderedDict)):
                     sub_param = dict()
-                    self.processDeviceConfigDictionary(None, None, sub_param,
-                                                       dvalue)
+                    self.processDeviceConfigDictionary(None, None, sub_param, dvalue)
                     dev_conf[dparam] = sub_param
                 else:
                     dev_conf[dparam] = dvalue
@@ -650,7 +696,7 @@ class ioServer(object):
                 self._running = False
                 break
             dur = sleep_interval - (Computer.getTime() - stime)
-            gevent.sleep(max(0.001, dur))
+            gevent.sleep(max(0.0, dur))
 
     def createNewMonitoredDevice(self, dev_cls_name, dev_conf):
         self._all_dev_conf_errors = dict()
@@ -727,7 +773,6 @@ class ioServer(object):
             elif Computer.platform.startswith('linux'):
                 from .devices import pyXHook
                 if hookManager is None:
-                    # iohub.log("Creating pyXHook Monitors....")
                     log_evt = self.config.get('log_raw_kb_mouse_events', False)
                     self._hookManager = pyXHook.HookManager(log_evt)
                     hookManager = self._hookManager
@@ -755,15 +800,15 @@ class ioServer(object):
                 if self._hookDevice is None:
                     self._hookDevice = []
                 if dev_cls_name not in self._hookDevice:
+                    msgpump_interval = self.config.get('msgpump_interval', 0.001)
                     if dev_cls_name == 'Mouse':
                         dmouse = deviceDict['Mouse']
-                        mouseHookMonitor = DeviceMonitor(dmouse, 0.004)
-                        self.deviceMonitors.append(mouseHookMonitor)
+                        self.deviceMonitors.append(DeviceMonitor(dmouse, msgpump_interval))
                         dmouse._CGEventTapEnable(dmouse._tap, True)
                         self._hookDevice.append('Mouse')
                     if dev_cls_name == 'Keyboard':
                         dkeyboard = deviceDict['Keyboard']
-                        kbHookMonitor = DeviceMonitor(dkeyboard, 0.004)
+                        kbHookMonitor = DeviceMonitor(dkeyboard, 0.001)
                         self.deviceMonitors.append(kbHookMonitor)
                         dkeyboard._CGEventTapEnable(dkeyboard._tap, True)
                         self._hookDevice.append('Keyboard')
@@ -776,7 +821,7 @@ class ioServer(object):
 
         DeviceClass = None
         cls_name_start = dev_cls_name.rfind('.')
-        iohub_submod = '%s.' % _pkgroot
+        iohub_submod = 'psychopy.iohub.'
         iohub_submod_len = len(iohub_submod)
         dev_mod_pth = iohub_submod + 'devices.'
         if cls_name_start > 0:
@@ -894,7 +939,7 @@ class ioServer(object):
             stime = Computer.getTime()
             self.processDeviceEvents()
             dur = sleep_interval - (Computer.getTime() - stime)
-            gevent.sleep(max(0.001, dur))
+            gevent.sleep(max(0, dur))
 
     def processDeviceEvents(self):
         for device in self.devices:
