@@ -2,19 +2,27 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """Container for all visual-related functions and classes
 """
 
-from __future__ import absolute_import, print_function
-
 import sys
 if sys.platform == 'win32':
     from pyglet.libs import win32  # pyglet patch for ANACONDA install
     from ctypes import *
+    from psychopy import prefs
     win32.PUINT = POINTER(wintypes.UINT)
+    # get the preference for high DPI
+    if 'highDPI' in prefs.hardware.keys():  # check if we have the option
+        enableHighDPI = prefs.hardware['highDPI']
+        # check if we have OS support for it
+        if enableHighDPI:
+            try:
+                windll.shcore.SetProcessDpiAwareness(enableHighDPI)
+            except OSError:
+                pass
 
 from psychopy import event  # import before visual or
 from psychopy.visual import filters
@@ -26,9 +34,11 @@ from .helpers import pointInPolygon, polygonsOverlap
 from .image import ImageStim
 from .text import TextStim
 from .form import Form
-from .button import ButtonStim
 from .brush import Brush
 from .textbox2.textbox2 import TextBox2
+from .button import ButtonStim
+from .roi import ROI
+from .target import TargetStim
 # window, should always be loaded first
 from .window import Window, getMsPerFrame, openWindows
 
@@ -87,6 +97,9 @@ from psychopy.visual.textbox import TextBox
 # rift support 
 from psychopy.visual.rift import Rift
 
+# VisualSystemHD support
+from psychopy.visual.nnlvs import VisualSystemHD
+
 # 3D stimuli support
 from psychopy.visual.stim3d import LightSource
 from psychopy.visual.stim3d import SceneSkybox
