@@ -6,16 +6,17 @@ in either dimension. One of the main stimuli for PsychoPy.
 """
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, division, print_function
-
 # Ensure setting pyglet.options['debug_gl'] to False is done prior to any
 # other calls to pyglet or pyglet submodules, otherwise it may not get picked
 # up by the pyglet GL engine and have no effect.
 # Shaders will work but require OpenGL2.0 drivers AND PyOpenGL3.0+
 import pyglet
+
+from psychopy.colors import Color
+
 pyglet.options['debug_gl'] = False
 import ctypes
 GL = pyglet.gl
@@ -70,7 +71,8 @@ class GratingStim(BaseVisualStim, TextureMixin, ColorMixin, ContainerMixin):
                  win,
                  tex="sin",
                  mask="none",
-                 units="",
+                 units=None,
+                 anchor="center",
                  pos=(0.0, 0.0),
                  size=None,
                  sf=None,
@@ -83,7 +85,7 @@ class GratingStim(BaseVisualStim, TextureMixin, ColorMixin, ContainerMixin):
                  color=(1.0, 1.0, 1.0),
                  colorSpace='rgb',
                  contrast=1.0,
-                 opacity=1.0,
+                 opacity=None,
                  depth=0,
                  rgbPedestal=(0.0, 0.0, 0.0),
                  interpolate=False,
@@ -101,8 +103,7 @@ class GratingStim(BaseVisualStim, TextureMixin, ColorMixin, ContainerMixin):
         # initialise parent class
         super(GratingStim, self).__init__(win, units=units, name=name,
                                           autoLog=False)
-        # use shaders if available by default, this is a good thing
-        self.__dict__['useShaders'] = win._haveShaders
+
         # UGLY HACK: Some parameters depend on each other for processing.
         # They are set "superficially" here.
         # TO DO: postpone calls to _createTexture, setColor and
@@ -126,36 +127,36 @@ class GratingStim(BaseVisualStim, TextureMixin, ColorMixin, ContainerMixin):
         self.rgbPedestal = val2array(rgbPedestal, False, length=3)
         # No need to invoke decorator for color updating. It is done just
         # below.
-        self.__dict__['colorSpace'] = colorSpace
+        self.colorSpace = colorSpace
+        self.color = color
         if rgb != None:
             logging.warning("Use of rgb arguments to stimuli are deprecated."
                             " Please use color and colorSpace args instead")
-            self.setColor(rgb, colorSpace='rgb', log=False)
+            self.color = Color(rgb, 'rgb')
         elif dkl != None:
             logging.warning("Use of dkl arguments to stimuli are deprecated."
                             " Please use color and colorSpace args instead")
-            self.setColor(dkl, colorSpace='dkl', log=False)
+            self.color = Color(dkl, 'dkl')
         elif lms != None:
             logging.warning("Use of lms arguments to stimuli are deprecated."
                             " Please use color and colorSpace args instead")
-            self.setColor(lms, colorSpace='lms', log=False)
-        else:
-            self.setColor(color, colorSpace=colorSpace, log=False)
+            self.color = Color(lms, 'lms')
 
         # set other parameters
         self.ori = float(ori)
         self.phase = val2array(phase, False)
         self._origSize = None  # updated if an image texture is loaded
         self._requestedSize = size
-        self.size = val2array(size)
+        self.size = size
         self.sf = val2array(sf)
         self.pos = val2array(pos, False, False)
         self.depth = depth
+        self.anchor = anchor
 
         self.tex = tex
         self.mask = mask
         self.contrast = float(contrast)
-        self.opacity = float(opacity)
+        self.opacity = opacity
         self.autoLog = autoLog
         self.autoDraw = autoDraw
         self.blendmode=blendmode
@@ -187,14 +188,10 @@ class GratingStim(BaseVisualStim, TextureMixin, ColorMixin, ContainerMixin):
     def sf(self, value):
         """Spatial frequency of the grating texture
 
-        Should be a :ref:`x,y-pair <attrib-xy>` or
-        :ref:`scalar <attrib-scalar>` or None.
-        If `units` == 'deg' or 'cm' units are in
-            cycles per deg or cm as appropriate.
-        If `units` == 'norm' then sf units are in cycles per stimulus
-            (and so SF scales with stimulus size).
-        If texture is an image loaded from a file then sf=None
-            defaults to 1/stimSize to give one cycle of the image.
+        Should be a :ref:`x,y-pair <attrib-xy>` or :ref:`scalar <attrib-scalar>` or None.
+        If `units` == 'deg' or 'cm' units are in cycles per deg or cm as appropriate.
+        If `units` == 'norm' then sf units are in cycles per stimulus (and so SF scales with stimulus size).
+        If texture is an image loaded from a file then sf=None defaults to 1/stimSize to give one cycle of the image.
         """
 
         # Recode phase to numpy array
@@ -230,6 +227,43 @@ class GratingStim(BaseVisualStim, TextureMixin, ColorMixin, ContainerMixin):
         self.__dict__['phase'] = value
         self._needUpdate = True
 
+    # overload ColorMixin methods so that they refresh the image after being called
+    @property
+    def foreColor(self):
+        # Call setter of parent mixin
+        return ColorMixin.foreColor.fget(self)
+    @foreColor.setter
+    def foreColor(self, value):
+        # Call setter of parent mixin
+        ColorMixin.foreColor.fset(self, value)
+        # Reset texture
+        self._needTextureUpdate = True
+        self._needUpdate = True
+
+    @property
+    def contrast(self):
+        # Call setter of parent mixin
+        return ColorMixin.contrast.fget(self)
+    @contrast.setter
+    def contrast(self, value):
+        # Call setter of parent mixin
+        ColorMixin.contrast.fset(self, value)
+        # Reset texture
+        self._needTextureUpdate = True
+        self._needUpdate = True
+
+    @property
+    def opacity(self):
+        # Call setter of parent mixin
+        return BaseVisualStim.opacity.fget(self)
+    @opacity.setter
+    def opacity(self, value):
+        # Call setter of parent mixin
+        BaseVisualStim.opacity.fset(self, value)
+        # Reset texture
+        self._needTextureUpdate = True
+        self._needUpdate = True
+
     @attributeSetter
     def tex(self, value):
         """Texture to used on the stimulus as a grating (aka carrier)
@@ -303,11 +337,7 @@ class GratingStim(BaseVisualStim, TextureMixin, ColorMixin, ContainerMixin):
         GL.glPushMatrix()  # push before the list, pop after
         win.setScale('pix')
         # the list just does the texture mapping
-
-        desiredRGB = self._getDesiredRGB(self.rgb, self.colorSpace,
-                                         self.contrast)
-        GL.glColor4f(desiredRGB[0], desiredRGB[1], desiredRGB[2],
-                     self.opacity)
+        GL.glColor4f(*self._foreColor.render('rgba1'))
 
         if self._needTextureUpdate:
             self.setTex(value=self.tex, log=False)
@@ -386,71 +416,6 @@ class GratingStim(BaseVisualStim, TextureMixin, ColorMixin, ContainerMixin):
 
         GL.glEndList()
 
-    # for the sake of older graphics cards------------------------------------
-    def _updateListNoShaders(self):
-        """The user shouldn't need this method since it gets called
-        after every call to .set() Basically it updates the OpenGL
-        representation of your stimulus if some parameter of the
-        stimulus changes. Call it if you change a property manually
-        rather than using the .set() command
-        """
-        self._needUpdate = False
-
-        GL.glNewList(self._listID, GL.GL_COMPILE)
-        # glColor can interfere with multitextures
-        GL.glColor4f(1.0, 1.0, 1.0, 1.0)
-        # mask
-        GL.glActiveTexture(GL.GL_TEXTURE1)
-        GL.glEnable(GL.GL_TEXTURE_2D)
-        GL.glBindTexture(GL.GL_TEXTURE_2D, self._maskID)
-
-        # main texture
-        GL.glActiveTexture(GL.GL_TEXTURE0)
-        GL.glEnable(GL.GL_TEXTURE_2D)
-        GL.glBindTexture(GL.GL_TEXTURE_2D, self._texID)
-
-        # depth = self.depth
-        Ltex = (-self._cycles[0] / 2) - self.phase[0] + 0.5
-        Rtex = (+self._cycles[0] / 2) - self.phase[0] + 0.5
-        Ttex = (+self._cycles[1] / 2) - self.phase[1] + 0.5
-        Btex = (-self._cycles[1] / 2) - self.phase[1] + 0.5
-        Lmask = Bmask = 0.0
-        Tmask = Rmask = 1.0  # mask
-
-        # access just once because it's slower than basic property
-        vertsPix = self.verticesPix
-        GL.glBegin(GL.GL_QUADS)  # draw a 4 sided polygon
-        # right bottom
-        GL.glMultiTexCoord2f(GL.GL_TEXTURE0, Rtex, Btex)
-        GL.glMultiTexCoord2f(GL.GL_TEXTURE1, Rmask, Bmask)
-        GL.glVertex2f(vertsPix[0, 0], vertsPix[0, 1])
-        # left bottom
-        GL.glMultiTexCoord2f(GL.GL_TEXTURE0, Ltex, Btex)
-        GL.glMultiTexCoord2f(GL.GL_TEXTURE1, Lmask, Bmask)
-        GL.glVertex2f(vertsPix[1, 0], vertsPix[1, 1])
-        # left top
-        GL.glMultiTexCoord2f(GL.GL_TEXTURE0, Ltex, Ttex)
-        GL.glMultiTexCoord2f(GL.GL_TEXTURE1, Lmask, Tmask)
-        GL.glVertex2f(vertsPix[2, 0], vertsPix[2, 1])
-        # right top
-        GL.glMultiTexCoord2f(GL.GL_TEXTURE0, Rtex, Ttex)
-        GL.glMultiTexCoord2f(GL.GL_TEXTURE1, Rmask, Tmask)
-        GL.glVertex2f(vertsPix[3, 0], vertsPix[3, 1])
-        GL.glEnd()
-
-        # disable mask
-        GL.glActiveTexture(GL.GL_TEXTURE1)
-        GL.glDisable(GL.GL_TEXTURE_2D)
-        GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
-
-        # main texture
-        GL.glActiveTexture(GL.GL_TEXTURE0)
-        GL.glDisable(GL.GL_TEXTURE_2D)
-        GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
-
-        # we're done!
-        GL.glEndList()
-
     def __del__(self):
         try:
             GL.glDeleteLists(self._listID, 1)
