@@ -58,14 +58,15 @@ Testing has only been done on Windows and Linux so far.
 """
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 #
 # Contributed by Sol Simpson, April 2014.
 # The MovieStim class was taken and rewritten to use cv2 and vlc instead
 # of avbin
 
-from __future__ import absolute_import, division, print_function
+
+
 
 # If True then, on each flip a new movie frame is displayed, the frame index,
 # flip time, and time since last movie frame flip will be printed
@@ -114,7 +115,7 @@ except Exception as err:
         bits = 64
     else:
         bits = 32
-    if "wrong architecture" in err:
+    if "wrong architecture" in str(err):
         msg = ("Failed to import vlc module for MovieStim2.\n"
                "You're using %i-bit python. Is your VLC install the same?"
                % bits)
@@ -187,6 +188,7 @@ class MovieStim2(BaseVisualStim, ContainerMixin):
                  units='pix',
                  size=None,
                  pos=(0.0, 0.0),
+                 anchor="center",
                  ori=0.0,
                  flipVert=False,
                  flipHoriz=False,
@@ -241,6 +243,7 @@ class MovieStim2(BaseVisualStim, ContainerMixin):
         self.flipVert = flipVert
         self.flipHoriz = flipHoriz
         self.pos = numpy.asarray(pos, float)
+        self.anchor = anchor
         self.depth = depth
         self.opacity = float(opacity)
         self.volume = volume
