@@ -2,14 +2,12 @@
 # -*- coding: utf-8 -*-
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 """shaders programs for either pyglet or pygame
 """
 
-from __future__ import absolute_import, print_function
-
 import pyglet.gl as GL
 import psychopy.tools.gltools as gltools
 from ctypes import c_int, c_char_p, c_char, cast, POINTER, byref
@@ -37,7 +35,6 @@ class Shader:
             status = c_int()
             GL.glGetShaderiv(shader, GL.GL_COMPILE_STATUS, byref(status))
             if not status.value:
-                print_log(shader)
                 GL.glDeleteShader(shader)
                 raise ValueError('Shader compilation failed')
             return shader
