@@ -4,43 +4,53 @@
 """Create geometric (vector) shapes by defining vertex locations."""
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL)
 
-from __future__ import absolute_import, print_function
-
-from builtins import str
-from past.builtins import basestring
+import copy
+import numpy
 
 # Ensure setting pyglet.options['debug_gl'] to False is done prior to any
 # other calls to pyglet or pyglet submodules, otherwise it may not get picked
 # up by the pyglet GL engine and have no effect.
 # Shaders will work but require OpenGL2.0 drivers AND PyOpenGL3.0+
 import pyglet
-pyglet.options['debug_gl'] = False
-GL = pyglet.gl
 
 import psychopy  # so we can get the __path__
 from psychopy import logging
+from psychopy.colors import Color
 
 # tools must only be imported *after* event or MovieStim breaks on win32
 # (JWP has no idea why!)
-from psychopy.tools.monitorunittools import cm2pix, deg2pix
-from psychopy.tools.attributetools import (attributeSetter, logAttrib,
+# from psychopy.tools.monitorunittools import cm2pix, deg2pix
+from psychopy.tools.attributetools import (attributeSetter,  # logAttrib,
                                            setAttribute)
 from psychopy.tools.arraytools import val2array
 from psychopy.visual.basevisual import (BaseVisualStim, ColorMixin,
-                                        ContainerMixin)
-from psychopy.visual.helpers import setColor
-
+                                        ContainerMixin, WindowMixin)
+# from psychopy.visual.helpers import setColor
+import psychopy.visual
 from psychopy.contrib import tesselate
-import copy
-import numpy
+
+pyglet.options['debug_gl'] = False
+GL = pyglet.gl
 
 
 knownShapes = {
-    "cross" :  [
-        (-0.1, +0.5), # up
+    "triangle": [
+        (+0.0, 0.5),  # Point
+        (-0.5, -0.5),  # Bottom left
+        (+0.5, -0.5),  # Bottom right
+    ],
+    "rectangle": [
+        [-.5,  .5],  # Top left
+        [ .5,  .5],  # Top right
+        [ .5, -.5],  # Bottom left
+        [-.5, -.5],  # Bottom right
+    ],
+    "circle": 100,  # Make 100 point equilateral
+    "cross": [
+        (-0.1, +0.5),  # up
         (+0.1, +0.5),
         (+0.1, +0.1),
         (+0.5, +0.1),  # right
@@ -53,12 +63,25 @@ knownShapes = {
         (-0.5, +0.1),
         (-0.1, +0.1),
     ],
-    "star7" : [(0.0,0.5),(0.09,0.18),(0.39,0.31),(0.19,0.04),
-             (0.49,-0.11),(0.16,-0.12),(0.22,-0.45),(0.0,-0.2),
-             (-0.22,-0.45),(-0.16,-0.12),(-0.49,-0.11),(-0.19,0.04),
-             (-0.39,0.31),(-0.09,0.18)]
+    "star7": [
+        (0.0, 0.5),
+        (0.09, 0.18),
+        (0.39, 0.31),
+        (0.19, 0.04),
+        (0.49, -0.11),
+        (0.16, -0.12),
+        (0.22, -0.45),
+        (0.0, -0.2),
+        (-0.22, -0.45),
+        (-0.16, -0.12),
+        (-0.49, -0.11),
+        (-0.19, 0.04),
+        (-0.39, 0.31),
+        (-0.09, 0.18)
+    ]
 }
 
+
 class BaseShapeStim(BaseVisualStim, ColorMixin, ContainerMixin):
     """Create geometric (vector) shapes by defining vertex locations.
 
@@ -72,32 +95,35 @@ class BaseShapeStim(BaseVisualStim, ColorMixin, ContainerMixin):
     is needed.
 
     v1.84.00: ShapeStim became BaseShapeStim.
-    """
 
+    """
     def __init__(self,
                  win,
                  units='',
                  lineWidth=1.5,
-                 lineColor=(1.0, 1.0, 1.0),
-                 lineColorSpace='rgb',
-                 fillColor=None,
-                 fillColorSpace='rgb',
+                 lineColor=False, # uses False in place of None to distinguish between "not set" and "transparent"
+                 fillColor=False, # uses False in place of None to distinguish between "not set" and "transparent"
+                 colorSpace='rgb',
                  vertices=((-0.5, 0), (0, +0.5), (+0.5, 0)),
                  closeShape=True,
                  pos=(0, 0),
                  size=1,
+                 anchor=None,
                  ori=0.0,
-                 opacity=1.0,
+                 opacity=None,
                  contrast=1.0,
                  depth=0,
                  interpolate=True,
-                 lineRGB=None,
-                 fillRGB=None,
                  name=None,
                  autoLog=None,
                  autoDraw=False,
-                 color=None,
-                 colorSpace=None):
+                 # legacy
+                 color=False,
+                 lineRGB=False,
+                 fillRGB=False,
+                 fillColorSpace=None,
+                 lineColorSpace=None
+                 ):
         """ """  # all doc is in the attributes
         # what local vars are defined (these are the init params) for use by
         # __repr__
@@ -109,54 +135,50 @@ class BaseShapeStim(BaseVisualStim, ColorMixin, ContainerMixin):
         super(BaseShapeStim, self).__init__(win, units=units,
                                             name=name, autoLog=False)
 
-        self.contrast = float(contrast)
-        self.opacity = float(opacity)
-        self.pos = numpy.array(pos, float)
+        self.pos = pos
         self.closeShape = closeShape
         self.lineWidth = lineWidth
         self.interpolate = interpolate
 
-        # Color stuff
-        self.useShaders = False  # don't need to combine textures with colors
-        # set color first but then potentially override
-        self.__dict__['colorSpace'] = colorSpace
-        self.__dict__['lineColorSpace'] = lineColorSpace
-        self.__dict__['fillColorSpace'] = fillColorSpace
-
-        if lineRGB is not None:
+        # Appearance
+        self.colorSpace = colorSpace
+        if fillColor is not False:
+            self.fillColor = fillColor
+        elif color is not False:
+            # Override fillColor with color if not set
+            self.fillColor = color
+        else:
+            # Default to None if neither are set
+            self.fillColor = None
+        if lineColor is not False:
+            self.lineColor = lineColor
+        elif color is not False:
+            # Override lineColor with color if not set
+            self.lineColor = color
+        else:
+            # Default to black if neither are set
+            self.lineColor = 'black'
+        if lineRGB is not False:
+            # Override with RGB if set
             logging.warning("Use of rgb arguments to stimuli are deprecated."
                             " Please use color and colorSpace args instead")
             self.setLineColor(lineRGB, colorSpace='rgb', log=None)
-        elif color is not None and lineColor is None:
-            pass  # user has set color but not lineColor. Don't override that
-        else:
-            self.setLineColor(lineColor, colorSpace=lineColorSpace, log=None)
-
-        if fillRGB is not None:
+        if fillRGB is not False:
+            # Override with RGB if set
             logging.warning("Use of rgb arguments to stimuli are deprecated."
                             " Please use color and colorSpace args instead")
             self.setFillColor(fillRGB, colorSpace='rgb', log=None)
-        elif color is not None and fillColor is None:
-            pass  # user has set color but not fillColor. Don't override that
-        else:
-            self.setFillColor(fillColor, colorSpace=fillColorSpace, log=None)
-
-        # if the fillColor and lineColor are not set but color is
-        # then the user probably wants color applied to both
-        if (lineColor is (1.0, 1.0, 1.0)  # check if exactly as the default arg
-                and fillColor is None
-                and color is not None):
-            self.color = color
-        else:
-            self.fillColor = fillColor
-            self.lineColor = lineColor
+        self.contrast = contrast
+        if opacity:
+            self.opacity = opacity
 
         # Other stuff
         self.depth = depth
         self.ori = numpy.array(ori, float)
-        self.size = numpy.array([0.0, 0.0]) + size  # make sure that it's 2D
+        self.size = size  # make sure that it's 2D
         if vertices != ():  # flag for when super-init'ing a ShapeStim
             self.vertices = vertices  # call attributeSetter
+        self.anchor = anchor
         self.autoDraw = autoDraw  # call attributeSetter
 
         # set autoLog now that params have been initialised
@@ -167,11 +189,13 @@ class BaseShapeStim(BaseVisualStim, ColorMixin, ContainerMixin):
 
     @attributeSetter
     def lineWidth(self, value):
-        """int or float
-        specifying the line width in **pixels**
+        """Width of the line in **pixels**.
 
         :ref:`Operations <attrib-operations>` supported.
         """
+        if isinstance(self, psychopy.visual.Line) and isinstance(value, (int, float)):
+            if value > 127:
+                logging.warning("lineWidth is greater than max width supported by OpenGL. For lines thicker than 127px, please use a filled Rect instead.")
         self.__dict__['lineWidth'] = value
 
     def setLineWidth(self, value, operation='', log=None):
@@ -179,147 +203,56 @@ class BaseShapeStim(BaseVisualStim, ColorMixin, ContainerMixin):
 
     @attributeSetter
     def closeShape(self, value):
-        """True or False
-        Should the last vertex be automatically connected to the first?
+        """Should the last vertex be automatically connected to the first?
 
-        If you're using `Polygon`, `Circle` or `Rect`, closeShape=True is
+        If you're using `Polygon`, `Circle` or `Rect`, `closeShape=True` is
         assumed and shouldn't be changed.
         """
         self.__dict__['closeShape'] = value
 
     @attributeSetter
     def interpolate(self, value):
-        """True or False
-        If True the edge of the line will be antialiased.
+        """If `True` the edge of the line will be anti-aliased.
         """
         self.__dict__['interpolate'] = value
 
     @attributeSetter
     def color(self, color):
+        """Set the color of the shape. Sets both `fillColor` and `lineColor`
+        simultaneously if applicable.
+        """
+        ColorMixin.foreColor.fset(self, color)
         self.fillColor = color
         self.lineColor = color
+        return ColorMixin.foreColor.fget(self)
 
-    @attributeSetter
-    def fillColor(self, color):
-        """Sets the color of the shape fill.
-
-        See :meth:`psychopy.visual.GratingStim.color` for further details
-        of how to use colors.
-
-        Note that shapes where some vertices point inwards will usually not
-        'fill' correctly.
-        """
-        setColor(self, color, rgbAttrib='fillRGB', colorAttrib='fillColor')
-
-    @attributeSetter
-    def lineColor(self, color):
-        """Sets the color of the shape lines.
-
-        See :meth:`psychopy.visual.GratingStim.color` for further details
-        of how to use colors.
-        """
-        setColor(self, color, rgbAttrib='lineRGB', colorAttrib='lineColor')
-
-    @attributeSetter
-    def fillColorSpace(self, value):
-        """
-        Sets color space for fill color. See documentation for fillColorSpace
-        """
-        self.__dict__['fillColorSpace'] = value
-
-    @attributeSetter
-    def lineColorSpace(self, value):
-        """
-        Sets color space for line color. See documentation for lineColorSpace
-        """
-        self.__dict__['lineColorSpace'] = value
+    #---legacy functions---
 
     def setColor(self, color, colorSpace=None, operation='', log=None):
-        """Sets both the line and fill to be the same color
+        """Sets both the line and fill to be the same color.
         """
+        ColorMixin.setForeColor(self, color, colorSpace, operation, log)
         self.setLineColor(color, colorSpace, operation, log)
         self.setFillColor(color, colorSpace, operation, log)
 
-    def setLineRGB(self, value, operation=''):
-        """DEPRECATED since v1.60.05: Please use :meth:`~ShapeStim.lineColor`
-        """
-        self._set('lineRGB', value, operation)
-
-    def setFillRGB(self, value, operation=''):
-        """DEPRECATED since v1.60.05: Please use :meth:`~ShapeStim.fillColor`
-        """
-        self._set('fillRGB', value, operation)
-
-    def setLineColor(self, color, colorSpace=None, operation='', log=None):
-        """Sets the color of the shape edge.
-
-        See :meth:`psychopy.visual.GratingStim.color` for further details.
-        """
-        setColor(self, color, colorSpace=colorSpace, operation=operation,
-                 rgbAttrib='lineRGB',  # the name for this rgb value
-                 colorAttrib='lineColor')  # the name for this color
-        logAttrib(self, log, 'lineColor', value='%s (%s)' %
-                  (self.lineColor, self.lineColorSpace))
-
-    def setFillColor(self, color, colorSpace=None, operation='', log=None):
-        """Sets the color of the shape fill.
-
-        See :meth:`psychopy.visual.GratingStim.color` for further details.
-
-        Note that shapes where some vertices point inwards will usually not
-        'fill' correctly.
-        """
-        # run the original setColor, which creates color and
-        setColor(self, color, colorSpace=colorSpace, operation=operation,
-                 rgbAttrib='fillRGB',  # the name for this rgb value
-                 colorAttrib='fillColor')  # the name for this color
-        logAttrib(self, log, 'fillColor', value='%s (%s)' %
-                  (self.fillColor, self.fillColorSpace))
-
-    @attributeSetter
-    def size(self, value):
-        """Int/Float or :ref:`x,y-pair <attrib-xy>`.
-        Sets the size of the shape.
-        Size is independent of the units of shape and will simply scale
-        the shape's vertices by the factor given.
-        Use a tuple or list of two values to scale asymmetrically.
-
-        :ref:`Operations <attrib-operations>` supported.
-        """
-        self.__dict__['size'] = numpy.array(value, float)
-        self._needVertexUpdate = True
-
-    def setSize(self, value, operation='', log=None):
-        """Usually you can use 'stim.attribute = value' syntax instead,
-        but use this method if you need to suppress the log message
-        """
-        setAttribute(self, 'size', value, log,
-                     operation)  # calls attributeSetter
-
-    @attributeSetter
-    def vertices(self, value):
-        """A list of lists or a numpy array (Nx2) specifying xy positions of
-        each vertex, relative to the center of the field.
-
-        If you're using `Polygon`, `Circle` or `Rect`, this shouldn't be used.
-
-        :ref:`Operations <attrib-operations>` supported.
-        """
-        self.__dict__['vertices'] = numpy.array(value, float)
-
-        # Check shape
-        if not (self.vertices.shape == (2,) or
-                (len(self.vertices.shape) == 2 and
-                 self.vertices.shape[1] == 2)):
-            raise ValueError("New value for setXYs should be 2x1 or Nx2")
-        self._needVertexUpdate = True
-
     def setVertices(self, value=None, operation='', log=None):
         """Usually you can use 'stim.attribute = value' syntax instead,
         but use this method if you need to suppress the log message
         """
         setAttribute(self, 'vertices', value, log, operation)
 
+    @staticmethod
+    def _calcEquilateralVertices(edges, radius=0.5):
+        """
+        Get vertices for an equilateral shape with a given number of sides, will assume radius is 0.5 (relative) but
+        can be manually specified
+        """
+        d = numpy.pi * 2 / edges
+        vertices = numpy.asarray(
+            [numpy.asarray((numpy.sin(e * d), numpy.cos(e * d))) * radius
+             for e in range(int(round(edges)))])
+        return vertices
+
     def draw(self, win=None, keepMatrix=False):
         """Draw the stimulus in its relevant window.
 
@@ -360,19 +293,14 @@ class BaseShapeStim(BaseVisualStim, ColorMixin, ContainerMixin):
 
         GL.glEnableClientState(GL.GL_VERTEX_ARRAY)
         if nVerts > 2:  # draw a filled polygon first
-            if self.fillRGB is not None:
-                # convert according to colorSpace
-                fillRGB = self._getDesiredRGB(
-                    self.fillRGB, self.fillColorSpace, self.contrast)
+            if self._fillColor != None:
                 # then draw
-                GL.glColor4f(fillRGB[0], fillRGB[1], fillRGB[2], self.opacity)
+                GL.glColor4f(*self._fillColor.render('rgba1'))
                 GL.glDrawArrays(GL.GL_POLYGON, 0, nVerts)
-        if self.lineRGB is not None and self.lineWidth != 0.0:
-            lineRGB = self._getDesiredRGB(
-                self.lineRGB, self.lineColorSpace, self.contrast)
+        if self._borderColor != None and self.lineWidth != 0.0:
             # then draw
             GL.glLineWidth(self.lineWidth)
-            GL.glColor4f(lineRGB[0], lineRGB[1], lineRGB[2], self.opacity)
+            GL.glColor4f(*self._borderColor.render('rgba1'))
             if self.closeShape:
                 GL.glDrawArrays(GL.GL_LINE_LOOP, 0, nVerts)
             else:
@@ -413,23 +341,104 @@ class ShapeStim(BaseShapeStim):
     Changed Nov 2015: v1.84.00. Now allows filling of complex shapes. This
     is almost completely backwards compatible (see changelog). The
     old version is accessible as `psychopy.visual.BaseShapeStim`.
-    """
 
+    Parameters
+    ----------
+    win : :class:`~psychopy.visual.Window`
+        Window this shape is being drawn to. The stimulus instance will
+        allocate its required resources using that Windows context. In many
+        cases, a stimulus instance cannot be drawn on different windows
+        unless those windows share the same OpenGL context, which permits
+        resources to be shared between them.
+    units : str
+        Units to use when drawing. This will affect how parameters and
+        attributes `pos`, `size` and `radius` are interpreted.
+    colorSpace : str
+        Sets the colorspace, changing how values passed to `lineColor` and
+        `fillColor` are interpreted.
+    lineWidth : float
+        Width of the shape outline.
+    lineColor, fillColor : array_like, str, :class:`~psychopy.colors.Color` or None
+        Color of the shape outline and fill. If `None`, a fully
+        transparent color is used which makes the fill or outline invisible.
+    vertices : array_like
+        Nx2 array of points (eg., `[[-0.5, 0], [0, 0.5], [0.5, 0]`).
+    windingRule : :class:`~pyglet.gl.GLenum` or None
+        Winding rule to use for tesselation, default is
+        `GLU_TESS_WINDING_ODD` if `None` is specified.
+    closeShape : bool
+        Close the shape's outline. If `True` the first and last vertex will
+        be joined by an edge. Must be `True` to use tesselation. Default is
+        `True`.
+    pos : array_like
+        Initial position (`x`, `y`) of the shape on-screen relative to
+        the origin located at the center of the window or buffer in `units`.
+        This can be updated after initialization by setting the `pos`
+        property. The default value is `(0.0, 0.0)` which results in no
+        translation.
+    size : array_like, float, int or None
+        Width and height of the shape as `(w, h)` or `[w, h]`. If a single
+        value is provided, the width and height will be set to the same
+        specified value. If `None` is specified, the `size` will be set
+        with values passed to `width` and `height`.
+    ori : float
+        Initial orientation of the shape in degrees about its origin.
+        Positive values will rotate the shape clockwise, while negative
+        values will rotate counterclockwise. The default value for `ori` is
+        0.0 degrees.
+    opacity : float
+        Opacity of the shape. A value of 1.0 indicates fully opaque and 0.0
+        is fully transparent (therefore invisible). Values between 1.0 and
+        0.0 will result in colors being blended with objects in the
+        background. This value affects the fill (`fillColor`) and outline
+        (`lineColor`) colors of the shape.
+    contrast : float
+        Contrast level of the shape (0.0 to 1.0). This value is used to
+        modulate the contrast of colors passed to `lineColor` and
+        `fillColor`.
+    depth : int
+        Depth layer to draw the shape when `autoDraw` is enabled.
+        *DEPRECATED*
+    interpolate : bool
+        Enable smoothing (anti-aliasing) when drawing shape outlines. This
+        produces a smoother (less-pixelated) outline of the shape.
+    name : str
+        Optional name of the stimuli for logging.
+    autoLog : bool
+        Enable auto-logging of events associated with this stimuli. Useful
+        for debugging and to track timing when used in conjunction with
+        `autoDraw`.
+    autoDraw : bool
+        Enable auto drawing. When `True`, the stimulus will be drawn every
+        frame without the need to explicitly call the
+        :py:meth:`~psychopy.visual.ShapeStim.draw` method.
+    color : array_like, str, :class:`~psychopy.colors.Color` or None
+        Sets both the initial `lineColor` and `fillColor` of the shape.
+    lineRGB, fillRGB: array_like, :class:`~psychopy.colors.Color` or None
+        *Deprecated*. Please use `lineColor` and `fillColor`. These
+        arguments may be removed in a future version.
+    lineColorSpace, fillColorSpace : str
+        Colorspace to use for the outline and fill. These change how the
+        values passed to `lineColor` and `fillColor` are interpreted.
+        *Deprecated*. Please use `colorSpace` to set both outline and fill
+        colorspace. These arguments may be removed in a future version.
+
+    """
     # Author: Jeremy Gray, November 2015, using psychopy.contrib.tesselate
 
     def __init__(self,
                  win,
                  units='',
+                 colorSpace='rgb',
+                 fillColor=False,
+                 lineColor=False,
                  lineWidth=1.5,
-                 lineColor='white',
-                 lineColorSpace='rgb',
-                 fillColor=None,
-                 fillColorSpace='rgb',
                  vertices=((-0.5, 0), (0, +0.5), (+0.5, 0)),
                  windingRule=None,  # default GL.GLU_TESS_WINDING_ODD
                  closeShape=True,  # False for a line
                  pos=(0, 0),
                  size=1,
+                 anchor=None,
                  ori=0.0,
                  opacity=1.0,
                  contrast=1.0,
@@ -437,9 +446,15 @@ class ShapeStim(BaseShapeStim):
                  interpolate=True,
                  name=None,
                  autoLog=None,
-                 autoDraw=False):
-        """
-        """
+                 autoDraw=False,
+                 # legacy
+                 color=False,
+                 lineRGB=False,
+                 fillRGB=False,
+                 fillColorSpace=None,
+                 lineColorSpace=None
+                 ):
+
         # what local vars are defined (init params, for use by __repr__)
         self._initParamsOrig = dir()
         self._initParamsOrig.remove('self')
@@ -447,6 +462,7 @@ class ShapeStim(BaseShapeStim):
         super(ShapeStim, self).__init__(win,
                                         units=units,
                                         lineWidth=lineWidth,
+                                        colorSpace=colorSpace,
                                         lineColor=lineColor,
                                         lineColorSpace=lineColorSpace,
                                         fillColor=fillColor,
@@ -455,6 +471,7 @@ class ShapeStim(BaseShapeStim):
                                         closeShape=self.closeShape,
                                         pos=pos,
                                         size=size,
+                                        anchor=anchor,
                                         ori=ori,
                                         opacity=opacity,
                                         contrast=contrast,
@@ -478,13 +495,15 @@ class ShapeStim(BaseShapeStim):
             logging.exp("Created %s = %s" % (self.name, str(self)))
 
     def _tesselate(self, newVertices):
-        """Set the .vertices and .border to new values, invoking tessellation.
+        """Set the `.vertices` and `.border` to new values, invoking
+        tessellation.
         """
         # TO-DO: handle borders properly for multiloop stim like holes
         # likely requires changes in ContainerMixin to iterate over each
         # border loop
 
         self.border = copy.deepcopy(newVertices)
+        tessVertices = []  # define to keep the linter happy
         if self.closeShape:
             # convert original vertices to triangles (= tesselation) if
             # possible. (not possible if closeShape is False, don't even try)
@@ -512,8 +531,8 @@ class ShapeStim(BaseShapeStim):
             initVertices = tessVertices
         self.__dict__['_tesselVertices'] = numpy.array(initVertices, float)
 
-    @attributeSetter
-    def vertices(self, newVerts):
+    @property
+    def vertices(self):
         """A list of lists or a numpy array (Nx2) specifying xy positions of
         each vertex, relative to the center of the field.
 
@@ -521,31 +540,25 @@ class ShapeStim(BaseShapeStim):
 
         :ref:`Operations <attrib-operations>` supported with `.setVertices()`.
         """
-        # check if this is a name of one of our known shapes
-        if isinstance(newVerts, basestring) and newVerts in knownShapes:
-            newVerts = knownShapes[newVerts]
+        return WindowMixin.vertices.fget(self)
 
+    @vertices.setter
+    def vertices(self, value):
+        # check if this is a name of one of our known shapes
+        if isinstance(value, str) and value in knownShapes:
+            value = knownShapes[value]
+        if isinstance(value, int):
+            value = self._calcEquilateralVertices(value)
         # Check shape
-        self.__dict__['vertices'] = val2array(newVerts, withNone=True,
-                                              withScalar=True, length=2)
+        WindowMixin.vertices.fset(self, value)
         self._needVertexUpdate = True
         self._tesselate(self.vertices)
 
-    @property
-    def verticesPix(self):
-        """This determines the coordinates of the vertices for the
-        current stimulus in pixels, accounting for size, ori, pos and units
-        """
-        # because this is a property getter we can check /on-access/ if it
-        # needs updating :-)
-        if self._needVertexUpdate:
-            self._updateVertices()
-        return self.__dict__['verticesPix']
-
     def draw(self, win=None, keepMatrix=False):
-        """Draw the stimulus in the relevant window. You must call this method
-        after every win.flip() if you want the stimulus to appear on that
-        frame and then update the screen again.
+        """Draw the stimulus in the relevant window.
+
+        You must call this method after every `win.flip()` if you want the
+        stimulus to appear on that frame and then update the screen again.
         """
         # mostly copied from BaseShapeStim. Uses GL_TRIANGLES and depends on
         # two arrays of vertices: tesselated (for fill) & original (for
@@ -585,20 +598,16 @@ class ShapeStim(BaseShapeStim):
         # fill interior triangles if there are any
         if (self.closeShape and
                 self.verticesPix.shape[0] > 2 and
-                self.fillRGB is not None):
+                self._fillColor != None):
             GL.glVertexPointer(2, GL.GL_DOUBLE, 0, self.verticesPix.ctypes)
-            fillRGB = self._getDesiredRGB(self.fillRGB, self.fillColorSpace,
-                                          self.contrast)
-            GL.glColor4f(fillRGB[0], fillRGB[1], fillRGB[2], self.opacity)
+            GL.glColor4f(*self._fillColor.render('rgba1'))
             GL.glDrawArrays(GL.GL_TRIANGLES, 0, self.verticesPix.shape[0])
 
         # draw the border (= a line connecting the non-tesselated vertices)
-        if self.lineRGB is not None and self.lineWidth:
+        if self._borderColor != None and self.lineWidth:
             GL.glVertexPointer(2, GL.GL_DOUBLE, 0, self._borderPix.ctypes)
-            lineRGB = self._getDesiredRGB(self.lineRGB, self.lineColorSpace,
-                                          self.contrast)
             GL.glLineWidth(self.lineWidth)
-            GL.glColor4f(lineRGB[0], lineRGB[1], lineRGB[2], self.opacity)
+            GL.glColor4f(*self._borderColor.render('rgba1'))
             if self.closeShape:
                 gl_line = GL.GL_LINE_LOOP
             else:
