@@ -6,11 +6,9 @@ the resolution and color in the file (subject to gamma correction if set).
 """
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2020 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, division, print_function
-from past.builtins import basestring
 import os
 
 # Ensure setting pyglet.options['debug_gl'] to False is done prior to any
@@ -18,6 +16,9 @@ import os
 # up by the pyglet GL engine and have no effect.
 # Shaders will work but require OpenGL2.0 drivers AND PyOpenGL3.0+
 import pyglet
+
+from ..layout import Size
+
 pyglet.options['debug_gl'] = False
 GL = pyglet.gl
 
@@ -75,10 +76,11 @@ class SimpleImageStim(MinimalStim, WindowMixin):
         self.units = units  # call attributeSetter
         # call attributeSetter. Use shaders if available by default, this is a
         # good thing
-        self.useShaders = win._haveShaders
+
         self.pos = pos  # call attributeSetter
         self.image = image  # call attributeSetter
         # check image size against window size
+        print(self.size)
         if (self.size[0] > self.win.size[0] or
                 self.size[1] > self.win.size[1]):
             msg = ("Image size (%s, %s)  was larger than window size "
@@ -148,7 +150,7 @@ class SimpleImageStim(MinimalStim, WindowMixin):
         setAttribute(self, 'flipVert', newVal, log)
 
     def _updateImageStr(self):
-        self._imStr = self.imArray.tostring()
+        self._imStr = self.imArray.tobytes()
         self._needStrUpdate = False
 
     def draw(self, win=None):
@@ -185,11 +187,11 @@ class SimpleImageStim(MinimalStim, WindowMixin):
 
         # move to center of stimulus
         x, y = self._posRendered[:2]
-        GL.glRasterPos2f(self.win.size[0]/2 - self.size[0]/2 + x,
-                         self.win.size[1]/2 - self.size[1]/2 + y)
+        GL.glRasterPos2f(self.win.size[0]/2 - self._size.pix[0]/2 + x,
+                         self.win.size[1]/2 - self._size.pix[1]/2 + y)
 
         # GL.glDrawPixelsub(GL.GL_RGB, self.imArr)
-        GL.glDrawPixels(self.size[0], self.size[1],
+        GL.glDrawPixels(int(self._size.pix[0]), int(self._size.pix[1]),
                         self.internalFormat, self.dataType, self._imStr)
         # return to 3D mode (go and pop the projection matrix)
         GL.glMatrixMode(GL.GL_PROJECTION)
@@ -211,20 +213,24 @@ class SimpleImageStim(MinimalStim, WindowMixin):
         if op is None:
             op = ''
         # format the input value as float vectors
-        if type(val) in (tuple, list):
+        if isinstance(val, (tuple, list)):
             val = numpy.array(val, float)
 
         setAttribute(self, attrib, val, log, op)
 
-    @attributeSetter
-    def pos(self, value):
+    @property
+    def pos(self):
         """:ref:`x,y-pair <attrib-xy>` specifying the centre of the image
         relative to the window center. Stimuli can be positioned off-screen,
         beyond the window!
 
         :ref:`operations <attrib-operations>` are supported.
         """
-        self.__dict__['pos'] = val2array(value, withNone=False)
+        return WindowMixin.pos.fget(self)
+
+    @pos.setter
+    def pos(self, value):
+        WindowMixin.pos.fset(self, value)
         self._calcPosRendered()
 
     def setPos(self, newPos, operation='', log=None):
@@ -258,7 +264,7 @@ class SimpleImageStim(MinimalStim, WindowMixin):
         """
         filename = pathToString(filename)
         self.__dict__['image'] = filename
-        if isinstance(filename, basestring):
+        if isinstance(filename, str):
             # is a string - see if it points to a file
             if os.path.isfile(filename):
                 self.filename = filename
@@ -276,7 +282,7 @@ class SimpleImageStim(MinimalStim, WindowMixin):
                 core.quit()
             self.filename = repr(filename)  # '<Image.Image image ...>'
 
-        self.size = im.size
+        self._size = Size(im.size, units='pix', win=self.win)
         # set correct formats for bytes/floats
         if im.mode == 'RGBA':
             self.imArray = numpy.array(im).astype(numpy.ubyte)
