@@ -4,7 +4,7 @@
 """Classes for 3D stimuli."""
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
 from psychopy import logging
@@ -26,7 +26,7 @@ import numpy as np
 import pyglet.gl as GL
 
 
-class LightSource(object):
+class LightSource:
     """Class for representing a light source in a scene.
 
     Only point and directional lighting is supported by this object for now. The
@@ -212,7 +212,7 @@ class LightSource(object):
         self._kAttenuation = np.asarray(value, np.float32)
 
 
-class SceneSkybox(object):
+class SceneSkybox:
     """Class to render scene skyboxes.
 
     A skybox provides background imagery to serve as a visual reference for the
@@ -397,7 +397,7 @@ class SceneSkybox(object):
         win.draw3d = False
 
 
-class BlinnPhongMaterial(object):
+class BlinnPhongMaterial:
     """Class representing a material using the Blinn-Phong lighting model.
 
     This class stores material information to modify the appearance of drawn
@@ -529,7 +529,6 @@ class BlinnPhongMaterial(object):
         self._normalTexture = None
 
         self._useTextures = False  # keeps track if textures are being used
-        self._useShaders = False
 
     @property
     def diffuseTexture(self):
@@ -652,7 +651,7 @@ class BlinnPhongMaterial(object):
     def shininess(self, value):
         self._shininess = float(value)
 
-    def begin(self, useTextures=True, useShaders=False):
+    def begin(self, useTextures=True):
         """Use this material for successive rendering calls.
 
         Parameters
@@ -664,13 +663,11 @@ class BlinnPhongMaterial(object):
         GL.glDisable(GL.GL_COLOR_MATERIAL)  # disable color tracking
         face = self._face
 
-        if useShaders:
-            # number of scene lights
-            self._useShaders = True
-            nLights = len(self.win.lights)
-            useTextures = useTextures and self.diffuseTexture is not None
-            shaderKey = (nLights, useTextures)
-            gt.useProgram(self.win._shaders['stim3d_phong'][shaderKey])
+        # number of scene lights
+        nLights = len(self.win.lights)
+        useTextures = useTextures and self.diffuseTexture is not None
+        shaderKey = (nLights, useTextures)
+        gt.useProgram(self.win._shaders['stim3d_phong'][shaderKey])
 
         # pass values to OpenGL
         GL.glMaterialfv(face, GL.GL_DIFFUSE, self._ptrDiffuse)
@@ -731,14 +728,12 @@ class BlinnPhongMaterial(object):
             gt.unbindTexture(self.diffuseTexture)
             GL.glDisable(GL.GL_TEXTURE_2D)
 
-        if self._useShaders:
-            gt.useProgram(0)
-            self._useShaders = False
+        gt.useProgram(0)
 
         GL.glEnable(GL.GL_COLOR_MATERIAL)
 
 
-class RigidBodyPose(object):
+class RigidBodyPose:
     """Class for representing rigid body poses.
 
     This class is an abstract representation of a rigid body pose, where the
@@ -1117,7 +1112,7 @@ class RigidBodyPose(object):
         viewMatrix = vt.lookAt(self._pos, fwdVec, upVec, dtype=np.float32)
 
         if inverse:
-            viewMatrix = mt.invertMatrix(viewMatrix, homogeneous=True)
+            viewMatrix = mt.invertMatrix(viewMatrix)
 
         return viewMatrix
 
@@ -1268,7 +1263,7 @@ class RigidBodyPose(object):
             self._ori, mt.alignTo(fwd, invPos, dtype=np.float32))
 
 
-class BoundingBox(object):
+class BoundingBox:
     """Class for representing object bounding boxes.
 
     A bounding box is a construct which represents a 3D rectangular volume about
@@ -1364,7 +1359,6 @@ class BaseRigidBodyStim(ColorMixin, WindowMixin):
                  colorSpace='rgb',
                  contrast=1.0,
                  opacity=1.0,
-                 useShaders=False,
                  name='',
                  autoLog=True):
         """
@@ -1393,7 +1387,6 @@ class BaseRigidBodyStim(ColorMixin, WindowMixin):
         self.color = color
 
         self._thePose = RigidBodyPose(pos, ori)
-        self._useShaders = useShaders
         self.material = None
 
         self._vao = None
@@ -1530,72 +1523,32 @@ class BaseRigidBodyStim(ColorMixin, WindowMixin):
         GL.glMultTransposeMatrixf(at.array2pointer(self.thePose.modelMatrix))
 
         if self.material is not None:  # has a material, use it
-            if self._useShaders:
-                useTexture = self.material.diffuseTexture is not None
-                self.material.begin(useTexture, useShaders=True)
-                gt.drawVAO(self._vao, GL.GL_TRIANGLES)
-                self.material.end()
-            else:
-                self.material.begin(self.material.diffuseTexture is not None)
-                gt.drawVAO(self._vao, GL.GL_TRIANGLES)
-                self.material.end()
+            useTexture = self.material.diffuseTexture is not None
+            self.material.begin(useTexture)
+            gt.drawVAO(self._vao, GL.GL_TRIANGLES)
+            self.material.end()
         else:  # doesn't have a material, use class colors
-            r, g, b = self._getDesiredRGB(
-                self.rgb, self.colorSpace, self.contrast)
+            r, g, b = self._foreColor.render('rgb')
             color = np.ctypeslib.as_ctypes(
                 np.array((r, g, b, self.opacity), np.float32))
 
-            if self._useShaders:
-                nLights = len(self.win.lights)
-                shaderKey = (nLights, False)
-                gt.useProgram(self.win._shaders['stim3d_phong'][shaderKey])
+            nLights = len(self.win.lights)
+            shaderKey = (nLights, False)
+            gt.useProgram(self.win._shaders['stim3d_phong'][shaderKey])
 
-                # pass values to OpenGL as material
-                GL.glColor4f(r, g, b, self.opacity)
-                GL.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, color)
-                GL.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, color)
+            # pass values to OpenGL as material
+            GL.glColor4f(r, g, b, self.opacity)
+            GL.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, color)
+            GL.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, color)
 
-                gt.drawVAO(self._vao, GL.GL_TRIANGLES)
+            gt.drawVAO(self._vao, GL.GL_TRIANGLES)
 
-                gt.useProgram(0)
-            else:
-                # material tracks color
-                GL.glEnable(GL.GL_COLOR_MATERIAL)  # enable color tracking
-                GL.glDisable(GL.GL_TEXTURE_2D)
-                GL.glColorMaterial(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE)
-                GL.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, color)
-                # 'rgb' is created and set when color is set
-                GL.glColor4f(r, g, b, self.opacity)
-
-                # draw the shape
-                gt.drawVAO(self._vao, GL.GL_TRIANGLES)
-                GL.glDisable(GL.GL_COLOR_MATERIAL)  # enable color tracking
+            gt.useProgram(0)
 
         GL.glPopMatrix()
 
         win.draw3d = False
 
-    @attributeSetter
-    def useShaders(self, value):
-        """Should shaders be used to render the stimulus
-        (typically leave as `True`)
-
-        If the system support the use of OpenGL shader language then leaving
-        this set to True is highly recommended. If shaders cannot be used then
-        various operations will be slower (notably, changes to stimulus color
-        or contrast)
-        """
-        if value is True and self.win._haveShaders is False:
-            logging.error("Shaders were requested but aren't available. "
-                          "Shaders need OpenGL 2.0+ drivers")
-        if value != self._useShaders:  # if there's a change...
-            self._useShaders = value
-
-    def setUseShaders(self, value=True, log=None):
-        """Usually you can use 'stim.attribute = value' syntax instead,
-        but use this method if you need to suppress the log message"""
-        setAttribute(self, 'useShaders', value, log)  # call attributeSetter
-
     @attributeSetter
     def units(self, value):
         """
@@ -1755,7 +1708,6 @@ class SphereStim(BaseRigidBodyStim):
                  contrast=1.0,
                  opacity=1.0,
                  useMaterial=None,
-                 useShaders=False,
                  name='',
                  autoLog=True):
         """
@@ -1809,7 +1761,6 @@ class SphereStim(BaseRigidBodyStim):
                                          colorSpace=colorSpace,
                                          contrast=contrast,
                                          opacity=opacity,
-                                         useShaders=useShaders,
                                          name=name,
                                          autoLog=autoLog)
 
@@ -1822,7 +1773,6 @@ class SphereStim(BaseRigidBodyStim):
         self._vao = self._createVAO(vertices, textureCoords, normals, faces)
 
         self.material = useMaterial
-        self._useShaders = useShaders
 
         self._radius = radius  # for raypicking
 
@@ -1880,7 +1830,6 @@ class BoxStim(BaseRigidBodyStim):
                  contrast=1.0,
                  opacity=1.0,
                  useMaterial=None,
-                 useShaders=False,
                  textureScale=None,
                  name='',
                  autoLog=True):
@@ -1939,7 +1888,6 @@ class BoxStim(BaseRigidBodyStim):
             colorSpace=colorSpace,
             contrast=contrast,
             opacity=opacity,
-            useShaders=useShaders,
             name=name,
             autoLog=autoLog)
 
@@ -1988,7 +1936,6 @@ class PlaneStim(BaseRigidBodyStim):
                  contrast=1.0,
                  opacity=1.0,
                  useMaterial=None,
-                 useShaders=False,
                  textureScale=None,
                  name='',
                  autoLog=True):
@@ -2042,7 +1989,6 @@ class PlaneStim(BaseRigidBodyStim):
             colorSpace=colorSpace,
             contrast=contrast,
             opacity=opacity,
-            useShaders=useShaders,
             name=name,
             autoLog=autoLog)
 
@@ -2114,7 +2060,6 @@ class ObjMeshStim(BaseRigidBodyStim):
                  colorSpace='rgb',
                  contrast=1.0,
                  opacity=1.0,
-                 useShaders=False,
                  name='',
                  autoLog=True):
         """
@@ -2147,8 +2092,6 @@ class ObjMeshStim(BaseRigidBodyStim):
             after initialization will be a dictionary where keys are material
             names and values are materials. Any textures associated with the
             model will be loaded as per the material requirements.
-        useShaders : bool
-            Use shaders when rendering.
 
         """
         super(ObjMeshStim, self).__init__(
@@ -2159,7 +2102,6 @@ class ObjMeshStim(BaseRigidBodyStim):
             colorSpace=colorSpace,
             contrast=contrast,
             opacity=opacity,
-            useShaders=useShaders,
             name=name,
             autoLog=autoLog)
 
@@ -2202,7 +2144,6 @@ class ObjMeshStim(BaseRigidBodyStim):
                  GL.GL_NORMAL_ARRAY: (vertexAttr, 3, 5, True)},
                 indexBuffer=indexBuffer, legacy=True)
 
-        self._useShaders = useShaders
         self.extents = objModel.extents
 
         self.thePose.bounds = BoundingBox()
@@ -2294,52 +2235,34 @@ class ObjMeshStim(BaseRigidBodyStim):
             # if material is a dictionary
             if isinstance(self.material, dict):
                 for materialName, materialDesc in self.material.items():
-                    materialDesc.begin(useShaders=self._useShaders)
+                    materialDesc.begin()
                     gt.drawVAO(self._vao[materialName], GL.GL_TRIANGLES)
                     materialDesc.end()
             else:
                 # material is a single item
-                self.material.begin(useShaders=self._useShaders)
+                self.material.begin()
                 for materialName, _ in self._vao.items():
                     gt.drawVAO(self._vao[materialName], GL.GL_TRIANGLES)
                 self.material.end()
         else:
-            r, g, b = self._getDesiredRGB(
-                self.rgb, self.colorSpace, self.contrast)
+            r, g, b = self._foreColor.render('rgb')
             color = np.ctypeslib.as_ctypes(
                 np.array((r, g, b, self.opacity), np.float32))
 
-            if self._useShaders:
-                nLights = len(self.win.lights)
-                shaderKey = (nLights, False)
-                gt.useProgram(self.win._shaders['stim3d_phong'][shaderKey])
+            nLights = len(self.win.lights)
+            shaderKey = (nLights, False)
+            gt.useProgram(self.win._shaders['stim3d_phong'][shaderKey])
 
-                # pass values to OpenGL as material
-                GL.glColor4f(r, g, b, self.opacity)
-                GL.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, color)
-                GL.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, color)
+            # pass values to OpenGL as material
+            GL.glColor4f(r, g, b, self.opacity)
+            GL.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, color)
+            GL.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, color)
 
-                for materialName, _ in self._vao.items():
-                    gt.drawVAO(self._vao[materialName], GL.GL_TRIANGLES)
+            for materialName, _ in self._vao.items():
+                gt.drawVAO(self._vao[materialName], GL.GL_TRIANGLES)
 
-                gt.useProgram(0)
-            else:
-                # material tracks color
-                GL.glEnable(GL.GL_COLOR_MATERIAL)  # enable color tracking
-                GL.glDisable(GL.GL_TEXTURE_2D)
-                GL.glColorMaterial(GL.GL_FRONT, GL.GL_AMBIENT_AND_DIFFUSE)
-                GL.glMaterialfv(GL.GL_FRONT, GL.GL_DIFFUSE, color)
-                GL.glMaterialfv(GL.GL_FRONT, GL.GL_AMBIENT, color)
-                # 'rgb' is created and set when color is set
-                GL.glColor4f(r, g, b, self.opacity)
-
-                # draw the shape
-                for materialName, _ in self._vao.items():
-                    gt.drawVAO(self._vao[materialName], GL.GL_TRIANGLES)
-
-                GL.glDisable(GL.GL_COLOR_MATERIAL)  # enable color tracking
+            gt.useProgram(0)
 
         GL.glPopMatrix()
 
         win.draw3d = False
-
