@@ -5,12 +5,9 @@
 '''
 
 # Part of the PsychoPy library
-# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019 Open Science Tools Ltd.
+# Copyright (C) 2002-2018 Jonathan Peirce (C) 2019-2022 Open Science Tools Ltd.
 # Distributed under the terms of the GNU General Public License (GPL).
 
-from __future__ import absolute_import, division, print_function
-
-from builtins import str
 import os
 import glob
 import warnings
@@ -31,8 +28,9 @@ from psychopy import logging
 # (JWP has no idea why!)
 from psychopy.tools.monitorunittools import cm2pix, deg2pix, convertToPix
 from psychopy.tools.attributetools import attributeSetter, setAttribute
-from psychopy.visual.basevisual import (BaseVisualStim, ColorMixin,
-    ContainerMixin)
+from psychopy.visual.basevisual import (BaseVisualStim, ForeColorMixin,
+                                        ContainerMixin, WindowMixin)
+from psychopy.colors import Color
 
 # for displaying right-to-left (possibly bidirectional) text correctly:
 from bidi import algorithm as bidi_algorithm # sufficient for Hebrew
@@ -72,7 +70,7 @@ defaultWrapWidth = {'cm': 15.0,
                     'pixels': 500}
 
 
-class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
+class TextStim(BaseVisualStim, ForeColorMixin, ContainerMixin):
     """Class of text stimuli to be displayed in a
     :class:`~psychopy.visual.Window`
     """
@@ -166,7 +164,6 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         self._needUpdate = True
         self._needVertexUpdate = True
         # use shaders if available by default, this is a good thing
-        self.__dict__['useShaders'] = win._haveShaders
         self.__dict__['antialias'] = antialias
         self.__dict__['font'] = font
         self.__dict__['bold'] = bold
@@ -179,7 +176,7 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         self.__dict__['flipVert'] = flipVert
         self.__dict__['languageStyle'] = languageStyle
         self._pygletTextObj = None
-        self.__dict__['pos'] = numpy.array(pos, float)
+        self.pos = pos
         # deprecated attributes
         if alignVert:
             self.__dict__['alignVert'] = alignVert
@@ -208,21 +205,18 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
 
         # Color stuff
         self.colorSpace = colorSpace
+        self.color = color
         if rgb != None:
-            msg = ("Use of rgb arguments to stimuli are deprecated. Please "
+            logging.warning("Use of rgb arguments to stimuli are deprecated. Please "
                    "use color and colorSpace args instead")
-            logging.warning(msg)
-            self.setColor(rgb, colorSpace='rgb', log=False)
-        else:
-            self.setColor(color, log=False)
-
+            self.color = Color(rgb, 'rgb')
         self.__dict__['fontFiles'] = []
         self.fontFiles = list(fontFiles)  # calls attributeSetter
         self.setHeight(height, log=False)  # calls setFont() at some point
         # calls attributeSetter without log
         setAttribute(self, 'wrapWidth', wrapWidth, log=False)
-        self.__dict__['opacity'] = float(opacity)
-        self.__dict__['contrast'] = float(contrast)
+        self.opacity = opacity
+        self.contrast = contrast
         # self.width and self._fontHeightPix get set with text and
         # calcSizeRendered is called
         self.setText(text, log=False)
@@ -236,7 +230,10 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
 
     def __del__(self):
         if GL:  # because of pytest fail otherwise
-            GL.glDeleteLists(self._listID, 1)
+            try:
+                GL.glDeleteLists(self._listID, 1)
+            except (ImportError, ModuleNotFoundError, TypeError):
+                pass  # if pyglet no longer exists
 
     @attributeSetter
     def height(self, height):
@@ -262,6 +259,17 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
 
         # need to update the font to reflect the change
         self.setFont(self.font, log=False)
+        return self.__dict__['height']
+
+    @property
+    def size(self):
+        self.size = (self.height*len(self.text), self.height)
+        return WindowMixin.size.fget(self)
+
+    @size.setter
+    def size(self, value):
+        WindowMixin.size.fset(self, value)
+        self.height = getattr(self._size, self.units)[1]
 
     def setHeight(self, height, log=None):
         """Usually you can use 'stim.attribute = value' syntax instead,
@@ -360,11 +368,10 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
 
             self.__dict__['text'] = text
 
-        if self.useShaders:
-            self._setTextShaders(text)
-        else:
-            self._setTextNoShaders(text)
+        self._setTextShaders(text)
+
         self._needSetText = False
+        return self.__dict__['text']
 
     def setText(self, text=None, log=None):
         """Usually you can use 'stim.attribute = value' syntax instead,
@@ -376,15 +383,17 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         """Set the text to be rendered using the current font
         """
         if self.win.winType in ["pyglet", "glfw"]:
+            rgba255 = self._foreColor.rgba255
+            rgba255[3] = rgba255[3]*255
+            rgba255 = [int(c) for c in rgba255]
             self._pygletTextObj = pyglet.text.Label(
                 self.text, self.font, int(self._heightPix*0.75),
+                italic=self.italic,
+                bold=self.bold,
                 anchor_x=self.anchorHoriz,
                 anchor_y=self.anchorVert,  # the point we rotate around
                 align=self.alignText,
-                color = (int(127.5 * self.rgb[0] + 127.5),
-                      int(127.5 * self.rgb[1] + 127.5),
-                      int(127.5 * self.rgb[2] + 127.5),
-                      int(255 * self.opacity)),
+                color=rgba255,
                 multiline=True, width=self._wrapWidthPix)  # width of the frame
             self.width = self._pygletTextObj.width
             self._fontHeightPix = self._pygletTextObj.height
@@ -499,124 +508,6 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         GL.glEndList()
         self._needUpdate = False
 
-    def _setTextNoShaders(self, value=None):
-        """Set the text to be rendered using the current font
-        """
-        desiredRGB = self._getDesiredRGB(self.rgb, self.colorSpace,
-                                         self.contrast)
-        if self.win.winType in ["pyglet", "glfw"]:
-            self._pygletTextObj = pyglet.text.Label(
-                self.text, self.font, int(self._heightPix*0.75),
-                anchor_x=self.anchorHoriz,
-                anchor_y=self.anchorVert,  # the point we rotate around
-                align=self.alignText,
-                color = (int(127.5 * self.rgb[0] + 127.5),
-                      int(127.5 * self.rgb[1] + 127.5),
-                      int(127.5 * self.rgb[2] + 127.5),
-                      int(255 * self.opacity)),
-                multiline=True, width=self._wrapWidthPix)  # width of the frame
-            self.width = self._pygletTextObj.width
-        else:
-            self._surf = self._font.render(value, self.antialias,
-                                           [desiredRGB[0] * 255,
-                                            desiredRGB[1] * 255,
-                                            desiredRGB[2] * 255])
-            self.width, self._fontHeightPix = self._surf.get_size()
-            if self.antialias:
-                smoothing = GL.GL_LINEAR
-            else:
-                smoothing = GL.GL_NEAREST
-            # generate the textures from pygame surface
-            GL.glEnable(GL.GL_TEXTURE_2D)
-            # bind that name to the target
-            GL.glBindTexture(GL.GL_TEXTURE_2D, self._texID)
-            GL.glTexImage2D(GL.GL_TEXTURE_2D, 0, GL.GL_RGBA,
-                            self.width, self._fontHeightPix, 0,
-                            GL.GL_RGBA, GL.GL_UNSIGNED_BYTE,
-                            pygame.image.tostring(self._surf, "RGBA", 1))
-            # linear smoothing if texture is stretched?
-            GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MAG_FILTER,
-                               smoothing)
-            # but nearest pixel value if it's compressed?
-            GL.glTexParameteri(GL.GL_TEXTURE_2D, GL.GL_TEXTURE_MIN_FILTER,
-                               smoothing)
-        self._needUpdate = True
-
-    def _updateListNoShaders(self):
-        """
-        The user shouldn't need this method since it gets called
-        after every call to .set() Basically it updates the OpenGL
-        representation of your stimulus if some parameter of the
-        stimulus changes. Call it if you change a property manually
-        rather than using the .set() command
-        """
-        if self._needSetText:
-            self.setText(log=False)
-        GL.glNewList(self._listID, GL.GL_COMPILE)
-
-        # coords:
-        if self.alignHoriz in ('center', 'centre'):
-            left = -self.width / 2.0
-            right = self.width / 2.0
-        elif self.alignHoriz == 'right':
-            left = -self.width
-            right = 0.0
-        else:
-            left = 0.0
-            right = self.width
-        # how much to move bottom
-        if self.alignVert in ('center', 'centre'):
-            bottom = -self._fontHeightPix /  2.0
-            top = self._fontHeightPix / 2.0
-        elif self.alignVert == 'top':
-            bottom = -self._fontHeightPix
-            top = 0
-        else:
-            bottom = 0.0
-            top = self._fontHeightPix
-        # there seems to be a rounding err in pygame font textures
-        Btex, Ttex, Ltex, Rtex = -0.01, 0.98, 0, 1.0
-        if self.win.winType in ["pyglet", "glfw"]:
-            # unbind the mask texture
-            GL.glActiveTexture(GL.GL_TEXTURE1)
-            GL.glEnable(GL.GL_TEXTURE_2D)
-            GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
-            # unbind the main texture
-            GL.glActiveTexture(GL.GL_TEXTURE0)
-            GL.glEnable(GL.GL_TEXTURE_2D)
-        else:
-            # bind the appropriate main texture
-            GL.glActiveTexture(GL.GL_TEXTURE0)
-            GL.glEnable(GL.GL_TEXTURE_2D)
-            GL.glBindTexture(GL.GL_TEXTURE_2D, self._texID)
-            # unbind the mask texture regardless
-            GL.glActiveTexture(GL.GL_TEXTURE1)
-            GL.glEnable(GL.GL_TEXTURE_2D)
-            GL.glBindTexture(GL.GL_TEXTURE_2D, 0)
-
-        if self.win.winType in ["pyglet", "glfw"]:
-            self._pygletTextObj.draw()
-        else:
-            # draw a 4 sided polygon
-            GL.glBegin(GL.GL_QUADS)
-            # right bottom
-            GL.glMultiTexCoord2fARB(GL.GL_TEXTURE0_ARB, Rtex, Btex)
-            GL.glVertex2f(right, bottom)
-            # left bottom
-            GL.glMultiTexCoord2fARB(GL.GL_TEXTURE0_ARB, Ltex, Btex)
-            GL.glVertex2f(left, bottom)
-            # left top
-            GL.glMultiTexCoord2fARB(GL.GL_TEXTURE0_ARB, Ltex, Ttex)
-            GL.glVertex2f(left, top)
-            # right top
-            GL.glMultiTexCoord2fARB(GL.GL_TEXTURE0_ARB, Rtex, Ttex)
-            GL.glVertex2f(right, top)
-            GL.glEnd()
-
-        GL.glDisable(GL.GL_TEXTURE_2D)
-        GL.glEndList()
-        self._needUpdate = False
-
     @attributeSetter
     def flipHoriz(self, value):
         """If set to True then the text will be flipped left-to-right.  The
@@ -775,13 +666,13 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         # because this is a property getter we can check /on-access/ if it
         # needs updating :-)
         if self._needVertexUpdate:
-            self.__dict__['posPix'] = convertToPix(vertices=[0, 0],
-                                                   pos=self.pos,
-                                                   units=self.units,
-                                                   win=self.win)
+            self.__dict__['posPix'] = self._pos.pix
         self._needVertexUpdate = False
         return self.__dict__['posPix']
 
+    def updateOpacity(self):
+        self._setTextShaders(value=self.text)
+
     def draw(self, win=None):
         """
         Draw the stimulus in its relevant window. You must call
@@ -810,24 +701,17 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
         GL.glScalef((1, -1)[self.flipHoriz], (1, -1)
                     [self.flipVert], 1)  # x,y,z; -1=flipped
 
-        if self.useShaders:  # then rgb needs to be set as glColor
-            # setup color
-            desiredRGB = self._getDesiredRGB(
-                self.rgb, self.colorSpace, self.contrast)
-            GL.glColor4f(desiredRGB[0], desiredRGB[1],
-                         desiredRGB[2], self.opacity)
+        # setup color
+        GL.glColor4f(*self._foreColor.render('rgba1'))
 
-            GL.glUseProgram(self.win._progSignedTexFont)
-            # GL.glUniform3iv(GL.glGetUniformLocation(
-            #       self.win._progSignedTexFont, "rgb"), 1,
-            #       desiredRGB.ctypes.data_as(ctypes.POINTER(ctypes.c_float)))
-            #  # set the texture to be texture unit 0
-            GL.glUniform3f(
-                GL.glGetUniformLocation(self.win._progSignedTexFont, b"rgb"),
-                desiredRGB[0], desiredRGB[1], desiredRGB[2])
-
-        else:  # color is set in texture, so set glColor to white
-            GL.glColor4f(1, 1, 1, 1)
+        GL.glUseProgram(self.win._progSignedTexFont)
+        # GL.glUniform3iv(GL.glGetUniformLocation(
+        #       self.win._progSignedTexFont, "rgb"), 1,
+        #       desiredRGB.ctypes.data_as(ctypes.POINTER(ctypes.c_float)))
+        #  # set the texture to be texture unit 0
+        GL.glUniform3f(
+            GL.glGetUniformLocation(self.win._progSignedTexFont, b"rgb"),
+            *self._foreColor.render('rgb1'))
 
         # should text have a depth or just on top?
         GL.glDisable(GL.GL_DEPTH_TEST)
@@ -855,10 +739,6 @@ class TextStim(BaseVisualStim, ColorMixin, ContainerMixin):
 
         # pyglets text.draw() method alters the blend func so reassert ours
         win.setBlendMode(blendMode, log=False)
-
-        if self.useShaders:
-            # disable shader (but command isn't available pre-OpenGL2.0)
-            GL.glUseProgram(0)
-
+        GL.glUseProgram(0)
         # GL.glEnable(GL.GL_DEPTH_TEST)  # Enables Depth Testing
         GL.glPopMatrix()
