@@ -14,24 +14,20 @@
 # Distributed under the Boost Software License, Version 1.0
 # (see http://www.boost.org/LICENSE_1_0.txt)
 #
-
+import re
 import sys, os
 import math
 import numpy as np
 import ctypes
 import freetype as ft
 from pyglet import gl  # import OpenGL.GL not compatible with Big Sur (2020)
-import glob
 from pathlib import Path
+import requests
 
 from psychopy import logging
 from psychopy import prefs
-from psychopy.constants import PY3
 from psychopy.exceptions import MissingFontError
 
-if PY3:
-    unichr = chr
-
 #  OS Font paths
 _X11FontDirectories = [
     # an old standard installation point
@@ -50,11 +46,24 @@ _OSXFontDirectories = [
     "/Network/Library/Fonts",
     "/System/Library/Fonts",
     # fonts installed via MacPorts
-    "/opt/local/share/fonts"
-    ""
+    "/opt/local/share/fonts",
 ]
 
-supportedExtensions = ['ttf', 'otf', 'ttc', 'dfont']
+_weightMap = {
+    # Map of various potential values for "bold" and the numeric font weight which they correspond to
+    100: 100, "thin": 100, "hairline": 100,
+    200: 200, "extralight": 200, "ultralight": 200,
+    300: 300, "light": 300,
+    400: 400, False: 400, "normal": 400, "regular": 400,
+    500: 500, "medium": 500,
+    600: 600, "semibold": 600, "demibold": 600,
+    700: 700, "bold": 700, True: 700,
+    800: 800, "extrabold": 800, "ultrabold": 800,
+    900: 900, "black": 900, "heavy": 900,
+    950: 950, "extrablack": 950, "ultrablack": 950
+}
+
+supportedExtensions = ['ttf', 'otf', 'ttc', 'dfont', 'truetype']
 
 
 def unicode(s, fmt='utf-8'):
@@ -312,9 +321,30 @@ class _TextureAtlas:
 class GLFont:
     """
     A GLFont gathers a set of glyphs for a given font filename and size.
+
+        size : int
+            Distance between the tops of capital letters and the bottoms of descenders
+
+        height : int
+            Total distance from one baseline to the next
+
+        capheight : int
+            Position of the tops of capital letters relative to the baseline
+
+        ascender : int
+            Position of the tops of ascenders relative to the baseline
+
+        descender : int
+            Position of the bottoms of descenders relative to the baseline
+
+        linegap : int
+            Distance between the bottoms of this line's descenders and the tops of the next line's ascenders
+
+        leading : int
+            Position of the tops of the next line's ascenders relative to this line's baseline
     """
 
-    def __init__(self, filename, size, textureSize=2048):
+    def __init__(self, filename, size, lineSpacing=1, textureSize=2048):
         """
         Initialize font
 
@@ -329,22 +359,26 @@ class GLFont:
 
         size : float
             Font size
+
+        lineSpacing : float
+            Leading between lines, proportional to font size
         """
         self.scale = 64.0
         self.atlas = _TextureAtlas(textureSize, textureSize, format='alpha')
+        self.format = self.atlas.format
         self.filename = filename
+        self.face = ft.Face(str(filename))  # ft.Face doesn't support Pathlib yet
         self.size = size
         self.glyphs = {}
-        face = ft.Face(str(filename))  # ft.Face doesn't support Pathlib yet
-        face.set_char_size(int(self.size * self.scale))
-        self.info = FontInfo(filename, face)
+        self.info = FontInfo(filename, self.face)
         self._dirty = False
-        metrics = face.size
+        # Get metrics
+        metrics = self.face.size
         self.ascender = metrics.ascender / self.scale
         self.descender = metrics.descender / self.scale
         self.height = metrics.height / self.scale
-        self.linegap = self.height - self.ascender + self.descender
-        self.format = self.atlas.format
+        # Set spacing
+        self.lineSpacing = lineSpacing
 
     def __getitem__(self, charcode):
         """
@@ -358,6 +392,57 @@ class GLFont:
         """Returns a string rep of the font, such as 'Arial_24_bold' """
         return "{}_{}".format(self.info, self.size)
 
+    @property
+    def leading(self):
+        """
+        Position of the next row's ascender line relative to this row's base line.
+        """
+        return self.ascender - self.height
+
+    @leading.setter
+    def leading(self, value):
+        self.height = self.ascender - value
+
+    @property
+    def linegap(self):
+        return -(self.leading - self.descender)
+
+    @linegap.setter
+    def linegap(self, value):
+        self.leading = self.descender - value
+
+    @property
+    def capheight(self):
+        """
+        Position of the top of capital letters relative to the base line.
+        """
+        return self.descender + self.size
+
+    @capheight.setter
+    def capheight(self, value):
+        self.size = value - self.descender
+
+    @property
+    def size(self):
+        """
+        Distance from the descender line to the capheight line.
+        """
+        if hasattr(self, "_size"):
+            return self._size
+
+    @size.setter
+    def size(self, value):
+        self._size = value
+        self.face.set_char_size(int(self.size * self.scale))
+
+    @property
+    def lineSpacing(self):
+        return self.height / (self.ascender - self.descender)
+
+    @lineSpacing.setter
+    def lineSpacing(self, value):
+        self.height = value * (self.ascender - self.descender)
+
     @property
     def name(self):
         """Name of the Font (e.g. 'Arial_24_bold')
@@ -388,7 +473,7 @@ class GLFont:
         face = ft.Face(str(self.filename))  # ft.Face doesn't support Pathlib
 
         chrs = (list(face.get_chars()))[:nMax]
-        charcodes = [unichr(c[1]) for c in chrs]
+        charcodes = [chr(c[1]) for c in chrs]
         self.fetch(charcodes, face=face)
         logging.debug("Preloading of glyph set for Texture Font {} complete"
                       .format(self.name))
@@ -567,13 +652,15 @@ def findFontFiles(folders=(), recursive=True):
     -------
     list of pathlib.Path objects
     """
-    if sys.platform == 'win32':
-        searchPaths = []  # just leave it to matplotlib as below
-    elif sys.platform == 'darwin':
-        # on mac matplotlib doesn't include 'ttc' files (which are fine)
-        searchPaths = _OSXFontDirectories
-    elif sys.platform.startswith('linux'):
-        searchPaths = _X11FontDirectories
+    searchPaths = folders
+    if searchPaths is None or len(searchPaths)==0:
+        if sys.platform == 'win32':
+            searchPaths = []  # just leave it to matplotlib as below
+        elif sys.platform == 'darwin':
+            # on mac matplotlib doesn't include 'ttc' files (which are fine)
+            searchPaths = _OSXFontDirectories
+        elif sys.platform.startswith('linux'):
+            searchPaths = _X11FontDirectories
     # search those folders
     fontPaths = []
     for thisFolder in searchPaths:
@@ -585,14 +672,21 @@ def findFontFiles(folders=(), recursive=True):
                 fontPaths.extend(thisFolder.glob("*.{}".format(thisExt)))
 
     # if we failed let matplotlib have a go
-    if fontPaths:
-        return fontPaths
-    else:
+    if not fontPaths:
         from matplotlib import font_manager
-        return font_manager.findSystemFonts()
+        fontPaths = font_manager.findSystemFonts()
+
+    # search resources folder and user's own fonts folder
+    for thisFolder in [Path(prefs.paths['fonts']), Path(prefs.paths['resources']) / "fonts"]:
+        for thisExt in supportedExtensions:
+            if recursive:
+                fontPaths.extend(thisFolder.rglob("*.{}".format(thisExt)))
+            else:
+                fontPaths.extend(thisFolder.glob("*.{}".format(thisExt)))
+    return fontPaths
 
 
-class FontManager(object):
+class FontManager():
     """FontManager provides a simple API for finding and loading font files
     (.ttf) via the FreeType lib
 
@@ -687,13 +781,18 @@ class FontManager(object):
         """
         if type(fontName) != bytes:
             fontName = bytes(fontName, sys.getfilesystemencoding())
+        # Convert value of "bold" to a numeric font weight
+        if bold in _weightMap or str(bold).lower().strip() in _weightMap:
+            bold = _weightMap[bold]
+        else:
+            bold = _weightMap[False] # Default to regular
         style_dict = self._fontInfos.get(fontName)
         if not style_dict:
             if not fallback:
                 return None
             similar = self.getFontNamesSimilar(fontName)
             if len(similar) == 0:
-                logging.warning("Font {} was requested. No similar font found.")
+                logging.warning("Font {} was requested. No similar font found.".format(repr(fontName)))
                 return [self.getDefaultSansFont()]
             elif len(similar) == 1:
                 logging.warning("Font {} was requested. Exact match wasn't "
@@ -723,6 +822,30 @@ class FontManager(object):
                    (fontName.lower() in this.lower())]
         return similar
 
+    def addGoogleFont(self, fontName):
+        """Add a font directly from the Google Font repository, saving it to the user prefs folder"""
+
+        # Construct and send Google Font url from name
+        repoURL = f"https://fonts.googleapis.com/css2?family={ fontName.replace(' ', '+') }&display=swap"
+        repoResp = requests.get(repoURL)
+        if not repoResp.ok:
+            # If font name is not found, raise error
+            raise MissingFontError("Font `{}` could not be retrieved from the Google Font library.".format(fontName))
+        # Get and send file url from returned CSS data
+        fileURL = re.findall(r"(?<=src: url\().*(?=\) format)", repoResp.content.decode())[0]
+        fileFormat = re.findall(r"(?<=format\(\').*(?=\'\)\;)", repoResp.content.decode())[0]
+        fileResp = requests.get(fileURL)
+        if not fileResp.ok:
+            # If font file is not available, raise error
+            raise MissingFontError("OST file for Google font `{}` could not be accessed".format(fontName))
+        # Save retrieved font as an OST file
+        fileName = Path(prefs.paths['fonts']) / f"{fontName}.{fileFormat}"
+        logging.info("Font \"{}\" was successfully installed at: {}".format(fontName, prefs.paths['fonts']))
+        with open(fileName, "wb") as fileObj:
+            fileObj.write(fileResp.content)
+        # Add font and return
+        return self.addFontFile(fileName)
+
     def addFontFile(self, fontPath, monospaceOnly=False):
         """Add a Font File to the FontManger font search space. The
         fontPath must be a valid path including the font file name.
@@ -787,7 +910,7 @@ class FontManager(object):
         a monospace font (as only monospace fonts are currently supported by
         TextBox).
 
-        Adding fonts to the FontManager is not persistant across runs of
+        Adding fonts to the FontManager is not persistent across runs of
         the script, so any extra font paths need to be added each time the
         script starts.
         """
@@ -797,7 +920,7 @@ class FontManager(object):
     # Class methods for FontManager below this comment should not need to be
     # used by user scripts in most situations. Accessing them is okay.
 
-    def getFont(self, name, size=32, bold=False, italic=False,
+    def getFont(self, name, size=32, bold=False, italic=False, lineSpacing=1,
                 monospace=False):
         """
         Return a FontAtlas object that matches the family name, style info,
@@ -806,14 +929,23 @@ class FontManager(object):
         then the existing FontAtlas is returned. Otherwise, a new FontAtlas is
         created , added to the cache, and returned.
         """
-        fontInfos = self.getFontsMatching(name, bold, italic)
+        fontInfos = self.getFontsMatching(name, bold, italic, fallback=False)
         if not fontInfos:
-            return False
+            # If font not found, try to retrieve it from Google
+            try:
+                self.addGoogleFont(name)
+            except (MissingFontError, ValueError):
+                pass
+            # Then try again with fallback
+            fontInfos = self.getFontsMatching(name, bold, italic, fallback=True)
+            if not fontInfos:
+                return False
+        # If font is found, make glfont
         fontInfo = fontInfos[0]
         identifier = "{}_{}".format(str(fontInfo), size)
         glFont = self._glFonts.get(identifier)
         if glFont is None:
-            glFont = GLFont(fontInfo.path, size)
+            glFont = GLFont(fontInfo.path, size, lineSpacing=lineSpacing)
             self._glFonts[identifier] = glFont
 
         return glFont
@@ -835,12 +967,15 @@ class FontManager(object):
         s = style.lower().strip()
         if type(s) == bytes:
             s = s.decode('utf-8')
-        if s == 'regular':
-            return False, False
+        # Work out Italic
+        italic = False # Default false
         if s.find('italic') >= 0 or s.find('oblique') >= 0:
             italic = True
-        if s.find('bold') >= 0:
-            bold = True
+        # Work out font weight
+        bold = _weightMap[False] # Default regular weight
+        for key in _weightMap:
+            if s.find(str(key)) >= 0:
+                bold = _weightMap[key]
         return bold, italic
 
     def _createFontInfo(self, fp, fface):
@@ -869,7 +1004,7 @@ class FontManager(object):
             self._fontInfos = None
 
 
-class FontInfo(object):
+class FontInfo():
 
     def __init__(self, fp, face):
         self.path = fp
